import json
SCHEMAS = json.loads(json.dumps([{'$id': 'model', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}, {'$id': 'workflow', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'workflow schema', 'type': 'object', 'allOf': [{'$id': 'workflow/base', 'title': 'base workflow schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'properties': {'description': 'Array of characteristic properties calculated by this workflow (TODO: add enums)', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'isUsingDataset': {'description': 'Whether to use the dataset tab in the job designer. Mutually exclusive with using the materials tab.', 'type': 'boolean'}, 'workflows': {'description': 'Array of workflows with the same schema as the current one.', 'type': 'array', 'items': {'type': 'object'}}}}], 'properties': {'subworkflows': {'description': 'Array of subworkflows. Subworkflow can be an instance of workflow to allow for nesting', 'type': 'array', 'items': {'allOf': [{'$id': 'workflow/subworkflow', 'title': 'Subworkflow', 'type': 'object', 'allOf': [{'$id': 'workflow/base-flow', 'title': 'BaseFlow', 'type': 'object', 'properties': {'_id': {'description': 'subworkflow identity', 'type': 'string'}, 'name': {'description': 'Human-readable name of the subworkflow. e.g. Total-energy', 'type': 'string'}, 'properties': {'description': 'Array of characteristic properties calculated by this subworkflow', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}}, 'required': ['name', 'units']}], 'properties': {'units': {'description': 'Contains the Units of the subworkflow', 'type': 'array', 'items': {'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}}, 'model': {'description': 'Model used inside the subworkflow', '$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}, 'application': {'description': 'information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'isDraft': {'description': 'Defines whether to store the results/properties extracted in this unit to properties collection', 'type': 'boolean', 'default': False}}, 'required': ['model', 'application']}]}}, 'units': {'description': 'Contains the Units of the Workflow', 'type': 'array', 'items': {'oneOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/io', 'title': 'data IO unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['io']}, 'subtype': {'enum': ['input', 'output', 'dataFrame']}, 'source': {'enum': ['api', 'db', 'object_storage']}, 'input': {'type': 'array', 'items': {'anyOf': [{'$id': 'workflow/unit/io/api', 'title': 'data IO rest API input schema', 'type': 'object', 'properties': {'endpoint': {'description': 'rest API endpoint', 'type': 'string'}, 'endpoint_options': {'description': 'rest API endpoint options', 'type': 'object'}, 'name': {'description': 'the name of the variable in local scope to save the data under', 'type': 'string'}}, 'required': ['endpoint', 'endpoint_options']}, {'$id': 'workflow/unit/io/db', 'title': 'data IO database input/output schema', 'type': 'object', 'oneOf': [{'properties': {'ids': {'description': 'IDs of item to retrieve from db', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['ids']}, {'properties': {'collection': {'description': 'db collection name', 'type': 'string'}, 'draft': {'description': 'whether the result should be saved as draft', 'type': 'boolean', 'default': True}}, 'required': ['collection', 'draft']}]}, {'$id': 'workflow/unit/io/object-storage', 'title': 'object_storage io schema', 'type': 'object', 'allof': [{'$id': 'core/reusable/file-metadata', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}], 'properties': {'objectData': {'$id': 'core/reusable/object-storage-container-data', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}, 'overwrite': {'description': 'if a file with the same filename already exists, whether to overwrite the old file', 'type': 'boolean', 'default': False}}, 'required': ['objectData', 'basename']}]}}}, 'required': ['subtype', 'source', 'input']}, {'$id': 'workflow/unit/map', 'title': 'map unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['map']}, 'workflowId': {'description': 'Id of workflow to run inside map', 'type': 'string'}, 'input': {'description': 'Input information for map.', 'type': 'object', 'properties': {'target': {'description': 'Name of the target variable to substitute using the values below. e.g. K_POINTS', 'type': 'string'}, 'scope': {'description': 'Scope to retrieve `values` from, global or flowchartId. Optional if `values` is given.', 'type': 'string'}, 'name': {'description': 'Name of the variable inside the scope to retrieve `values` from. Optional if `values` is given.', 'type': 'string'}, 'values': {'description': 'Sequence of values for the target Jinja variable. Optional if `scope` and `name` are given. This can be used for map-reduce type parallel execution', 'type': 'array', 'items': {'oneOf': [{'type': 'string'}, {'type': 'number'}, {'type': 'object'}]}}, 'useValues': {'type': 'boolean'}}, 'required': ['target']}}, 'required': ['input', 'workflowId']}, {'$id': 'workflow/unit/reduce', 'title': 'reduce unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['reduce']}, 'mapFlowchartId': {'description': 'corresponding map unit flowchart ID', 'type': 'string'}, 'input': {'description': 'input information for reduce unit', 'type': 'array', 'items': {'type': 'object', 'properties': {'operation': {'description': 'reduce operation, e.g. aggregate', 'type': 'string'}, 'arguments': {'description': 'arguments which are passed to reduce operation function', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['operation', 'arguments']}}}, 'required': ['mapFlowchartId', 'input']}, {'$id': 'workflow/unit/condition', 'title': 'condition unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['condition']}, 'input': {'description': 'Input information for condition.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'statement': {'description': "Condition statement. e.g. 'abs(x-total_energy) < 1e-5'", 'type': 'string'}, 'then': {'description': 'Flowchart ID reference for `then` part of the condition.', 'type': 'string'}, 'else': {'description': 'Flowchart ID reference for `else` part of the condition.', 'type': 'string'}, 'maxOccurrences': {'description': 'Maximum occurrence of the condition, usable for loops.', 'type': 'integer'}, 'throwException': {'description': 'Throw exception on reaching to maximum occurence.', 'type': 'boolean'}}, 'required': ['input', 'statement', 'then', 'else', 'maxOccurrences']}, {'$id': 'workflow/unit/assertion', 'title': 'assertion unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assertion']}, 'statement': {'type': 'string', 'description': 'The statement to be evaluated'}, 'errorMessage': {'type': 'string', 'description': 'The error message to be displayed if the assertion fails'}}, 'required': ['name', 'statement']}, {'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}, {'$id': 'workflow/unit/assignment', 'title': 'assignment unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assignment']}, 'input': {'description': 'Input information for assignment. if omitted, means that it is an initialization unit, otherwise it is an assignment.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'operand': {'description': "Name of the global variable. e.g. 'x'", 'type': 'string'}, 'value': {'description': "Value of the variable. The value content could be a simple integer, string or a python expression. e.g. '0' (initialization), 'sin(x)+1' (expression)", 'type': 'string'}}, 'required': ['name', 'operand', 'value']}, {'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}]}}}, 'required': ['units', 'subworkflows']}, {'$id': 'project', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'project schema', 'type': 'object', 'allOf': [{'description': 'in-memory entity', '$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'gid': {'description': 'project GID', 'type': 'number'}, 'clusterBasedChargeRates': {'description': 'charge rates info for project', 'type': 'array', 'items': {'type': 'object', 'properties': {'rate': {'type': 'number'}, 'timestamp': {'type': 'number'}, 'hostname': {'type': 'string'}}}}, 'isExternal': {'type': 'boolean', 'default': False}}}, {'$id': 'job', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'job schema', 'allOf': [{'$id': 'job/base', 'title': 'base job schema', 'allOf': [{'description': 'in-memory entity', '$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'rmsId': {'description': 'Identity used to track jobs originated from command-line', 'type': 'string'}, 'status': {'description': 'job status', 'enum': ['pre-submission', 'queued', 'submitted', 'active', 'finished', 'terminate-queued', 'terminated', 'error', 'deleted', 'timeout']}, 'startTime': {'description': 'Approximate start time of the job. e.g. within 10 min', 'type': 'string'}, 'workDir': {'description': 'The path to the working directory of this job, when the job originates from command-line', 'type': 'string'}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}, '_project': {'description': 'Subset of the full information about the project that this job belongs to.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, '_material': {'description': 'Subset of the full information about the materials used inside this job.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, 'parent': {'description': 'Subset of the full information about the parent job for the job. Only a job from the same project can be assigned as a parent one.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, 'runtimeContext': {'description': 'Context variables that the job will have access to at runtime', 'type': 'object'}, 'scopeTrack': {'description': 'history of the workflow scope on each update', 'type': 'array', 'items': {'type': 'object', 'properties': {'repetition': {'type': 'number'}, 'scope': {'$id': 'workflow/scope', 'title': 'workflow scope schema', 'type': 'object', 'properties': {'global': {'type': 'object', 'additionalProperties': True}, 'local': {'type': 'object', 'additionalProperties': True}}, 'required': ['global', 'local']}}}}}, 'required': ['status', 'compute', '_project']}], 'properties': {'workflow': {'description': 'Arch-workflow for the job', '$id': 'workflow', 'title': 'workflow schema', 'type': 'object', 'allOf': [{'$id': 'workflow/base', 'title': 'base workflow schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'properties': {'description': 'Array of characteristic properties calculated by this workflow (TODO: add enums)', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'isUsingDataset': {'description': 'Whether to use the dataset tab in the job designer. Mutually exclusive with using the materials tab.', 'type': 'boolean'}, 'workflows': {'description': 'Array of workflows with the same schema as the current one.', 'type': 'array', 'items': {'type': 'object'}}}}], 'properties': {'subworkflows': {'description': 'Array of subworkflows. Subworkflow can be an instance of workflow to allow for nesting', 'type': 'array', 'items': {'allOf': [{'$id': 'workflow/subworkflow', 'title': 'Subworkflow', 'type': 'object', 'allOf': [{'$id': 'workflow/base-flow', 'title': 'BaseFlow', 'type': 'object', 'properties': {'_id': {'description': 'subworkflow identity', 'type': 'string'}, 'name': {'description': 'Human-readable name of the subworkflow. e.g. Total-energy', 'type': 'string'}, 'properties': {'description': 'Array of characteristic properties calculated by this subworkflow', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}}, 'required': ['name', 'units']}], 'properties': {'units': {'description': 'Contains the Units of the subworkflow', 'type': 'array', 'items': {'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}}, 'model': {'description': 'Model used inside the subworkflow', '$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}, 'application': {'description': 'information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'isDraft': {'description': 'Defines whether to store the results/properties extracted in this unit to properties collection', 'type': 'boolean', 'default': False}}, 'required': ['model', 'application']}]}}, 'units': {'description': 'Contains the Units of the Workflow', 'type': 'array', 'items': {'oneOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/io', 'title': 'data IO unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['io']}, 'subtype': {'enum': ['input', 'output', 'dataFrame']}, 'source': {'enum': ['api', 'db', 'object_storage']}, 'input': {'type': 'array', 'items': {'anyOf': [{'$id': 'workflow/unit/io/api', 'title': 'data IO rest API input schema', 'type': 'object', 'properties': {'endpoint': {'description': 'rest API endpoint', 'type': 'string'}, 'endpoint_options': {'description': 'rest API endpoint options', 'type': 'object'}, 'name': {'description': 'the name of the variable in local scope to save the data under', 'type': 'string'}}, 'required': ['endpoint', 'endpoint_options']}, {'$id': 'workflow/unit/io/db', 'title': 'data IO database input/output schema', 'type': 'object', 'oneOf': [{'properties': {'ids': {'description': 'IDs of item to retrieve from db', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['ids']}, {'properties': {'collection': {'description': 'db collection name', 'type': 'string'}, 'draft': {'description': 'whether the result should be saved as draft', 'type': 'boolean', 'default': True}}, 'required': ['collection', 'draft']}]}, {'$id': 'workflow/unit/io/object-storage', 'title': 'object_storage io schema', 'type': 'object', 'allof': [{'$id': 'core/reusable/file-metadata', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}], 'properties': {'objectData': {'$id': 'core/reusable/object-storage-container-data', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}, 'overwrite': {'description': 'if a file with the same filename already exists, whether to overwrite the old file', 'type': 'boolean', 'default': False}}, 'required': ['objectData', 'basename']}]}}}, 'required': ['subtype', 'source', 'input']}, {'$id': 'workflow/unit/map', 'title': 'map unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['map']}, 'workflowId': {'description': 'Id of workflow to run inside map', 'type': 'string'}, 'input': {'description': 'Input information for map.', 'type': 'object', 'properties': {'target': {'description': 'Name of the target variable to substitute using the values below. e.g. K_POINTS', 'type': 'string'}, 'scope': {'description': 'Scope to retrieve `values` from, global or flowchartId. Optional if `values` is given.', 'type': 'string'}, 'name': {'description': 'Name of the variable inside the scope to retrieve `values` from. Optional if `values` is given.', 'type': 'string'}, 'values': {'description': 'Sequence of values for the target Jinja variable. Optional if `scope` and `name` are given. This can be used for map-reduce type parallel execution', 'type': 'array', 'items': {'oneOf': [{'type': 'string'}, {'type': 'number'}, {'type': 'object'}]}}, 'useValues': {'type': 'boolean'}}, 'required': ['target']}}, 'required': ['input', 'workflowId']}, {'$id': 'workflow/unit/reduce', 'title': 'reduce unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['reduce']}, 'mapFlowchartId': {'description': 'corresponding map unit flowchart ID', 'type': 'string'}, 'input': {'description': 'input information for reduce unit', 'type': 'array', 'items': {'type': 'object', 'properties': {'operation': {'description': 'reduce operation, e.g. aggregate', 'type': 'string'}, 'arguments': {'description': 'arguments which are passed to reduce operation function', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['operation', 'arguments']}}}, 'required': ['mapFlowchartId', 'input']}, {'$id': 'workflow/unit/condition', 'title': 'condition unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['condition']}, 'input': {'description': 'Input information for condition.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'statement': {'description': "Condition statement. e.g. 'abs(x-total_energy) < 1e-5'", 'type': 'string'}, 'then': {'description': 'Flowchart ID reference for `then` part of the condition.', 'type': 'string'}, 'else': {'description': 'Flowchart ID reference for `else` part of the condition.', 'type': 'string'}, 'maxOccurrences': {'description': 'Maximum occurrence of the condition, usable for loops.', 'type': 'integer'}, 'throwException': {'description': 'Throw exception on reaching to maximum occurence.', 'type': 'boolean'}}, 'required': ['input', 'statement', 'then', 'else', 'maxOccurrences']}, {'$id': 'workflow/unit/assertion', 'title': 'assertion unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assertion']}, 'statement': {'type': 'string', 'description': 'The statement to be evaluated'}, 'errorMessage': {'type': 'string', 'description': 'The error message to be displayed if the assertion fails'}}, 'required': ['name', 'statement']}, {'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}, {'$id': 'workflow/unit/assignment', 'title': 'assignment unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assignment']}, 'input': {'description': 'Input information for assignment. if omitted, means that it is an initialization unit, otherwise it is an assignment.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'operand': {'description': "Name of the global variable. e.g. 'x'", 'type': 'string'}, 'value': {'description': "Value of the variable. The value content could be a simple integer, string or a python expression. e.g. '0' (initialization), 'sin(x)+1' (expression)", 'type': 'string'}}, 'required': ['name', 'operand', 'value']}, {'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}]}}}, 'required': ['units', 'subworkflows']}}, 'required': ['workflow']}, {'$id': 'material', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'material schema', 'allOf': [{'description': 'in-memory entity', '$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'formula': {'description': 'reduced chemical formula', 'type': 'string'}, 'unitCellFormula': {'description': 'chemical formula based on the number of atoms of each element in the supercell', 'type': 'string'}, 'basis': {'description': 'object containing an array of elements and coordinates in the supercell', '$id': 'properties-directory/structural/basis', 'title': 'basis schema', 'type': 'object', 'properties': {'elements': {'type': 'array', 'items': {'$id': 'properties-directory/structural/basis/atomic-element', 'title': 'atomic elements', 'description': 'elements of atoms by ids, string, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'type': 'string'}, 'occurrence': {'description': 'Occurrence is for fractional occupations', 'type': 'number'}, 'oxidationState': {'type': 'number'}}, 'required': ['id', 'value']}}, 'coordinates': {'type': 'array', 'items': {'$id': 'properties-directory/structural/basis/atomic-coordinate', 'title': 'atomic coordinate', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, 'name': {'type': 'string'}, 'units': {'type': 'string'}, 'bonds': {'$id': 'properties-directory/structural/basis/bonds', 'title': 'bonds schema', 'type': 'array', 'items': {'type': 'object', 'properties': {'atomPair': {'description': 'indices of the two connected atoms', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'minItems': 2, 'maxItems': 2}, 'bondType': {'type': 'string', 'enum': ['single', 'double', 'triple', 'quadruple', 'aromatic', 'tautomeric', 'dative', 'other']}}}, 'uniqueItems': True}}, 'required': ['elements', 'coordinates']}, 'lattice': {'description': 'object defining the lattice cell size and shape', '$id': 'properties-directory/structural/lattice', 'title': 'lattice schema', 'type': 'object', 'allOf': [{'properties': {'vectors': {'$id': 'properties-directory/structural/lattice/lattice-vectors', 'title': 'lattice explicit unit', 'allOf': [{'$id': 'core/abstract/3d-vector-basis', 'title': '3 dimensional vector basis', 'type': 'object', 'properties': {'a': {'description': 'first vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'b': {'description': 'second vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'c': {'description': 'third vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}, 'required': ['a', 'b', 'c']}], 'properties': {'alat': {'description': 'lattice parameter for fractional coordinates', 'type': 'number', 'default': 1}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}}}, {'$id': 'properties-directory/structural/lattice/lattice-bravais', 'title': 'lattice implicit schema', 'allOf': [{'$id': 'core/primitive/3d-lattice', 'title': '3 dimensional lattice schema', 'type': 'object', 'properties': {'a': {'description': 'length of the first lattice vector', 'type': 'number'}, 'b': {'description': 'length of the second lattice vector', 'type': 'number'}, 'c': {'description': 'length of the third lattice vector', 'type': 'number'}, 'alpha': {'description': 'angle between first and second lattice vector', 'type': 'number'}, 'beta': {'description': 'angle between second and third lattice vector', 'type': 'number'}, 'gamma': {'description': 'angle between first and third lattice vector', 'type': 'number'}}, 'required': ['a', 'b', 'c', 'alpha', 'beta', 'gamma']}], 'properties': {'type': {'description': 'Bravais lattice type in short notation', '$id': 'properties-directory/structural/lattice/type-enum', 'title': 'lattice type schema', 'type': 'string', 'enum': ['CUB', 'BCC', 'FCC', 'TET', 'MCL', 'ORC', 'ORCC', 'ORCF', 'ORCI', 'HEX', 'BCT', 'TRI', 'MCLC', 'RHL']}, 'units': {'type': 'object', 'properties': {'length': {'type': 'string', 'enum': ['angstrom', 'bohr']}, 'angle': {'type': 'string', 'enum': ['degree', 'radian']}}}}, 'required': ['type']}], 'properties': {'name': {'enum': ['lattice']}}}, 'derivedProperties': {'description': 'object defining the lattice cell size and shape', '$id': 'properties-directory/derived-properties', 'title': 'derived properties schema', 'type': 'array', 'items': {'anyOf': [{'$id': 'properties-directory/structural/volume', 'title': 'volume schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['volume']}, 'units': {'enum': ['angstrom^3']}}}, {'$id': 'properties-directory/structural/density', 'title': 'density schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['density']}, 'units': {'enum': ['g/cm^3']}}}, {'$id': 'properties-directory/structural/symmetry', 'title': 'symmetry schema', 'properties': {'pointGroupSymbol': {'description': 'point group symbol in Schoenflies notation', 'type': 'string'}, 'spaceGroupSymbol': {'description': 'space group symbol in Hermann–Mauguin notation', 'type': 'string'}, 'tolerance': {'description': 'tolerance used for symmetry calculation', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'units': {'enum': ['angstrom']}}}, 'name': {'enum': ['symmetry']}}}, {'$id': 'properties-directory/structural/elemental-ratio', 'title': 'elemental-ratio', 'description': 'ration of this element in the compound', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['elemental_ratio']}, 'value': {'type': 'number', 'min': 0, 'max': 1}, 'element': {'type': 'string', 'description': 'the element this ratio is for'}}}, {'$id': 'properties-directory/structural/p-norm', 'title': 'p_norm', 'description': 'https://en.wikipedia.org/wiki/Norm_(mathematics)#p-norm', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['p-norm']}, 'degree': {'type': 'integer', 'description': 'degree of the dimensionality of the norm'}}}, {'$id': 'properties-directory/structural/inchi', 'title': 'InChI representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi']}}}, {'$id': 'properties-directory/structural/inchi-key', 'title': 'InChI key representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi_key']}}}]}}, 'external': {'description': 'information about a database source', '$id': 'system/database-source', 'title': 'database source schema', 'type': 'object', 'properties': {'id': {'description': 'ID string for the materials uploaded from a third party source inside the third party source. For materialsproject.org an example ID is mp-32', 'type': 'string'}, 'source': {'description': 'Third party source name, e.g. materials project, 2dmatpedia, ICSD, etc.', 'type': 'string'}, 'origin': {'description': 'A flag that is true when material is initially imported from a third party * (as opposed to being independently designed from scratch).', 'type': 'boolean', 'comment': 'Deprecated. To be removed.'}, 'data': {'description': 'Original response from external source.', 'type': 'object'}, 'doi': {'description': 'Digital Object Identifier, e.g. 10.1088/0953-8984/25/10/105506', 'type': 'string'}, 'url': {'description': 'The URL of the original record, e.g. https://next-gen.materialsproject.org/materials/mp-48', 'type': 'string', 'comment': 'ToDo: update to use URI type per https://json-schema.org/understanding-json-schema/reference/string#resource-identifiers'}}, 'required': ['id', 'source', 'origin']}, 'src': {'description': 'file source with the information inside', '$id': 'system/file-source', 'title': 'file source schema', 'type': 'object', 'properties': {'extension': {'description': 'file extension', 'type': 'string'}, 'filename': {'description': 'file name without extension', 'type': 'string'}, 'text': {'description': 'file content as raw text', 'type': 'string'}, 'hash': {'description': 'MD5 hash based on file content', 'type': 'string'}}, 'required': ['filename', 'text', 'hash']}, 'scaledHash': {'description': 'Hash string for a scaled structure with lattice vector a set to 1 (eg. for materials under pressure).', 'type': 'string'}, 'icsdId': {'description': 'Corresponding ICSD id of the material', 'type': 'integer'}, 'isNonPeriodic': {'description': 'Whether to work in the finite molecular picture (usually with atomic orbital basis)', 'type': 'boolean'}}, 'required': ['basis', 'lattice']}, {'$id': 'method', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}, {'$id': 'element', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'element schema', 'properties': {'symbol': {'description': 'Element symbol.', 'type': 'string'}, 'properties': {'type': 'array', 'description': 'list of elemental properties', 'items': {'anyOf': [{'$id': 'properties-directory/elemental/atomic-radius', 'title': 'atomic radius', 'description': 'atomic radius', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['atomic_radius']}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, {'$id': 'properties-directory/elemental/electronegativity', 'title': 'electronegativity', 'description': 'electronegativity for the element (Pauling scale)', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['electronegativity']}}}, {'$id': 'properties-directory/elemental/ionization-potential', 'title': 'ionization potential', 'description': 'ionization potential for the element', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['ionization_potential']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}]}}}}, {'$id': 'core/reference', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'reference schema (using `anyOf` instead of `oneOf` below b/c current reference schemas overlap)', 'anyOf': [{'$id': 'core/reference/modeling', 'title': 'info for property obtained by modeling, only supports exabyte-originated data atm, but easily extendable', 'oneOf': [{'$id': 'core/reference/modeling/exabyte', 'title': 'info for characteristic obtained by exabyte calculation', 'properties': {'type': {'enum': ['exabyte']}, 'title': {'description': 'Human-readable title of the job', 'type': 'string', 'maxLength': 300}, '_id': {'description': 'job identifier', 'type': 'string'}, 'owner': {'description': 'owner entity reference', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}}, 'required': ['title', '_id', 'owner']}]}, {'$id': 'core/reference/experiment', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}, {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}]}, {'$id': 'core/abstract/2d-data', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}, {'$id': 'core/abstract/3d-tensor', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '3 dimensional tensor schema', 'type': 'array', 'items': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/abstract/3d-grid', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '3 dimensional grid schema', 'properties': {'dimensions': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'shifts': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}, 'required': ['dimensions', 'shifts']}, {'$id': 'core/abstract/3d-vector-basis', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '3 dimensional vector basis', 'type': 'object', 'properties': {'a': {'description': 'first vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'b': {'description': 'second vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'c': {'description': 'third vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}, 'required': ['a', 'b', 'c']}, {'$id': 'core/abstract/point', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, {'$id': 'core/abstract/2d-plot', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}, {'$id': 'core/abstract/vector', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}, {'$id': 'core/primitive/slugified-entry-or-slug', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'slugified entry or slug', 'description': 'contains either object with slugified entry or slug only as a string', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, {'$id': 'core/primitive/array-of-3-numbers', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/3d-lattice', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '3 dimensional lattice schema', 'type': 'object', 'properties': {'a': {'description': 'length of the first lattice vector', 'type': 'number'}, 'b': {'description': 'length of the second lattice vector', 'type': 'number'}, 'c': {'description': 'length of the third lattice vector', 'type': 'number'}, 'alpha': {'description': 'angle between first and second lattice vector', 'type': 'number'}, 'beta': {'description': 'angle between second and third lattice vector', 'type': 'number'}, 'gamma': {'description': 'angle between first and third lattice vector', 'type': 'number'}}, 'required': ['a', 'b', 'c', 'alpha', 'beta', 'gamma']}, {'$id': 'core/primitive/array-of-3-booleans', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/slugified-entry', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'$id': 'core/primitive/linked-list', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'linked list schema', 'type': 'array', 'items': {'type': 'object', 'anyOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'core/primitive/linked-list/named-node', 'title': 'Named node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}]}, {'$id': 'core/primitive/linked-list/named-node-in-group', 'title': 'Named node in group schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/named-node', 'title': 'Named node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}]}, {'$id': 'core/primitive/group-info', 'title': 'Group info schema for nodes in a graph', 'type': 'object', 'properties': {'groupName': {'description': 'Human-readable name of group of nodes', 'type': 'string'}, 'groupId': {'description': 'Unique identifier of the group a node belongs to', 'type': 'string'}}}]}, {'$id': 'core/primitive/linked-list/node-with-type', 'title': 'Typed node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}], 'properties': {'type': {'type': 'string'}}}], 'uniqueItems': True}}, {'$id': 'core/primitive/integer-one-or-zero', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}, {'$id': 'core/primitive/1d-data-series', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}, {'$id': 'core/primitive/string', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}, {'$id': 'core/primitive/axis', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, {'$id': 'core/primitive/group-info', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Group info schema for nodes in a graph', 'type': 'object', 'properties': {'groupName': {'description': 'Human-readable name of group of nodes', 'type': 'string'}, 'groupId': {'description': 'Unique identifier of the group a node belongs to', 'type': 'string'}}}, {'$id': 'core/primitive/array-of-strings', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'array of strings', 'description': 'array of strings, e.g. metadata tags', 'type': 'array', 'items': {'type': 'string', 'uniqueItems': True}}, {'$id': 'core/primitive/array-of-ids', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}, {'$id': 'core/primitive/scalar', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}, {'$id': 'core/primitive/linked-list/node-with-type', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Typed node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}], 'properties': {'type': {'type': 'string'}}}, {'$id': 'core/primitive/linked-list/named-node', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}]}, {'$id': 'core/primitive/linked-list/named-node-in-group', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named node in group schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/named-node', 'title': 'Named node schema', 'type': 'object', 'allOf': [{'$id': 'core/primitive/linked-list/base-node', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}]}, {'$id': 'core/primitive/group-info', 'title': 'Group info schema for nodes in a graph', 'type': 'object', 'properties': {'groupName': {'description': 'Human-readable name of group of nodes', 'type': 'string'}, 'groupId': {'description': 'Unique identifier of the group a node belongs to', 'type': 'string'}}}]}, {'$id': 'core/primitive/linked-list/base-node', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'basic node schema (linked list)', 'type': 'object', 'allOf': [{'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}], 'properties': {'next': {'description': 'Flowchart ID of next node', 'type': 'string'}, 'head': {'description': 'Whether node is head node or not', 'type': 'boolean'}}, 'definitions': {'flowchart-id': {'properties': {'flowchartId': {'description': 'Unique flowchart ID of node', 'type': 'string'}}, 'required': ['flowchartId']}}}, {'$id': 'core/reference/literature', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}, {'$id': 'core/reference/exabyte', '$schema': 'http://json-schema.org/draft-04/schema#', 'type': 'object', 'properties': {'materialId': {'description': "Material's identity. Used for protoProperties.", 'type': 'string'}, 'jobId': {'description': "Job's identity", 'type': 'string'}, 'unitId': {'description': 'Id of the unit that extracted the result', 'type': 'string'}}}, {'$id': 'core/reference/modeling', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'info for property obtained by modeling, only supports exabyte-originated data atm, but easily extendable', 'oneOf': [{'$id': 'core/reference/modeling/exabyte', 'title': 'info for characteristic obtained by exabyte calculation', 'properties': {'type': {'enum': ['exabyte']}, 'title': {'description': 'Human-readable title of the job', 'type': 'string', 'maxLength': 300}, '_id': {'description': 'job identifier', 'type': 'string'}, 'owner': {'description': 'owner entity reference', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}}, 'required': ['title', '_id', 'owner']}]}, {'$id': 'core/reference/experiment', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}, {'$id': 'core/reference/literature/name', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}, {'$id': 'core/reference/literature/pages', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}, {'$id': 'core/reference/modeling/exabyte', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'info for characteristic obtained by exabyte calculation', 'properties': {'type': {'enum': ['exabyte']}, 'title': {'description': 'Human-readable title of the job', 'type': 'string', 'maxLength': 300}, '_id': {'description': 'job identifier', 'type': 'string'}, 'owner': {'description': 'owner entity reference', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}}, 'required': ['title', '_id', 'owner']}, {'$id': 'core/reference/experiment/location', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, {'$id': 'core/reference/experiment/condition', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}, {'$id': 'core/reusable/categories', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, {'$id': 'core/reusable/atomic-scalars', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic scalars vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}}}}, {'$id': 'core/reusable/atomic-data-per-orbital-numeric', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data per orbital numeric', 'description': 'Atomic properties per orbital pair with numeric value e.g., Hubbard V parameters.', 'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data/per-orbital', 'title': 'Atomic data per orbital', 'description': 'Atomic properties per orbital e.g., Hubbard U parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}}}, {'$id': 'core/reusable/atomic-data/value-number', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}]}, {'$id': 'core/reusable/file-metadata', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}, {'$id': 'core/reusable/atomic-vectors', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, {'$id': 'core/reusable/object-storage-container-data', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}, {'$id': 'core/reusable/atomic-strings', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic strings vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'type': 'string'}}}}, {'$id': 'core/reusable/energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}, {'$id': 'core/reusable/frequency-function-matrix', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': '', 'description': 'Schema for a function of frequency yielding a nx3 matrix', 'type': 'object', 'properties': {'frequencies': {'description': 'Frequencies', 'type': 'array', 'items': {'type': 'number'}}, 'components': {'description': 'Matrix with 3 columns, e.g. x, y, z', 'type': 'array', 'items': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}}}, {'$id': 'core/reusable/dielectric-tensor-component', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Dielectric Tensor', 'allOf': [{'$id': 'core/reusable/frequency-function-matrix', 'title': '', 'description': 'Schema for a function of frequency yielding a nx3 matrix', 'type': 'object', 'properties': {'frequencies': {'description': 'Frequencies', 'type': 'array', 'items': {'type': 'number'}}, 'components': {'description': 'Matrix with 3 columns, e.g. x, y, z', 'type': 'array', 'items': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}}}], 'properties': {'part': {'description': 'Real or imaginary part of the dielectric tensor component', 'type': 'string', 'enum': ['real', 'imaginary']}, 'spin': {'type': 'number'}}, 'required': ['part', 'frequencies', 'components']}, {'$id': 'core/reusable/atomic-orbital', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic orbital schema', 'type': 'object', 'properties': {'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalIndex': {'type': 'integer', 'minimum': 1}, 'principalNumber': {'type': 'integer', 'minimum': 1, 'maximum': 7}, 'angularMomentum': {'type': 'integer', 'minimum': 0, 'maximum': 3}, 'occupation': {'type': 'number', 'description': 'Shell occupation', 'minimum': 0, 'maximum': 14}}}, {'$id': 'core/reusable/band-gap', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'band gap schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'kpointConduction': {'description': 'kpoint inside conduction band in crystal coordinates', '$id': 'core/abstract/point', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, 'kpointValence': {'description': 'kpoint inside valence band in crystal coordinates', '$id': 'core/abstract/point', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, 'eigenvalueConduction': {'description': 'eigenvalue at k-point in conduction band', 'type': 'number'}, 'eigenvalueValence': {'description': 'eigenvalue at k-point in valence band', 'type': 'number'}, 'spin': {'type': 'number'}, 'type': {'type': 'string', 'enum': ['direct', 'indirect']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}, 'required': ['type']}, {'$id': 'core/reusable/category-path', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}, {'$id': 'core/reusable/atomic-data-per-orbital-pair-numeric', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data per orbital pair numeric', 'description': 'Atomic properties per orbital pair with numeric value e.g., Hubbard V parameters.', 'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data/per-orbital-pair', 'title': 'Atomic data per orbital pair', 'description': 'Atomic properties per orbital pair e.g., Hubbard V parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'id2': {'type': 'integer', 'description': 'Site number or index in the lattice of second site'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'atomicSpecies2': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co2, O'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalName2': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'distance': {'type': 'number', 'description': 'Distance between two sites in Bohr.'}}}, {'$id': 'core/reusable/atomic-data/value-number', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}]}, {'$id': 'core/reusable/atomic-data/per-orbital', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data per orbital', 'description': 'Atomic properties per orbital e.g., Hubbard U parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}}}, {'$id': 'core/reusable/atomic-data/string-value', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data string properties', 'description': 'String value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'string', 'description': 'String value specific to atomic data'}}}, {'$id': 'core/reusable/atomic-data/per-orbital-pair', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data per orbital pair', 'description': 'Atomic properties per orbital pair e.g., Hubbard V parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'id2': {'type': 'integer', 'description': 'Site number or index in the lattice of second site'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'atomicSpecies2': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co2, O'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalName2': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'distance': {'type': 'number', 'description': 'Distance between two sites in Bohr.'}}}, {'$id': 'core/reusable/atomic-data/value-number', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}, {'$id': 'property/source', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'The source of a property. This could be an article, a simulation on Exabyte, an external simulation, etc.', 'type': 'object', 'properties': {'type': {'description': "Type of the material property's source.", 'type': 'string'}, 'url': {'description': 'Internet address of the reference.', 'type': 'string'}, 'info': {'oneOf': [{'$id': 'core/reference/exabyte', 'type': 'object', 'properties': {'materialId': {'description': "Material's identity. Used for protoProperties.", 'type': 'string'}, 'jobId': {'description': "Job's identity", 'type': 'string'}, 'unitId': {'description': 'Id of the unit that extracted the result', 'type': 'string'}}}, {'$id': 'core/reference/experiment', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}]}}}, {'$id': 'property/raw', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': "Schema of material's preliminary property", 'allOf': [{'description': 'entity schema', '$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}], 'properties': {'slug': {'description': 'property slug, e.g. total_energy', 'type': 'string'}, 'group': {'description': 'property group, e.g. qe:dft:gga:pbe', 'type': 'string'}, 'data': {'description': 'container of the information, specific to each property', 'type': 'object'}, 'source': {'$id': 'property/source', 'title': 'The source of a property. This could be an article, a simulation on Exabyte, an external simulation, etc.', 'type': 'object', 'properties': {'type': {'description': "Type of the material property's source.", 'type': 'string'}, 'url': {'description': 'Internet address of the reference.', 'type': 'string'}, 'info': {'oneOf': [{'$id': 'core/reference/exabyte', 'type': 'object', 'properties': {'materialId': {'description': "Material's identity. Used for protoProperties.", 'type': 'string'}, 'jobId': {'description': "Job's identity", 'type': 'string'}, 'unitId': {'description': 'Id of the unit that extracted the result', 'type': 'string'}}}, {'$id': 'core/reference/experiment', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}]}}}, 'exabyteId': {'description': 'Id of the corresponding item in the entity bank that this property is obtained for', 'type': 'array', 'items': {'type': 'string'}}, 'precision': {'type': 'object'}, 'count': {'description': 'total number of properties among which this property is the best.', 'type': 'number'}, 'systemTags': {'description': 'property system tags, marks property system characteristics, values refined or best (could be both)', 'type': 'array', 'items': {'type': 'string', 'enum': ['isRefined', 'isBest']}}}, 'required': ['data', 'source']}, {'$id': 'property/base', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': "Schema of base material's preliminary property", 'allOf': [{'description': 'entity schema', '$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}], 'properties': {'slug': {'description': 'property slug, e.g. total_energy', 'type': 'string'}, 'group': {'description': 'property group, e.g. qe:dft:gga:pbe', 'type': 'string'}, 'data': {'description': 'container of the information, specific to each property', 'type': 'object'}, 'source': {'$id': 'property/source', 'title': 'The source of a property. This could be an article, a simulation on Exabyte, an external simulation, etc.', 'type': 'object', 'properties': {'type': {'description': "Type of the material property's source.", 'type': 'string'}, 'url': {'description': 'Internet address of the reference.', 'type': 'string'}, 'info': {'oneOf': [{'$id': 'core/reference/exabyte', 'type': 'object', 'properties': {'materialId': {'description': "Material's identity. Used for protoProperties.", 'type': 'string'}, 'jobId': {'description': "Job's identity", 'type': 'string'}, 'unitId': {'description': 'Id of the unit that extracted the result', 'type': 'string'}}}, {'$id': 'core/reference/experiment', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}]}}}, 'exabyteId': {'description': 'Id of the corresponding item in the entity bank that this property is obtained for', 'type': 'array', 'items': {'type': 'string'}}, 'precision': {'type': 'object'}, 'count': {'description': 'total number of properties among which this property is the best.', 'type': 'number'}, 'systemTags': {'description': 'property system tags, marks property system characteristics, values refined or best (could be both)', 'type': 'array', 'items': {'type': 'string', 'enum': ['isRefined', 'isBest']}}}, 'required': ['data', 'source']}, {'$id': 'property/meta', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': "Schema of material's meta properties", 'allOf': [{'$id': 'property/raw', 'title': "Schema of material's preliminary property", 'allOf': [{'description': 'entity schema', '$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}], 'properties': {'slug': {'description': 'property slug, e.g. total_energy', 'type': 'string'}, 'group': {'description': 'property group, e.g. qe:dft:gga:pbe', 'type': 'string'}, 'data': {'description': 'container of the information, specific to each property', 'type': 'object'}, 'source': {'$id': 'property/source', 'title': 'The source of a property. This could be an article, a simulation on Exabyte, an external simulation, etc.', 'type': 'object', 'properties': {'type': {'description': "Type of the material property's source.", 'type': 'string'}, 'url': {'description': 'Internet address of the reference.', 'type': 'string'}, 'info': {'oneOf': [{'$id': 'core/reference/exabyte', 'type': 'object', 'properties': {'materialId': {'description': "Material's identity. Used for protoProperties.", 'type': 'string'}, 'jobId': {'description': "Job's identity", 'type': 'string'}, 'unitId': {'description': 'Id of the unit that extracted the result', 'type': 'string'}}}, {'$id': 'core/reference/experiment', 'title': 'info for characteristic obtained by experiment', 'properties': {'type': {'enum': ['experiment']}, 'authors': {'description': 'experiment authors', 'type': 'array', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'title': {'type': 'string', 'description': 'experiment title'}, 'method': {'type': 'string', 'description': 'method used in experiment'}, 'conditions': {'type': 'array', 'items': {'$id': 'core/reference/experiment/condition', 'title': 'condition schema', 'properties': {'units': {'description': 'condition unit', 'type': 'string'}, 'scalar': {'description': 'array of condition values', 'type': 'array', 'items': {'properties': {'value': {'type': 'string'}}}}, 'name': {'description': 'human-readable name of the condition', 'type': 'string'}}, 'required': ['name']}}, 'location': {'$id': 'core/reference/experiment/location', 'title': 'location schema', 'properties': {'latitude': {'description': 'location latitude', 'type': 'number'}, 'longitude': {'description': 'location longitude', 'type': 'number'}}, 'required': ['latitude', 'longitude']}, 'timestamp': {'description': 'epoch time.', 'type': 'number'}, 'note': {'description': 'Note about experiment', 'type': 'string'}, 'references': {'type': 'array', 'description': 'references to literature articles', 'items': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}}, 'required': ['conditions', 'authors', 'title', 'method', 'timestamp']}]}}}, 'exabyteId': {'description': 'Id of the corresponding item in the entity bank that this property is obtained for', 'type': 'array', 'items': {'type': 'string'}}, 'precision': {'type': 'object'}, 'count': {'description': 'total number of properties among which this property is the best.', 'type': 'number'}, 'systemTags': {'description': 'property system tags, marks property system characteristics, values refined or best (could be both)', 'type': 'array', 'items': {'type': 'string', 'enum': ['isRefined', 'isBest']}}}, 'required': ['data', 'source']}]}, {'$id': '3pse/file/applications/espresso/7.2/pw.x', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'pwx main schema', 'additionalProperties': False, 'properties': {'&CONTROL': {'$id': '3pse/file/applications/espresso/7.2/pw.x/control', 'title': 'control schema', 'type': 'object', 'additionalProperties': False, 'properties': {'calculation': {'description': 'A string describing the task to be performed', 'type': 'string', 'enum': ['scf', 'nscf', 'bands', 'relax', 'md', 'vc-relax', 'vc-md'], 'default': 'scf'}, 'title': {'description': 'reprinted on output.', 'type': 'string', 'default': ''}, 'verbosity': {'description': "Currently two verbosity levels are implemented: high, low. 'debug' and 'medium' have the same effect as 'high'; 'default' and 'minimal' as 'low'", 'type': 'string', 'enum': ['high', 'low', 'debug', 'medium', 'minimal', 'default'], 'default': 'low'}, 'restart_mode': {'type': 'string', 'enum': ['from_scratch', 'restart'], 'default': 'from_scratch'}, 'wf_collect': {'type': 'boolean', 'description': 'OBSOLETE - NO LONGER IMPLEMENTED'}, 'nstep': {'description': 'Default: 1 if calculation == \'scf\', \'nscf\', \'bands\'; 50 for the other cases; Number of molecular-dynamics or structural optimization steps performed in this run. If set to 0, the code performs a quick "dry run", stopping just after initialization. This is useful to check for input correctness and to have the summary printed. NOTE: in MD calculations, the code will perform "nstep" steps even if restarting from a previously interrupted calculation.', 'type': 'number', 'default': 1}, 'iprint': {'description': 'band energies are written every iprint iterations', 'type': 'number'}, 'tstress': {'type': 'boolean', 'default': False, 'description': "calculate stress. It is set to .TRUE. automatically if calculation == 'vc-md' or 'vc-relax'"}, 'tprnfor': {'type': 'boolean', 'description': "calculate forces. It is set to .TRUE. automatically if calculation == 'relax','md','vc-md'"}, 'dt': {'type': 'number', 'description': 'time step for molecular dynamics, in Rydberg atomic units (1 a.u.=4.8378 * 10^-17 s : beware, the CP code uses Hartree atomic units, half that much!!!)', 'default': 20}, 'outdir': {'type': 'string', 'description': 'input, temporary, output files are found in this directory, see also wfcdir'}, 'wfcdir': {'type': 'string', 'description': 'This directory specifies where to store files generated by each processor (*.wfc{N}, *.igk{N}, etc.). Useful for machines without a parallel file system: set wfcdir to a local file system, while outdir should be a parallel or network file system, visible to all processors. Beware: in order to restart from interrupted runs, or to perform further calculations using the produced data files, you may need to copy files to outdir. Works only for pw.x.'}, 'prefix': {'type': 'string', 'description': 'prepended to input/output filenames: prefix.wfc, prefix.rho, etc.', 'default': 'pwscf'}, 'lkpoint_dir': {'type': 'boolean', 'description': 'OBSOLETE - NO LONGER IMPLEMENTED'}, 'max_seconds': {'type': 'number', 'default': 10000000, 'description': 'Jobs stops after max_seconds CPU time. Use this option in conjunction with option restart_mode if you need to split a job too long to complete into shorter jobs that fit into your batch queues.'}, 'etot_conv_thr': {'type': 'number', 'default': 0.0001, 'description': 'Convergence threshold on total energy (a.u) for ionic minimization: the convergence criterion is satisfied when the total energy changes less than etot_conv_thr between two consecutive scf steps. Note that etot_conv_thr is extensive, like the total energy. See also forc_conv_thr - both criteria must be satisfied'}, 'forc_conv_thr': {'type': 'number', 'default': 0.001, 'description': 'Convergence threshold on forces (a.u) for ionic minimization: the convergence criterion is satisfied when all components of all forces are smaller than forc_conv_thr. See also etot_conv_thr - both criteria must be satisfied'}, 'disk_io': {'type': 'string', 'enum': ['high', 'medium', 'low', 'nowf', 'none'], 'description': 'Specifies the amount of disk I/O activity: (only for binary files and xml data file in data directory; other files printed at each molecular dynamics / structural optimization step are not controlled by this option )'}, 'pseudo_dir': {'type': 'string', 'description': "directory containing pseudopotential files. Default: value of the $ESPRESSO_PSEUDO environment variable if set; '$HOME/espresso/pseudo/' otherwise"}, 'tefield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. a saw-like potential simulating an electric field is added to the bare ionic potential. See variables edir, eamp, emaxpos, eopreg for the form and size of the added potential.'}, 'dipfield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. and tefield==.TRUE. a dipole correction is also added to the bare ionic potential - implements the recipe of L. Bengtsson, PRB 59, 12301 (1999). See variables edir, emaxpos, eopreg for the form of the correction. Must be used ONLY in a slab geometry, for surface calculations, with the discontinuity FALLING IN THE EMPTY SPACE.'}, 'lelfield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. a homogeneous finite electric field described through the modern theory of the polarization is applied. This is different from tefield == .true. !'}, 'nberrycyc': {'type': 'integer', 'default': 1, 'description': 'In the case of a finite electric field  ( lelfield == .TRUE. ) it defines the number of iterations for converging the wavefunctions in the electric field Hamiltonian, for each external iteration on the charge density'}, 'lorbm': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform orbital magnetization calculation.'}, 'lberry': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a Berry phase calculation. See the header of PW/src/bp_c_phase.f90 for documentation'}, 'gdir': {'type': 'number', 'description': 'For Berry phase calculation: direction of the k-point strings in reciprocal space. Allowed values: 1, 2, 3 1=first, 2=second, 3=third reciprocal lattice vector For calculations with finite electric fields (lelfield==.true.) "gdir" is the direction of the field.'}, 'nppstr': {'type': 'number', 'description': 'For Berry phase calculation: number of k-points to be calculated along each symmetry-reduced string. The same for calculation with finite electric fields (lelfield==.true.).'}, 'gate': {'type': 'boolean', 'default': False, 'description': 'In the case of charged cells (tot_charge .ne. 0) setting gate = .TRUE. represents the counter charge (i.e. -tot_charge) not by a homogeneous background charge but with a charged plate, which is placed at zgate (see below). Details of the gate potential can be found in T. Brumme, M. Calandra, F. Mauri; PRB 89, 245406 (2014). Note, that in systems which are not symmetric with respect to the plate, one needs to enable the dipole correction! (dipfield=.true.). Currently, symmetry can be used with gate=.true. but carefully check that no symmetry is included which maps z to -z even if in principle one could still use them for symmetric systems (i.e. no dipole correction). For nosym=.false. verbosity is set to \'high\'. Note: this option was called "monopole" in v6.0 and 6.1 of pw.x'}, 'twochem': {'type': 'boolean', 'default': False, 'description': 'IF .TRUE. , a two chemical potential calculation for the simulation of photoexcited systems is performed, constraining a fraction of the electrons in the conduction manifold.'}, 'lfcp': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a constant bias potential (constant-mu) calculation for a system with ESM method. See the header of PW/src/fcp_module.f90 for documentation. To perform the calculation, you must set a namelist FCP.'}, 'trism': {'type': 'boolean', 'default': False, 'description': "If .TRUE. perform a 3D-RISM-SCF calculation [for details see H.Sato et al., JCP 112, 9463 (2000), doi:10.1063/1.481564]. The solvent's distributions are calculated by 3D-RISM, though solute is treated as SCF. The charge density and the atomic positions are optimized, simultaneously with the solvents. To perform the calculation, you must set a namelist RISM and a card SOLVENTS. If assume_isolated = 'esm' and esm_bc = 'bc1', Laue-RISM is calculated instead of 3D-RISM and coupled with ESM method (i.e. ESM-RISM). [for details see S.Nishihara and M.Otani, PRB 96, 115429 (2017)]. The default of mixing_beta is 0.2 for both 3D-RISM and Laue-RISM. For structural relaxation with BFGS, ignore_wolfe is always .TRUE. ."}}}, '&SYSTEM': {'$id': '3pse/file/applications/espresso/7.2/pw.x/system', 'title': 'system schema', 'type': 'object', 'additionalProperties': False, 'anyOf': [{'properties': {'celldm': {'type': 'array', 'items': {'type': 'number'}, 'minLength': 6, 'maxLength': 6}}}, {'properties': {'A': {'type': 'number'}, 'B': {'type': 'number'}, 'C': {'type': 'number'}, 'cosAB': {'type': 'number'}, 'cosAC': {'type': 'number'}, 'cosBC': {'type': 'number'}}}], 'properties': {'ibrav': {'type': 'integer'}, 'nat': {'type': 'integer', 'description': 'number of atoms in the unit cell (ALL atoms, except if space_group is set, in which case, INEQUIVALENT atoms)'}, 'ntyp': {'type': 'integer', 'description': 'number of types of atoms in the unit cell'}, 'nbnd': {'type': 'integer', 'description': 'Default: for an insulator, nbnd = number of valence bands (nbnd = # of electrons /2); for a metal, 20% more (minimum 4 more)'}, 'nbnd_cond': {'type': 'integer', 'description': 'Default: nbnd_cond = nbnd - # of electrons / 2 in the collinear case; nbnd_cond = nbnd - # of electrons in the noncollinear case.'}, 'tot_charge': {'type': 'number', 'default': 0}, 'starting_charge': {'type': 'number', 'default': 0, 'description': "starting charge on atomic type 'i', to create starting potential with startingpot = 'atomic'."}, 'tot_magnetization': {'type': 'number', 'default': -10000, 'description': 'Total majority spin charge - minority spin charge. Used to impose a specific total electronic magnetization. If unspecified then tot_magnetization variable is ignored and the amount of electronic magnetization is determined during the self-consistent cycle.'}, 'starting_magnetization': {'type': 'array', 'items': {'type': 'number', 'default': 0, 'minimum': -1, 'maximum': 1}}, 'ecutwfc': {'type': 'number', 'description': 'kinetic energy cutoff (Ry) for wavefunctions'}, 'ecutrho': {'type': 'number', 'description': 'Kinetic energy cutoff (Ry) for charge density and potential For norm-conserving pseudopotential you should stick to the default value, you can reduce it by a little but it will introduce noise especially on forces and stress. Default: 4 * ecutwfc'}, 'ecutfock': {'type': 'number', 'description': 'Kinetic energy cutoff (Ry) for the exact exchange operator in EXX type calculations. By default this is the same as ecutrho but in some EXX calculations, a significant speed-up can be obtained by reducing ecutfock, at the expense of some loss in accuracy. Must be .gt. ecutwfc. Not implemented for stress calculation and for US-PP and PAW pseudopotentials.'}, 'nr1': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr2': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr3': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr1s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nr2s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nr3s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nosym': {'type': 'boolean', 'default': False}, 'nosym_evc': {'type': 'boolean', 'default': False}, 'noinv': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) disable the usage of k => -k symmetry (time reversal) in k-point generation'}, 'no_t_rev': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) disable the usage of magnetic symmetry operations that consist in a rotation + time reversal.'}, 'force_symmorphic': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) force the symmetry group to be symmorphic by disabling symmetry operations having an associated fractionary translation'}, 'use_all_frac': {'type': 'boolean', 'default': False}, 'occupations': {'type': 'string', 'enum': ['smearing', 'tetrahedra', 'tetrahedra_lin', 'tetrahedra_opt', 'fixed', 'from_input']}, 'one_atom_occupations': {'type': 'boolean', 'default': False}, 'starting_spin_angle': {'type': 'boolean', 'default': False}, 'degauss_cond': {'type': 'number', 'default': 0, 'description': 'value of the gaussian spreading (Ry) for brillouin-zone integration in the conduction manifold in a two-chemical potential calculation (twochem=.true.).'}, 'nelec_cond': {'type': 'number', 'default': 0, 'description': 'Number of electrons placed in the conduction manifold in a two-chemical potential calculation (twochem=.true.). Of the total # of electrons nelec, nelec-nelec_cond will occupy the valence manifold and nelec_cond will be constrained in the conduction manifold.'}, 'degauss': {'type': 'number', 'default': 0, 'description': 'value of the gaussian spreading (Ry) for brillouin-zone integration in metals.'}, 'smearing': {'type': 'string', 'enum': ['gaussian', 'gauss', 'methfessel-paxton', 'm-p', 'mp', 'marzari-vanderbilt', 'cold', 'm-v', 'mv', 'fermi-dirac', 'f-d', 'fd'], 'default': 'gaussian'}, 'nspin': {'type': 'integer', 'default': 1}, 'sic_gamma': {'type': 'number', 'default': 0, 'description': 'Strength of the gammaDFT potential.'}, 'pol_type': {'type': 'string', 'enum': ['e', 'h'], 'description': 'Type of polaron in gammaDFT.'}, 'sic_energy': {'type': 'boolean', 'default': False, 'description': 'Enable the calculation of the total energy in gammaDFT. When .true., a preliminary calculation is performed to calculate the electron density in the absence of the polaron. When .false., the total energy printed in output should not be considered. For structural relaxations, it is recommended to use .false. to avoid doubling the computational cost.'}, 'sci_vb': {'type': 'number', 'default': 0, 'description': 'Valence band shift (in eV) through self-consistent scissor operator. When performing gammaDFT calculations of polarons, the polaron level is not shifted.'}, 'sci_cb': {'type': 'number', 'default': 0, 'description': 'Conduction band band shift (in eV) through self-consistent scissor operator. When performing gammaDFT calculations of polarons, the polaron level is not shifted.'}, 'noncolin': {'type': 'boolean', 'default': False, 'description': 'if .true. the program will perform a noncollinear calculation.'}, 'ecfixed': {'type': 'number', 'default': 0}, 'qcutz': {'type': 'number', 'default': 0}, 'q2sigma': {'type': 'number', 'default': 0.1}, 'input_dft': {'type': 'string', 'description': "Exchange-correlation functional: eg 'PBE', 'BLYP' etc See Modules/funct.f90 for allowed values. Overrides the value read from pseudopotential files. Use with care and if you know what you are doing!"}, 'ace': {'type': 'boolean', 'default': True, 'description': 'Use Adaptively Compressed Exchange operator as in Lin Lin, J. Chem. Theory Comput. 2016, 12, 2242--2249, doi:10.1021/acs.jctc.6b00092'}, 'exx_fraction': {'type': 'number', 'description': "Fraction of EXX for hybrid functional calculations. In the case of input_dft='PBE0', the default value is 0.25, while for input_dft='B3LYP' the exx_fraction default value is 0.20."}, 'screening_parameter': {'type': 'number', 'default': 0.106, 'description': 'screening_parameter for HSE like hybrid functionals.'}, 'exxdiv_treatment': {'type': 'string', 'default': 'gygi-baldereschi', 'enum': ['gygi-baldereschi', 'vcut_spherical', 'vcut_ws', 'none']}, 'x_gamma_extrapolation': {'type': 'boolean', 'default': True, 'description': 'Specific for EXX. If .true., extrapolate the G=0 term of the potential'}, 'ecutvcut': {'type': 'number', 'description': 'Reciprocal space cutoff for correcting Coulomb potential divergencies at small q vectors.'}, 'nqx1': {'type': 'integer', 'default': 0, 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'nqx2': {'type': 'integer', 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'nqx3': {'type': 'integer', 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'localization_thr': {'type': 'number', 'default': 0, 'description': 'Overlap threshold over which the exchange integral over a pair of localized orbitals is included in the evaluation of EXX operator. Any value greater than 0.0 triggers the SCDM localization and the evaluation on EXX using the localized orbitals. Very small value of the threshold should yield the same result as the default EXX evaluation'}, 'Hubbard_occ': {'type': 'array', 'items': {'type': 'array', 'items': [{'type': 'integer', 'minimum': 1}, {'type': 'integer', 'minimum': 1, 'maximum': 3}, {'type': 'number'}]}}, 'Hubbard_alpha': {'type': 'array', 'items': {'type': 'number', 'default': 0}}, 'Hubbard_beta': {'type': 'array', 'items': {'type': 'number', 'default': 0}}, 'starting_ns_eigenvalue': {'type': 'array', 'items': {'type': 'array', 'items': {'type': 'array', 'items': {'type': 'number'}}}}, 'dmft': {'type': 'boolean', 'default': False, 'description': 'If true, nscf calculation will exit in restart mode, scf calculation will restart from there if DMFT updates are provided as hdf5 archive. Scf calculation should be used only with electron_maxstep = 1.'}, 'dmft_prefix': {'type': 'string', 'description': 'prepended to hdf5 archive: dmft_prefix.h5'}, 'ensemble_energies': {'type': 'boolean', 'default': False, 'description': 'If ensemble_energies = .true., an ensemble of xc energies is calculated non-selfconsistently for perturbed exchange-enhancement factors and LDA vs. PBE correlation ratios after each converged electronic ground state calculation.'}, 'edir': {'type': 'integer', 'description': 'The direction of the electric field or dipole correction is parallel to the bg(:,edir) reciprocal lattice vector, so the potential is constant in planes defined by FFT grid points; edir = 1, 2 or 3. Used only if tefield is .TRUE.'}, 'emaxpos': {'type': 'number', 'default': 0.5, 'description': 'Position of the maximum of the saw-like potential along crystal axis edir, within the  unit cell (see below), 0 < emaxpos < 1 Used only if tefield is .TRUE.'}, 'eopreg': {'type': 'number', 'default': 0.1, 'description': 'Zone in the unit cell where the saw-like potential decreases. ( see below, 0 < eopreg < 1 ). Used only if tefield is .TRUE.'}, 'eamp': {'type': 'number', 'default': 0.001}, 'angle1': {'type': 'array', 'items': {'type': 'number'}, 'maxItems': 1, 'minItems': 1, 'description': 'The angle expressed in degrees between the initial magnetization and the z-axis. For noncollinear calculations only; index i runs over the atom types.'}, 'angle2': {'type': 'array', 'items': {'type': 'number'}, 'maxItems': 1, 'minItems': 1, 'description': 'The angle expressed in degrees between the projection of the initial magnetization on x-y plane and the x-axis. For noncollinear calculations only.'}, 'lforcet': {'type': 'boolean', 'description': 'When starting a non collinear calculation using an existing density file from a collinear lsda calculation assumes previous density points in z direction and rotates it in the direction described by angle1 and angle2 variables for atomic type 1'}, 'constrained_magnetization': {'type': 'string', 'enum': ['none', 'total', 'atomic', 'total direction', 'atomic direction'], 'default': 'none', 'description': 'Used to perform constrained calculations in magnetic systems.'}, 'fixed_magnetization': {'type': 'array', 'items': {'type': 'number', 'default': 0}, 'maxItems': 3, 'minItems': 3}, 'lambda': {'type': 'number', 'default': 1, 'description': 'parameter used for constrained_magnetization calculations N.B.: if the scf calculation does not converge, try to reduce lambda to obtain convergence, then restart the run with a larger lambda'}, 'report': {'type': 'integer', 'default': -1, 'description': 'determines when atomic magnetic moments are printed on output'}, 'lspinorb': {'type': 'boolean', 'description': 'if .TRUE. the noncollinear code can use a pseudopotential with spin-orbit.'}, 'assume_isolated': {'type': 'string', 'enum': ['none', 'makov-payne', 'm-p', 'mp', 'martyna-tuckerman', 'm-t', 'mt', 'esm', '2D'], 'default': 'none', 'definition': 'Used to perform calculation assuming the system to be isolated (a molecule or a cluster in a 3D supercell)'}, 'esm_bc': {'type': 'string', 'enum': ['pbc', 'bc1', 'bc2', 'bc3'], 'default': 'pbc', 'definition': "If assume_isolated = 'esm', determines the boundary conditions used for either side of the slab."}, 'esm_w': {'type': 'number', 'default': 0, 'description': "If assume_isolated = 'esm', determines the position offset [in a.u.] of the start of the effective screening region, measured relative to the cell edge. (ESM region begins at z = +/- [L_z/2 + esm_w] )."}, 'esm_efield': {'type': 'number', 'default': 0, 'description': "If assume_isolated = 'esm' and esm_bc = 'bc2', gives the magnitude of the electric field [Ry/a.u.] to be applied between semi-infinite ESM electrodes."}, 'esm_nfit': {'type': 'integer', 'default': 4, 'description': "If assume_isolated = 'esm', gives the number of z-grid points for the polynomial fit along the cell edge."}, 'lgcscf': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a constant bias potential (constant-mu) calculation with Grand-Canonical SCF.'}, 'gcscf_mu': {'type': 'number', 'description': 'The target Fermi energy (eV) of GC-SCF. One can start with appropriate total charge of the system by giving tot_charge'}, 'gcscf_conv_thr': {'type': 'number', 'default': 0.01, 'description': 'Convergence threshold of Fermi energy (eV) for GC-SCF.'}, 'gcscf_beta': {'type': 'number', 'default': 0.05, 'description': 'Mixing factor for GC-SCF. Larger values are recommended, if systems with small DOS on Fermi surface as graphite.'}, 'vdw_corr': {'type': 'string', 'enum': ['grimme-d2', 'Grimme-D2', 'DFT-D', 'dft-d', 'grimme-d3', 'Grimme-D3', 'DFT-D3', 'dft-d3', 'TS', 'ts', 'ts-vdw', 'ts-vdW', 'tkatchenko-scheffler', 'MBD', 'mbd', 'many-body-dispersion', 'mbd_vdw', 'XDM', 'xdm'], 'default': 'none', 'description': 'Type of Van der Waals correction'}, 'london': {'type': 'boolean', 'default': False, 'description': "OBSOLESCENT, same as vdw_corr='DFT-D'"}, 'london_s6': {'type': 'number', 'default': 0.75, 'description': 'global scaling parameter for DFT-D. Default is good for PBE.'}, 'london_c6': {'type': 'number', 'description': 'atomic C6 coefficient of each atom type'}, 'london_rvdw': {'type': 'number', 'description': 'atomic vdw radii of each atom type'}, 'london_rcut': {'type': 'number', 'default': 200, 'description': 'cutoff radius (a.u.) for dispersion interactions'}, 'dftd3_version': {'type': 'integer', 'default': 3, 'minimum': 2, 'maximum': 6, 'description': 'Version of Grimme implementation of Grimme-D3'}, 'dftd3_threebody': {'type': 'boolean', 'default': True, 'description': 'Turn three-body terms in Grimme-D3 on. If .false. two-body contributions only are computed, using two-body parameters of Grimme-D3. If dftd3_version=2, three-body contribution is always disabled.'}, 'ts_vdw_econv_thr': {'type': 'number', 'default': 1e-06, 'description': 'Optional: controls the convergence of the vdW energy (and forces). The default value is a safe choice, likely too safe, but you do not gain much in increasing it'}, 'ts_vdw_isolated': {'type': 'boolean', 'default': False, 'description': 'Optional: set it to .TRUE. when computing the Tkatchenko-Scheffler vdW energy or the Many-Body dispersion (MBD) energy for an isolated (non-periodic) system.'}, 'xdm': {'type': 'boolean', 'default': False, 'description': "OBSOLESCENT, same as vdw_corr='xdm'"}, 'xdm_a1': {'type': 'number', 'default': 0.6836, 'description': 'Damping function parameter a1 (adimensional)'}, 'xdm_a2': {'type': 'number', 'default': 1.5045, 'description': 'Damping function parameter a2 (angstrom)'}, 'space_group': {'type': 'integer', 'default': 0, 'description': 'The number of the space group of the crystal, as given in the International Tables of Crystallography A (ITA)'}, 'uniqueb': {'type': 'boolean', 'default': False, 'description': 'Used only for monoclinic lattices'}, 'origin_choice': {'type': 'integer', 'default': 1, 'description': 'Used only for space groups that in the ITA allow the use of two different origins'}, 'rhombohedral': {'type': 'boolean', 'default': True, 'description': 'Used only for rhombohedral space groups.'}, 'zgate': {'type': 'number', 'default': 0.5, 'description': 'used only if gate = .TRUE.'}, 'relaxz': {'type': 'boolean', 'default': False, 'description': 'used only if gate = .TRUE.'}, 'block': {'type': 'boolean', 'default': False, 'description': 'used only if gate = .TRUE.'}, 'block_1': {'type': 'number', 'default': 0.45, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'block_2': {'type': 'number', 'default': 0.55, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'block_height': {'type': 'number', 'default': 0.1, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'nextffield': {'type': 'integer', 'default': 0, 'description': 'Number of activated external ionic force fields.'}}, 'required': ['ibrav', 'nat', 'ntyp', 'ecutwfc']}, '&ELECTRONS': {'$id': '3pse/file/applications/espresso/7.2/pw.x/electrons', 'title': 'electrons schema', 'type': 'object', 'additionalProperties': False, 'properties': {'electron_maxstep': {'type': 'integer', 'default': 100, 'description': 'maximum number of iterations in a scf step. If exact exchange is active, this will affect the inner loops.'}, 'exx_maxstep': {'type': 'integer', 'default': 100, 'description': 'maximum number of outer iterations in a scf calculation with exact exchange.'}, 'scf_must_converge': {'type': 'boolean', 'default': True, 'description': 'If .false. do not stop molecular dynamics or ionic relaxation when electron_maxstep is reached. Use with care.'}, 'conv_thr': {'type': 'number'}, 'adaptive_thr': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. this turns on the use of an adaptive conv_thr for the inner scf loops when using EXX.'}, 'conv_thr_init': {'type': 'number', 'description': 'When adaptive_thr = .TRUE. this is the convergence threshold used for the first scf cycle.'}, 'conv_thr_multi': {'type': 'number', 'description': 'When adaptive_thr = .TRUE. the convergence threshold for each scf cycle is given by: max( conv_thr, conv_thr_multi * dexx )'}, 'mixing_mode': {'type': 'string', 'enum': ['plain', 'TF', 'local-TF'], 'default': 'plain'}, 'mixing_beta': {'type': 'number', 'description': 'mixing factor for self-consistency'}, 'mixing_ndim': {'type': 'integer', 'default': 8, 'description': 'number of iterations used in mixing scheme'}, 'mixing_fixed_ns': {'type': 'integer', 'default': 0, 'description': 'For DFT+U : number of iterations with fixed ns ( ns is the atomic density appearing in the Hubbard term ).'}, 'diagonalization': {'type': 'string', 'enum': ['david', 'cg', 'ppcg', 'paro', 'ParO', 'rmm-davidson', 'rmm-paro'], 'default': 'david'}, 'diago_thr_init': {'type': 'number', 'description': 'Convergence threshold (ethr) for iterative diagonalization (the check is on eigenvalue convergence).'}, 'diago_cg_maxiter': {'type': 'integer', 'description': 'For conjugate gradient diagonalization:  max number of iterations'}, 'diago_ppcg_maxiter': {'type': 'integer', 'description': 'For ppcg diagonalization:  max number of iterations'}, 'diago_david_ndim': {'type': 'integer', 'default': 2, 'description': 'For Davidson diagonalization: dimension of workspace (number of wavefunction packets, at least 2 needed).'}, 'diago_rmm_ndim': {'type': 'integer', 'default': 4, 'description': 'For RMM-DIIS diagonalization: dimension of workspace (number of wavefunction packets, at least 2 needed).'}, 'diago_rmm_conv': {'type': 'boolean', 'default': False, 'description': 'If .TRUE., RMM-DIIS is performed up to converge. If .FALSE., RMM-DIIS is performed only once.'}, 'diago_gs_nblock': {'type': 'integer', 'default': 16, 'description': 'For RMM-DIIS diagonalization: blocking size of Gram-Schmidt orthogonalization'}, 'diago_full_acc': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. all the empty states are diagonalized at the same level of accuracy of the occupied ones. Otherwise the empty states are diagonalized using a larger threshold (this should not affect total energy, forces, and other ground-state properties).'}, 'efield': {'type': 'number', 'description': 'Amplitude of the finite electric field (in Ry a.u.; 1 a.u. = 36.3609*10^10 V/m). Used only if lelfield==.TRUE. and if k-points (K_POINTS card) are not automatic.'}, 'efield_cart': {'description': 'Finite electric field (in Ry a.u.=36.3609*10^10 V/m) in cartesian axis. Used only if lelfield==.TRUE. and if k-points (K_POINTS card) are automatic.', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'efield_phase': {'type': 'string', 'enum': ['read', 'write', 'none'], 'default': 'none'}, 'startingpot': {'type': 'string', 'enum': ['atomic', 'file']}, 'startingwfc': {'type': 'string', 'enum': ['atomic', 'atomic+random', 'random', 'file'], 'default': 'atomic+random'}, 'tqr': {'type': 'boolean', 'default': False, 'description': 'If .true., use a real-space algorithm for augmentation charges of ultrasoft pseudopotentials and PAWsets. Faster but numerically less accurate than the default G-space algorithm. Use with care and after testing!'}, 'real_space': {'type': 'boolean', 'default': False, 'description': 'If .true., exploit real-space localization to compute matrix elements for nonlocal projectors. Faster and in principle better scaling than the default G-space algorithm, but numerically less accurate, may lead to some loss of translational invariance. Use with care and after testing!'}}}, '&IONS': {'oneOf': [{'$id': '3pse/file/applications/espresso/7.2/pw.x/ions', 'title': 'ions schema', 'type': 'object', 'additionalProperties': False, 'oneOf': [{'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['bfgs', 'damp', 'fire'], 'default': 'bfgs', 'description': "CASE: calculation == 'relax'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['verlet', 'langevin', 'langevin-smc'], 'default': 'verlet', 'description': "CASE: calculation == 'md'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['bfgs', 'damp'], 'default': 'bfgs', 'description': "CASE: calculation == 'vc-relax'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['beeman'], 'default': 'beeman', 'description': "CASE: calculation == 'vc-md'"}}}], 'properties': {'ion_positions': {'type': 'string', 'enum': ['default', 'from_input'], 'default': 'default'}, 'ion_velocities': {'type': 'string', 'enum': ['default', 'from_input'], 'default': 'default'}, 'pot_extrapolation': {'type': 'string', 'enum': ['none', 'atomic', 'first_order', 'second_order'], 'default': 'atomic', 'description': 'Used to extrapolate the potential from preceding ionic steps.'}, 'wfc_extrapolation': {'type': 'string', 'enum': ['none', 'first_order', 'second_order'], 'default': 'none', 'description': 'Used to extrapolate the wavefunctions from preceding ionic steps.'}, 'remove_rigid_rot': {'type': 'boolean', 'default': False, 'description': 'This keyword is useful when simulating the dynamics and/or the thermodynamics of an isolated system. If set to true the total torque of the internal forces is set to zero by adding new forces that compensate the spurious interaction with the periodic images. This allows for the use of smaller supercells.'}, 'ion_temperature': {'type': 'string', 'enum': ['rescaling', 'rescale-v', 'rescale-T', 'reduce-T', 'berendsen', 'andersen', 'svr', 'initial', 'not_controlled'], 'default': 'not_controlled'}, 'tempw': {'type': 'number', 'description': 'Starting temperature (Kelvin) in MD runs target temperature for most thermostats.', 'default': 300}, 'tolp': {'type': 'number', 'description': 'Tolerance for velocity rescaling. Velocities are rescaled if the run-averaged and target temperature differ more than tolp.', 'default': 100}, 'delta_t': {'type': 'number', 'default': 1}, 'nraise': {'type': 'integer', 'default': 1}, 'refold_pos': {'type': 'boolean', 'default': False, 'description': 'This keyword applies only in the case of molecular dynamics or damped dynamics. If true the ions are refolded at each step into the supercell.'}, 'upscale': {'type': 'number', 'description': 'Max reduction factor for conv_thr during structural optimization conv_thr is automatically reduced when the relaxation approaches convergence so that forces are still accurate, but conv_thr will not be reduced to less that conv_thr / upscale.', 'default': 100}, 'bfgs_ndim': {'type': 'integer', 'default': 1, 'description': 'Number of old forces and displacements vectors used in the PULAY mixing of the residual vectors obtained on the basis of the inverse hessian matrix given by the BFGS algorithm.'}, 'trust_radius_max': {'type': 'number', 'description': 'Maximum ionic displacement in the structural relaxation. (bfgs only)', 'default': 0.8}, 'trust_radius_min': {'type': 'number', 'description': 'Minimum ionic displacement in the structural relaxation BFGS is reset when trust_radius < trust_radius_min. (bfgs only)', 'default': 0.001}, 'trust_radius_ini': {'type': 'number', 'description': 'Initial ionic displacement in the structural relaxation. (bfgs only)', 'default': 0.5}, 'w_1': {'type': 'number', 'default': 0.01}, 'w_2': {'type': 'number', 'description': 'Parameters used in line search based on the Wolfe conditions. (bfgs only)', 'default': 0.5}, 'fire_alpha_init': {'type': 'number', 'description': 'Initial value of the alpha mixing factor in the FIRE minimization scheme; recommended values are between 0.1 and 0.3', 'default': 0.2}, 'fire_falpha': {'type': 'number', 'description': 'Scaling of the alpha mixing parameter for steps with P > 0;', 'default': 0.99}, 'fire_nmin': {'type': 'integer', 'default': 5, 'description': 'Minimum number of steps with P > 0 before increase of dt'}, 'fire_f_inc': {'type': 'number', 'description': 'Factor for increasing dt', 'default': 1.1}, 'fire_f_dec': {'type': 'number', 'description': 'Factor for decreasing dt', 'default': 0.5}, 'fire_dtmax': {'type': 'number', 'description': 'Determines the maximum value of dt in the FIRE minimization; dtmax = fire_dtmax*dt', 'default': 10}}}, {'type': 'null'}]}, '&CELL': {'oneOf': [{'$id': '3pse/file/applications/espresso/7.2/pw.x/cell', 'title': 'cell schema', 'type': 'object', 'additionalProperties': False, 'oneOf': [{'type': 'object', 'additionalProperties': False, 'properties': {'cell_dynamics': {'type': 'string', 'enum': ['none', 'sd', 'damp-pr', 'damp-w', 'bfgs'], 'default': 'bfgs', 'description': "CASE ( calculation == 'vc-relax' )"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'cell_dynamics': {'type': 'string', 'enum': ['none', 'pr', 'w'], 'description': "CASE ( calculation == 'vc-md' )"}}}], 'properties': {'press': {'type': 'number', 'description': 'Target pressure [KBar] in a variable-cell md or relaxation run.', 'default': 0}, 'wmass': {'type': 'number', 'description': "Fictitious cell mass [amu] for variable-cell simulations (both 'vc-md' and 'vc-relax'). Default: 0.75*Tot_Mass/pi**2 for Parrinello-Rahman MD; 0.75*Tot_Mass/pi**2/Omega**(2/3) for Wentzcovitch MD"}, 'cell_factor': {'type': 'number', 'description': 'Used in the construction of the pseudopotential tables. It should exceed the maximum linear contraction of the cell during a simulation. Default: 2.0 for variable-cell calculations, 1.0 otherwise'}, 'press_conv_thr': {'type': 'number', 'default': 0.5, 'description': "Convergence threshold on the pressure for variable cell relaxation ('vc-relax' : note that the other convergence thresholds for ionic relaxation apply as well)."}, 'cell_dofree': {'type': 'string', 'enum': ['all', 'ibrav', 'a', 'b', 'c', 'fixa', 'fixb', 'fixc', 'x', 'y', 'xy', 'xz', 'xyz', 'shape', 'volume', '2Dxy', '2Dshape', 'epitaxial_ab', 'epitaxial_ac', 'epitaxial_bc'], 'default': 'all', 'description': 'Select which of the cell parameters should be moved'}}}, {'type': 'null'}]}, 'ATOMIC_SPECIES': {'$id': '3pse/file/applications/espresso/7.2/pw.x/atomic-species', 'title': 'atomic species schema', 'type': 'object', 'additionalProperties': False, 'properties': {'values': {'type': 'array', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'X': {'type': 'string', 'description': 'label of the atom. Acceptable syntax: chemical symbol X (1 or 2 characters, case-insensitive) or chemical symbol plus a number or a letter, as in "Xn" (e.g. Fe1) or "X_*" or "X-*" (e.g. C1, C_h; max total length cannot exceed 3 characters)'}, 'Mass_X': {'type': 'number', 'description': 'mass of the atomic species [amu: mass of C = 12]. Used only when performing Molecular Dynamics run or structural optimization runs using Damped MD. Not actually used in all other cases (but stored in data files, so phonon calculations will use these values unless other values are provided)'}, 'PseudoPot_X': {'type': 'string', 'description': 'PseudoPot_X'}}}}}}, 'ATOMIC_POSITIONS': {'$id': '3pse/file/applications/espresso/7.2/pw.x/atomic-positions', 'title': 'atomic positions schema', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1493', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['alat', 'bohr', 'angstrom', 'crystal', 'crystal_sg'], 'default': 'alat'}, 'values': {'type': 'array', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'X': {'type': 'string', 'description': 'label of the atom as specified in ATOMIC_SPECIES'}, 'x': {'type': 'number', 'description': 'atomic positions'}, 'y': {'type': 'number', 'description': 'atomic positions'}, 'z': {'type': 'number', 'description': 'atomic positions'}, 'if_pos(1)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}, 'if_pos(2)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}, 'if_pos(3)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}}, 'required': ['x', 'y', 'z']}}}}, 'K_POINTS': {'$id': '3pse/file/applications/espresso/7.2/pw.x/k-points', 'title': 'k points schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['tpiba', 'automatic', 'crystal', 'gamma', 'tpiba_b', 'crystal_b', 'tpiba_c', 'crystal_c']}, 'values': {'oneOf': [{'type': 'array', 'description': 'K_POINTS tpiba | crystal | tpiba_b | crystal_b | tpiba_c | crystal_c', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'nks': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'xk_x': {'type': 'number'}, 'xk_y': {'type': 'number'}, 'xk_z': {'type': 'number'}, 'wk': {'type': 'number'}}}}, {'type': 'object', 'description': 'K_POINTS automatic', 'additionalProperties': False, 'properties': {'nk1': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'nk2': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'nk3': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk1': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk2': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk3': {'type': 'integer', 'description': 'Number of supplied special k-points.'}}}, {'type': 'null', 'description': 'K_POINTS gamma'}]}}}, 'CELL_PARAMETERS': {'$id': '3pse/file/applications/espresso/7.2/pw.x/cell-parameters', 'title': 'cell parameters schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['alat', 'bohr', 'angstrom'], 'description': 'label of the atom. Acceptable syntax: chemical symbol X (1 or 2 characters, case-insensitive) or chemical symbol plus a number or a letter, as in "Xn" (e.g. Fe1) or "X_*" or "X-*" (e.g. C1, C_h; max total length cannot exceed 3 characters)'}, 'values': {'type': 'object', 'additionalProperties': False, 'properties': {'v1': {'description': '1st lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'v2': {'description': '2nd lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'v3': {'description': '3rd lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}}}}, 'HUBBARD': {'$id': '3pse/file/applications/espresso/7.2/pw.x/hubbard', 'title': 'hubbard schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['atomic', 'ortho-atomic', 'norm-atomic', 'wf', 'pseudo']}, 'values': {'oneOf': [{'description': 'IF DFT+U', 'type': 'array', 'items': {'oneOf': [{'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1764', 'additionalProperties': False, 'properties': {'U': {'type': 'string', 'enum': ['U'], 'description': 'string constant "U"; indicates the specs for the U parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'u_val': {'type': 'number', 'description': 'value of the U parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1784', 'additionalProperties': False, 'properties': {'J0': {'type': 'string', 'enum': ['J0'], 'description': 'string constant "J0"; indicates the specs for the J0 parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'j0_val': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}]}}, {'description': 'IF DFT+U+J', 'type': 'array', 'items': {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1817', 'additionalProperties': False, 'properties': {'paramType': {'type': 'string', 'enum': ['U', 'J', 'B', 'E2', 'E3'], 'description': 'character describing the type of Hubbard parameter allowed values: U, J and either B (for d-orbitals) or E2 and E3 (for f-orbitals)'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'paramValue': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}}, {'description': 'IF DFT+U+V', 'type': 'array', 'items': {'oneOf': [{'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1847', 'additionalProperties': False, 'properties': {'U': {'type': 'string', 'enum': ['U'], 'description': 'string constant "U"; indicates the specs for the U parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'u_val': {'type': 'number', 'description': 'value of the U parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1865', 'additionalProperties': False, 'properties': {'J0': {'type': 'string', 'enum': ['J0'], 'description': 'string constant "J0"; indicates the specs for the J0 parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'j0_val': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1884', 'additionalProperties': False, 'properties': {'V': {'type': 'string', 'enum': ['V'], 'description': 'string constant "V"; indicates the specs for the V parameter will be given'}, 'label(I)': {'type': 'string', 'description': 'label of the atom I (as defined in ATOMIC_SPECIES)'}, 'manifold(I)': {'type': 'string', 'description': 'specs of the manifold for atom I (e.g., 3d, 2p...)'}, 'label(J)': {'type': 'string', 'description': 'label of the atom J (as defined in ATOMIC_SPECIES)'}, 'manifold(J)': {'type': 'string', 'description': 'specs of the manifold for atom J (e.g., 3d, 2p...)'}, 'I': {'type': 'integer', 'description': 'index of the atom I'}, 'J': {'type': 'integer', 'description': 'index of the atom J'}, 'v_val(I,J)': {'type': 'number', 'description': 'value of the V parameter for the atom pair I,J (in eV)'}}}]}}]}}}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/electrons', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'electrons schema', 'type': 'object', 'additionalProperties': False, 'properties': {'electron_maxstep': {'type': 'integer', 'default': 100, 'description': 'maximum number of iterations in a scf step. If exact exchange is active, this will affect the inner loops.'}, 'exx_maxstep': {'type': 'integer', 'default': 100, 'description': 'maximum number of outer iterations in a scf calculation with exact exchange.'}, 'scf_must_converge': {'type': 'boolean', 'default': True, 'description': 'If .false. do not stop molecular dynamics or ionic relaxation when electron_maxstep is reached. Use with care.'}, 'conv_thr': {'type': 'number'}, 'adaptive_thr': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. this turns on the use of an adaptive conv_thr for the inner scf loops when using EXX.'}, 'conv_thr_init': {'type': 'number', 'description': 'When adaptive_thr = .TRUE. this is the convergence threshold used for the first scf cycle.'}, 'conv_thr_multi': {'type': 'number', 'description': 'When adaptive_thr = .TRUE. the convergence threshold for each scf cycle is given by: max( conv_thr, conv_thr_multi * dexx )'}, 'mixing_mode': {'type': 'string', 'enum': ['plain', 'TF', 'local-TF'], 'default': 'plain'}, 'mixing_beta': {'type': 'number', 'description': 'mixing factor for self-consistency'}, 'mixing_ndim': {'type': 'integer', 'default': 8, 'description': 'number of iterations used in mixing scheme'}, 'mixing_fixed_ns': {'type': 'integer', 'default': 0, 'description': 'For DFT+U : number of iterations with fixed ns ( ns is the atomic density appearing in the Hubbard term ).'}, 'diagonalization': {'type': 'string', 'enum': ['david', 'cg', 'ppcg', 'paro', 'ParO', 'rmm-davidson', 'rmm-paro'], 'default': 'david'}, 'diago_thr_init': {'type': 'number', 'description': 'Convergence threshold (ethr) for iterative diagonalization (the check is on eigenvalue convergence).'}, 'diago_cg_maxiter': {'type': 'integer', 'description': 'For conjugate gradient diagonalization:  max number of iterations'}, 'diago_ppcg_maxiter': {'type': 'integer', 'description': 'For ppcg diagonalization:  max number of iterations'}, 'diago_david_ndim': {'type': 'integer', 'default': 2, 'description': 'For Davidson diagonalization: dimension of workspace (number of wavefunction packets, at least 2 needed).'}, 'diago_rmm_ndim': {'type': 'integer', 'default': 4, 'description': 'For RMM-DIIS diagonalization: dimension of workspace (number of wavefunction packets, at least 2 needed).'}, 'diago_rmm_conv': {'type': 'boolean', 'default': False, 'description': 'If .TRUE., RMM-DIIS is performed up to converge. If .FALSE., RMM-DIIS is performed only once.'}, 'diago_gs_nblock': {'type': 'integer', 'default': 16, 'description': 'For RMM-DIIS diagonalization: blocking size of Gram-Schmidt orthogonalization'}, 'diago_full_acc': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. all the empty states are diagonalized at the same level of accuracy of the occupied ones. Otherwise the empty states are diagonalized using a larger threshold (this should not affect total energy, forces, and other ground-state properties).'}, 'efield': {'type': 'number', 'description': 'Amplitude of the finite electric field (in Ry a.u.; 1 a.u. = 36.3609*10^10 V/m). Used only if lelfield==.TRUE. and if k-points (K_POINTS card) are not automatic.'}, 'efield_cart': {'description': 'Finite electric field (in Ry a.u.=36.3609*10^10 V/m) in cartesian axis. Used only if lelfield==.TRUE. and if k-points (K_POINTS card) are automatic.', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'efield_phase': {'type': 'string', 'enum': ['read', 'write', 'none'], 'default': 'none'}, 'startingpot': {'type': 'string', 'enum': ['atomic', 'file']}, 'startingwfc': {'type': 'string', 'enum': ['atomic', 'atomic+random', 'random', 'file'], 'default': 'atomic+random'}, 'tqr': {'type': 'boolean', 'default': False, 'description': 'If .true., use a real-space algorithm for augmentation charges of ultrasoft pseudopotentials and PAWsets. Faster but numerically less accurate than the default G-space algorithm. Use with care and after testing!'}, 'real_space': {'type': 'boolean', 'default': False, 'description': 'If .true., exploit real-space localization to compute matrix elements for nonlocal projectors. Faster and in principle better scaling than the default G-space algorithm, but numerically less accurate, may lead to some loss of translational invariance. Use with care and after testing!'}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/cell', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'cell schema', 'type': 'object', 'additionalProperties': False, 'oneOf': [{'type': 'object', 'additionalProperties': False, 'properties': {'cell_dynamics': {'type': 'string', 'enum': ['none', 'sd', 'damp-pr', 'damp-w', 'bfgs'], 'default': 'bfgs', 'description': "CASE ( calculation == 'vc-relax' )"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'cell_dynamics': {'type': 'string', 'enum': ['none', 'pr', 'w'], 'description': "CASE ( calculation == 'vc-md' )"}}}], 'properties': {'press': {'type': 'number', 'description': 'Target pressure [KBar] in a variable-cell md or relaxation run.', 'default': 0}, 'wmass': {'type': 'number', 'description': "Fictitious cell mass [amu] for variable-cell simulations (both 'vc-md' and 'vc-relax'). Default: 0.75*Tot_Mass/pi**2 for Parrinello-Rahman MD; 0.75*Tot_Mass/pi**2/Omega**(2/3) for Wentzcovitch MD"}, 'cell_factor': {'type': 'number', 'description': 'Used in the construction of the pseudopotential tables. It should exceed the maximum linear contraction of the cell during a simulation. Default: 2.0 for variable-cell calculations, 1.0 otherwise'}, 'press_conv_thr': {'type': 'number', 'default': 0.5, 'description': "Convergence threshold on the pressure for variable cell relaxation ('vc-relax' : note that the other convergence thresholds for ionic relaxation apply as well)."}, 'cell_dofree': {'type': 'string', 'enum': ['all', 'ibrav', 'a', 'b', 'c', 'fixa', 'fixb', 'fixc', 'x', 'y', 'xy', 'xz', 'xyz', 'shape', 'volume', '2Dxy', '2Dshape', 'epitaxial_ab', 'epitaxial_ac', 'epitaxial_bc'], 'default': 'all', 'description': 'Select which of the cell parameters should be moved'}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/cell-parameters', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'cell parameters schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['alat', 'bohr', 'angstrom'], 'description': 'label of the atom. Acceptable syntax: chemical symbol X (1 or 2 characters, case-insensitive) or chemical symbol plus a number or a letter, as in "Xn" (e.g. Fe1) or "X_*" or "X-*" (e.g. C1, C_h; max total length cannot exceed 3 characters)'}, 'values': {'type': 'object', 'additionalProperties': False, 'properties': {'v1': {'description': '1st lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'v2': {'description': '2nd lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'v3': {'description': '3rd lattice vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/ions', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ions schema', 'type': 'object', 'additionalProperties': False, 'oneOf': [{'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['bfgs', 'damp', 'fire'], 'default': 'bfgs', 'description': "CASE: calculation == 'relax'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['verlet', 'langevin', 'langevin-smc'], 'default': 'verlet', 'description': "CASE: calculation == 'md'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['bfgs', 'damp'], 'default': 'bfgs', 'description': "CASE: calculation == 'vc-relax'"}}}, {'type': 'object', 'additionalProperties': False, 'properties': {'ion_dynamics': {'type': 'string', 'enum': ['beeman'], 'default': 'beeman', 'description': "CASE: calculation == 'vc-md'"}}}], 'properties': {'ion_positions': {'type': 'string', 'enum': ['default', 'from_input'], 'default': 'default'}, 'ion_velocities': {'type': 'string', 'enum': ['default', 'from_input'], 'default': 'default'}, 'pot_extrapolation': {'type': 'string', 'enum': ['none', 'atomic', 'first_order', 'second_order'], 'default': 'atomic', 'description': 'Used to extrapolate the potential from preceding ionic steps.'}, 'wfc_extrapolation': {'type': 'string', 'enum': ['none', 'first_order', 'second_order'], 'default': 'none', 'description': 'Used to extrapolate the wavefunctions from preceding ionic steps.'}, 'remove_rigid_rot': {'type': 'boolean', 'default': False, 'description': 'This keyword is useful when simulating the dynamics and/or the thermodynamics of an isolated system. If set to true the total torque of the internal forces is set to zero by adding new forces that compensate the spurious interaction with the periodic images. This allows for the use of smaller supercells.'}, 'ion_temperature': {'type': 'string', 'enum': ['rescaling', 'rescale-v', 'rescale-T', 'reduce-T', 'berendsen', 'andersen', 'svr', 'initial', 'not_controlled'], 'default': 'not_controlled'}, 'tempw': {'type': 'number', 'description': 'Starting temperature (Kelvin) in MD runs target temperature for most thermostats.', 'default': 300}, 'tolp': {'type': 'number', 'description': 'Tolerance for velocity rescaling. Velocities are rescaled if the run-averaged and target temperature differ more than tolp.', 'default': 100}, 'delta_t': {'type': 'number', 'default': 1}, 'nraise': {'type': 'integer', 'default': 1}, 'refold_pos': {'type': 'boolean', 'default': False, 'description': 'This keyword applies only in the case of molecular dynamics or damped dynamics. If true the ions are refolded at each step into the supercell.'}, 'upscale': {'type': 'number', 'description': 'Max reduction factor for conv_thr during structural optimization conv_thr is automatically reduced when the relaxation approaches convergence so that forces are still accurate, but conv_thr will not be reduced to less that conv_thr / upscale.', 'default': 100}, 'bfgs_ndim': {'type': 'integer', 'default': 1, 'description': 'Number of old forces and displacements vectors used in the PULAY mixing of the residual vectors obtained on the basis of the inverse hessian matrix given by the BFGS algorithm.'}, 'trust_radius_max': {'type': 'number', 'description': 'Maximum ionic displacement in the structural relaxation. (bfgs only)', 'default': 0.8}, 'trust_radius_min': {'type': 'number', 'description': 'Minimum ionic displacement in the structural relaxation BFGS is reset when trust_radius < trust_radius_min. (bfgs only)', 'default': 0.001}, 'trust_radius_ini': {'type': 'number', 'description': 'Initial ionic displacement in the structural relaxation. (bfgs only)', 'default': 0.5}, 'w_1': {'type': 'number', 'default': 0.01}, 'w_2': {'type': 'number', 'description': 'Parameters used in line search based on the Wolfe conditions. (bfgs only)', 'default': 0.5}, 'fire_alpha_init': {'type': 'number', 'description': 'Initial value of the alpha mixing factor in the FIRE minimization scheme; recommended values are between 0.1 and 0.3', 'default': 0.2}, 'fire_falpha': {'type': 'number', 'description': 'Scaling of the alpha mixing parameter for steps with P > 0;', 'default': 0.99}, 'fire_nmin': {'type': 'integer', 'default': 5, 'description': 'Minimum number of steps with P > 0 before increase of dt'}, 'fire_f_inc': {'type': 'number', 'description': 'Factor for increasing dt', 'default': 1.1}, 'fire_f_dec': {'type': 'number', 'description': 'Factor for decreasing dt', 'default': 0.5}, 'fire_dtmax': {'type': 'number', 'description': 'Determines the maximum value of dt in the FIRE minimization; dtmax = fire_dtmax*dt', 'default': 10}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/k-points', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'k points schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['tpiba', 'automatic', 'crystal', 'gamma', 'tpiba_b', 'crystal_b', 'tpiba_c', 'crystal_c']}, 'values': {'oneOf': [{'type': 'array', 'description': 'K_POINTS tpiba | crystal | tpiba_b | crystal_b | tpiba_c | crystal_c', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'nks': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'xk_x': {'type': 'number'}, 'xk_y': {'type': 'number'}, 'xk_z': {'type': 'number'}, 'wk': {'type': 'number'}}}}, {'type': 'object', 'description': 'K_POINTS automatic', 'additionalProperties': False, 'properties': {'nk1': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'nk2': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'nk3': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk1': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk2': {'type': 'integer', 'description': 'Number of supplied special k-points.'}, 'sk3': {'type': 'integer', 'description': 'Number of supplied special k-points.'}}}, {'type': 'null', 'description': 'K_POINTS gamma'}]}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/atomic-positions', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic positions schema', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1493', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['alat', 'bohr', 'angstrom', 'crystal', 'crystal_sg'], 'default': 'alat'}, 'values': {'type': 'array', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'X': {'type': 'string', 'description': 'label of the atom as specified in ATOMIC_SPECIES'}, 'x': {'type': 'number', 'description': 'atomic positions'}, 'y': {'type': 'number', 'description': 'atomic positions'}, 'z': {'type': 'number', 'description': 'atomic positions'}, 'if_pos(1)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}, 'if_pos(2)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}, 'if_pos(3)': {'default': 1, 'description': 'component i of the force for this atom is multiplied by if_pos(i), which must be either 0 or 1.  Used to keep selected atoms and/or selected components fixed in MD dynamics or structural optimization run.', '$id': 'core/primitive/integer-one-or-zero', 'title': 'integer one or zero', 'type': 'integer', 'minimum': 0, 'maximum': 1}}, 'required': ['x', 'y', 'z']}}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/hubbard', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'hubbard schema', 'type': 'object', 'additionalProperties': False, 'properties': {'card_option': {'type': 'string', 'enum': ['atomic', 'ortho-atomic', 'norm-atomic', 'wf', 'pseudo']}, 'values': {'oneOf': [{'description': 'IF DFT+U', 'type': 'array', 'items': {'oneOf': [{'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1764', 'additionalProperties': False, 'properties': {'U': {'type': 'string', 'enum': ['U'], 'description': 'string constant "U"; indicates the specs for the U parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'u_val': {'type': 'number', 'description': 'value of the U parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1784', 'additionalProperties': False, 'properties': {'J0': {'type': 'string', 'enum': ['J0'], 'description': 'string constant "J0"; indicates the specs for the J0 parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'j0_val': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}]}}, {'description': 'IF DFT+U+J', 'type': 'array', 'items': {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1817', 'additionalProperties': False, 'properties': {'paramType': {'type': 'string', 'enum': ['U', 'J', 'B', 'E2', 'E3'], 'description': 'character describing the type of Hubbard parameter allowed values: U, J and either B (for d-orbitals) or E2 and E3 (for f-orbitals)'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'paramValue': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}}, {'description': 'IF DFT+U+V', 'type': 'array', 'items': {'oneOf': [{'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1847', 'additionalProperties': False, 'properties': {'U': {'type': 'string', 'enum': ['U'], 'description': 'string constant "U"; indicates the specs for the U parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'u_val': {'type': 'number', 'description': 'value of the U parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1865', 'additionalProperties': False, 'properties': {'J0': {'type': 'string', 'enum': ['J0'], 'description': 'string constant "J0"; indicates the specs for the J0 parameter will be given'}, 'label': {'type': 'string', 'description': 'label of the atom (as defined in ATOMIC_SPECIES)'}, 'manifold': {'type': 'string', 'description': 'specs of the manifold (e.g., 3d, 2p...)'}, 'j0_val': {'type': 'number', 'description': 'value of the J0 parameter (in eV)'}}}, {'type': 'object', 'description': 'https://www.quantum-espresso.org/Doc/INPUT_PW.html#idm1884', 'additionalProperties': False, 'properties': {'V': {'type': 'string', 'enum': ['V'], 'description': 'string constant "V"; indicates the specs for the V parameter will be given'}, 'label(I)': {'type': 'string', 'description': 'label of the atom I (as defined in ATOMIC_SPECIES)'}, 'manifold(I)': {'type': 'string', 'description': 'specs of the manifold for atom I (e.g., 3d, 2p...)'}, 'label(J)': {'type': 'string', 'description': 'label of the atom J (as defined in ATOMIC_SPECIES)'}, 'manifold(J)': {'type': 'string', 'description': 'specs of the manifold for atom J (e.g., 3d, 2p...)'}, 'I': {'type': 'integer', 'description': 'index of the atom I'}, 'J': {'type': 'integer', 'description': 'index of the atom J'}, 'v_val(I,J)': {'type': 'number', 'description': 'value of the V parameter for the atom pair I,J (in eV)'}}}]}}]}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/atomic-species', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic species schema', 'type': 'object', 'additionalProperties': False, 'properties': {'values': {'type': 'array', 'items': {'type': 'object', 'additionalProperties': False, 'properties': {'X': {'type': 'string', 'description': 'label of the atom. Acceptable syntax: chemical symbol X (1 or 2 characters, case-insensitive) or chemical symbol plus a number or a letter, as in "Xn" (e.g. Fe1) or "X_*" or "X-*" (e.g. C1, C_h; max total length cannot exceed 3 characters)'}, 'Mass_X': {'type': 'number', 'description': 'mass of the atomic species [amu: mass of C = 12]. Used only when performing Molecular Dynamics run or structural optimization runs using Damped MD. Not actually used in all other cases (but stored in data files, so phonon calculations will use these values unless other values are provided)'}, 'PseudoPot_X': {'type': 'string', 'description': 'PseudoPot_X'}}}}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/control', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'control schema', 'type': 'object', 'additionalProperties': False, 'properties': {'calculation': {'description': 'A string describing the task to be performed', 'type': 'string', 'enum': ['scf', 'nscf', 'bands', 'relax', 'md', 'vc-relax', 'vc-md'], 'default': 'scf'}, 'title': {'description': 'reprinted on output.', 'type': 'string', 'default': ''}, 'verbosity': {'description': "Currently two verbosity levels are implemented: high, low. 'debug' and 'medium' have the same effect as 'high'; 'default' and 'minimal' as 'low'", 'type': 'string', 'enum': ['high', 'low', 'debug', 'medium', 'minimal', 'default'], 'default': 'low'}, 'restart_mode': {'type': 'string', 'enum': ['from_scratch', 'restart'], 'default': 'from_scratch'}, 'wf_collect': {'type': 'boolean', 'description': 'OBSOLETE - NO LONGER IMPLEMENTED'}, 'nstep': {'description': 'Default: 1 if calculation == \'scf\', \'nscf\', \'bands\'; 50 for the other cases; Number of molecular-dynamics or structural optimization steps performed in this run. If set to 0, the code performs a quick "dry run", stopping just after initialization. This is useful to check for input correctness and to have the summary printed. NOTE: in MD calculations, the code will perform "nstep" steps even if restarting from a previously interrupted calculation.', 'type': 'number', 'default': 1}, 'iprint': {'description': 'band energies are written every iprint iterations', 'type': 'number'}, 'tstress': {'type': 'boolean', 'default': False, 'description': "calculate stress. It is set to .TRUE. automatically if calculation == 'vc-md' or 'vc-relax'"}, 'tprnfor': {'type': 'boolean', 'description': "calculate forces. It is set to .TRUE. automatically if calculation == 'relax','md','vc-md'"}, 'dt': {'type': 'number', 'description': 'time step for molecular dynamics, in Rydberg atomic units (1 a.u.=4.8378 * 10^-17 s : beware, the CP code uses Hartree atomic units, half that much!!!)', 'default': 20}, 'outdir': {'type': 'string', 'description': 'input, temporary, output files are found in this directory, see also wfcdir'}, 'wfcdir': {'type': 'string', 'description': 'This directory specifies where to store files generated by each processor (*.wfc{N}, *.igk{N}, etc.). Useful for machines without a parallel file system: set wfcdir to a local file system, while outdir should be a parallel or network file system, visible to all processors. Beware: in order to restart from interrupted runs, or to perform further calculations using the produced data files, you may need to copy files to outdir. Works only for pw.x.'}, 'prefix': {'type': 'string', 'description': 'prepended to input/output filenames: prefix.wfc, prefix.rho, etc.', 'default': 'pwscf'}, 'lkpoint_dir': {'type': 'boolean', 'description': 'OBSOLETE - NO LONGER IMPLEMENTED'}, 'max_seconds': {'type': 'number', 'default': 10000000, 'description': 'Jobs stops after max_seconds CPU time. Use this option in conjunction with option restart_mode if you need to split a job too long to complete into shorter jobs that fit into your batch queues.'}, 'etot_conv_thr': {'type': 'number', 'default': 0.0001, 'description': 'Convergence threshold on total energy (a.u) for ionic minimization: the convergence criterion is satisfied when the total energy changes less than etot_conv_thr between two consecutive scf steps. Note that etot_conv_thr is extensive, like the total energy. See also forc_conv_thr - both criteria must be satisfied'}, 'forc_conv_thr': {'type': 'number', 'default': 0.001, 'description': 'Convergence threshold on forces (a.u) for ionic minimization: the convergence criterion is satisfied when all components of all forces are smaller than forc_conv_thr. See also etot_conv_thr - both criteria must be satisfied'}, 'disk_io': {'type': 'string', 'enum': ['high', 'medium', 'low', 'nowf', 'none'], 'description': 'Specifies the amount of disk I/O activity: (only for binary files and xml data file in data directory; other files printed at each molecular dynamics / structural optimization step are not controlled by this option )'}, 'pseudo_dir': {'type': 'string', 'description': "directory containing pseudopotential files. Default: value of the $ESPRESSO_PSEUDO environment variable if set; '$HOME/espresso/pseudo/' otherwise"}, 'tefield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. a saw-like potential simulating an electric field is added to the bare ionic potential. See variables edir, eamp, emaxpos, eopreg for the form and size of the added potential.'}, 'dipfield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. and tefield==.TRUE. a dipole correction is also added to the bare ionic potential - implements the recipe of L. Bengtsson, PRB 59, 12301 (1999). See variables edir, emaxpos, eopreg for the form of the correction. Must be used ONLY in a slab geometry, for surface calculations, with the discontinuity FALLING IN THE EMPTY SPACE.'}, 'lelfield': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. a homogeneous finite electric field described through the modern theory of the polarization is applied. This is different from tefield == .true. !'}, 'nberrycyc': {'type': 'integer', 'default': 1, 'description': 'In the case of a finite electric field  ( lelfield == .TRUE. ) it defines the number of iterations for converging the wavefunctions in the electric field Hamiltonian, for each external iteration on the charge density'}, 'lorbm': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform orbital magnetization calculation.'}, 'lberry': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a Berry phase calculation. See the header of PW/src/bp_c_phase.f90 for documentation'}, 'gdir': {'type': 'number', 'description': 'For Berry phase calculation: direction of the k-point strings in reciprocal space. Allowed values: 1, 2, 3 1=first, 2=second, 3=third reciprocal lattice vector For calculations with finite electric fields (lelfield==.true.) "gdir" is the direction of the field.'}, 'nppstr': {'type': 'number', 'description': 'For Berry phase calculation: number of k-points to be calculated along each symmetry-reduced string. The same for calculation with finite electric fields (lelfield==.true.).'}, 'gate': {'type': 'boolean', 'default': False, 'description': 'In the case of charged cells (tot_charge .ne. 0) setting gate = .TRUE. represents the counter charge (i.e. -tot_charge) not by a homogeneous background charge but with a charged plate, which is placed at zgate (see below). Details of the gate potential can be found in T. Brumme, M. Calandra, F. Mauri; PRB 89, 245406 (2014). Note, that in systems which are not symmetric with respect to the plate, one needs to enable the dipole correction! (dipfield=.true.). Currently, symmetry can be used with gate=.true. but carefully check that no symmetry is included which maps z to -z even if in principle one could still use them for symmetric systems (i.e. no dipole correction). For nosym=.false. verbosity is set to \'high\'. Note: this option was called "monopole" in v6.0 and 6.1 of pw.x'}, 'twochem': {'type': 'boolean', 'default': False, 'description': 'IF .TRUE. , a two chemical potential calculation for the simulation of photoexcited systems is performed, constraining a fraction of the electrons in the conduction manifold.'}, 'lfcp': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a constant bias potential (constant-mu) calculation for a system with ESM method. See the header of PW/src/fcp_module.f90 for documentation. To perform the calculation, you must set a namelist FCP.'}, 'trism': {'type': 'boolean', 'default': False, 'description': "If .TRUE. perform a 3D-RISM-SCF calculation [for details see H.Sato et al., JCP 112, 9463 (2000), doi:10.1063/1.481564]. The solvent's distributions are calculated by 3D-RISM, though solute is treated as SCF. The charge density and the atomic positions are optimized, simultaneously with the solvents. To perform the calculation, you must set a namelist RISM and a card SOLVENTS. If assume_isolated = 'esm' and esm_bc = 'bc1', Laue-RISM is calculated instead of 3D-RISM and coupled with ESM method (i.e. ESM-RISM). [for details see S.Nishihara and M.Otani, PRB 96, 115429 (2017)]. The default of mixing_beta is 0.2 for both 3D-RISM and Laue-RISM. For structural relaxation with BFGS, ignore_wolfe is always .TRUE. ."}}}, {'$id': '3pse/file/applications/espresso/7.2/pw.x/system', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'system schema', 'type': 'object', 'additionalProperties': False, 'anyOf': [{'properties': {'celldm': {'type': 'array', 'items': {'type': 'number'}, 'minLength': 6, 'maxLength': 6}}}, {'properties': {'A': {'type': 'number'}, 'B': {'type': 'number'}, 'C': {'type': 'number'}, 'cosAB': {'type': 'number'}, 'cosAC': {'type': 'number'}, 'cosBC': {'type': 'number'}}}], 'properties': {'ibrav': {'type': 'integer'}, 'nat': {'type': 'integer', 'description': 'number of atoms in the unit cell (ALL atoms, except if space_group is set, in which case, INEQUIVALENT atoms)'}, 'ntyp': {'type': 'integer', 'description': 'number of types of atoms in the unit cell'}, 'nbnd': {'type': 'integer', 'description': 'Default: for an insulator, nbnd = number of valence bands (nbnd = # of electrons /2); for a metal, 20% more (minimum 4 more)'}, 'nbnd_cond': {'type': 'integer', 'description': 'Default: nbnd_cond = nbnd - # of electrons / 2 in the collinear case; nbnd_cond = nbnd - # of electrons in the noncollinear case.'}, 'tot_charge': {'type': 'number', 'default': 0}, 'starting_charge': {'type': 'number', 'default': 0, 'description': "starting charge on atomic type 'i', to create starting potential with startingpot = 'atomic'."}, 'tot_magnetization': {'type': 'number', 'default': -10000, 'description': 'Total majority spin charge - minority spin charge. Used to impose a specific total electronic magnetization. If unspecified then tot_magnetization variable is ignored and the amount of electronic magnetization is determined during the self-consistent cycle.'}, 'starting_magnetization': {'type': 'array', 'items': {'type': 'number', 'default': 0, 'minimum': -1, 'maximum': 1}}, 'ecutwfc': {'type': 'number', 'description': 'kinetic energy cutoff (Ry) for wavefunctions'}, 'ecutrho': {'type': 'number', 'description': 'Kinetic energy cutoff (Ry) for charge density and potential For norm-conserving pseudopotential you should stick to the default value, you can reduce it by a little but it will introduce noise especially on forces and stress. Default: 4 * ecutwfc'}, 'ecutfock': {'type': 'number', 'description': 'Kinetic energy cutoff (Ry) for the exact exchange operator in EXX type calculations. By default this is the same as ecutrho but in some EXX calculations, a significant speed-up can be obtained by reducing ecutfock, at the expense of some loss in accuracy. Must be .gt. ecutwfc. Not implemented for stress calculation and for US-PP and PAW pseudopotentials.'}, 'nr1': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr2': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr3': {'type': 'integer', 'description': 'Three-dimensional FFT mesh (hard grid) for charge density (and scf potential). If not specified the grid is calculated based on the cutoff for charge density (see also ecutrho)'}, 'nr1s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nr2s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nr3s': {'type': 'integer', 'description': 'Three-dimensional mesh for wavefunction FFT and for the smooth part of charge density ( smooth grid ). Coincides with nr1, nr2, nr3 if ecutrho = 4 * ecutwfc ( default )'}, 'nosym': {'type': 'boolean', 'default': False}, 'nosym_evc': {'type': 'boolean', 'default': False}, 'noinv': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) disable the usage of k => -k symmetry (time reversal) in k-point generation'}, 'no_t_rev': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) disable the usage of magnetic symmetry operations that consist in a rotation + time reversal.'}, 'force_symmorphic': {'type': 'boolean', 'default': False, 'description': 'if (.TRUE.) force the symmetry group to be symmorphic by disabling symmetry operations having an associated fractionary translation'}, 'use_all_frac': {'type': 'boolean', 'default': False}, 'occupations': {'type': 'string', 'enum': ['smearing', 'tetrahedra', 'tetrahedra_lin', 'tetrahedra_opt', 'fixed', 'from_input']}, 'one_atom_occupations': {'type': 'boolean', 'default': False}, 'starting_spin_angle': {'type': 'boolean', 'default': False}, 'degauss_cond': {'type': 'number', 'default': 0, 'description': 'value of the gaussian spreading (Ry) for brillouin-zone integration in the conduction manifold in a two-chemical potential calculation (twochem=.true.).'}, 'nelec_cond': {'type': 'number', 'default': 0, 'description': 'Number of electrons placed in the conduction manifold in a two-chemical potential calculation (twochem=.true.). Of the total # of electrons nelec, nelec-nelec_cond will occupy the valence manifold and nelec_cond will be constrained in the conduction manifold.'}, 'degauss': {'type': 'number', 'default': 0, 'description': 'value of the gaussian spreading (Ry) for brillouin-zone integration in metals.'}, 'smearing': {'type': 'string', 'enum': ['gaussian', 'gauss', 'methfessel-paxton', 'm-p', 'mp', 'marzari-vanderbilt', 'cold', 'm-v', 'mv', 'fermi-dirac', 'f-d', 'fd'], 'default': 'gaussian'}, 'nspin': {'type': 'integer', 'default': 1}, 'sic_gamma': {'type': 'number', 'default': 0, 'description': 'Strength of the gammaDFT potential.'}, 'pol_type': {'type': 'string', 'enum': ['e', 'h'], 'description': 'Type of polaron in gammaDFT.'}, 'sic_energy': {'type': 'boolean', 'default': False, 'description': 'Enable the calculation of the total energy in gammaDFT. When .true., a preliminary calculation is performed to calculate the electron density in the absence of the polaron. When .false., the total energy printed in output should not be considered. For structural relaxations, it is recommended to use .false. to avoid doubling the computational cost.'}, 'sci_vb': {'type': 'number', 'default': 0, 'description': 'Valence band shift (in eV) through self-consistent scissor operator. When performing gammaDFT calculations of polarons, the polaron level is not shifted.'}, 'sci_cb': {'type': 'number', 'default': 0, 'description': 'Conduction band band shift (in eV) through self-consistent scissor operator. When performing gammaDFT calculations of polarons, the polaron level is not shifted.'}, 'noncolin': {'type': 'boolean', 'default': False, 'description': 'if .true. the program will perform a noncollinear calculation.'}, 'ecfixed': {'type': 'number', 'default': 0}, 'qcutz': {'type': 'number', 'default': 0}, 'q2sigma': {'type': 'number', 'default': 0.1}, 'input_dft': {'type': 'string', 'description': "Exchange-correlation functional: eg 'PBE', 'BLYP' etc See Modules/funct.f90 for allowed values. Overrides the value read from pseudopotential files. Use with care and if you know what you are doing!"}, 'ace': {'type': 'boolean', 'default': True, 'description': 'Use Adaptively Compressed Exchange operator as in Lin Lin, J. Chem. Theory Comput. 2016, 12, 2242--2249, doi:10.1021/acs.jctc.6b00092'}, 'exx_fraction': {'type': 'number', 'description': "Fraction of EXX for hybrid functional calculations. In the case of input_dft='PBE0', the default value is 0.25, while for input_dft='B3LYP' the exx_fraction default value is 0.20."}, 'screening_parameter': {'type': 'number', 'default': 0.106, 'description': 'screening_parameter for HSE like hybrid functionals.'}, 'exxdiv_treatment': {'type': 'string', 'default': 'gygi-baldereschi', 'enum': ['gygi-baldereschi', 'vcut_spherical', 'vcut_ws', 'none']}, 'x_gamma_extrapolation': {'type': 'boolean', 'default': True, 'description': 'Specific for EXX. If .true., extrapolate the G=0 term of the potential'}, 'ecutvcut': {'type': 'number', 'description': 'Reciprocal space cutoff for correcting Coulomb potential divergencies at small q vectors.'}, 'nqx1': {'type': 'integer', 'default': 0, 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'nqx2': {'type': 'integer', 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'nqx3': {'type': 'integer', 'description': 'Three-dimensional mesh for q (k1-k2) sampling of the Fock operator (EXX). Can be smaller than the number of k-points.'}, 'localization_thr': {'type': 'number', 'default': 0, 'description': 'Overlap threshold over which the exchange integral over a pair of localized orbitals is included in the evaluation of EXX operator. Any value greater than 0.0 triggers the SCDM localization and the evaluation on EXX using the localized orbitals. Very small value of the threshold should yield the same result as the default EXX evaluation'}, 'Hubbard_occ': {'type': 'array', 'items': {'type': 'array', 'items': [{'type': 'integer', 'minimum': 1}, {'type': 'integer', 'minimum': 1, 'maximum': 3}, {'type': 'number'}]}}, 'Hubbard_alpha': {'type': 'array', 'items': {'type': 'number', 'default': 0}}, 'Hubbard_beta': {'type': 'array', 'items': {'type': 'number', 'default': 0}}, 'starting_ns_eigenvalue': {'type': 'array', 'items': {'type': 'array', 'items': {'type': 'array', 'items': {'type': 'number'}}}}, 'dmft': {'type': 'boolean', 'default': False, 'description': 'If true, nscf calculation will exit in restart mode, scf calculation will restart from there if DMFT updates are provided as hdf5 archive. Scf calculation should be used only with electron_maxstep = 1.'}, 'dmft_prefix': {'type': 'string', 'description': 'prepended to hdf5 archive: dmft_prefix.h5'}, 'ensemble_energies': {'type': 'boolean', 'default': False, 'description': 'If ensemble_energies = .true., an ensemble of xc energies is calculated non-selfconsistently for perturbed exchange-enhancement factors and LDA vs. PBE correlation ratios after each converged electronic ground state calculation.'}, 'edir': {'type': 'integer', 'description': 'The direction of the electric field or dipole correction is parallel to the bg(:,edir) reciprocal lattice vector, so the potential is constant in planes defined by FFT grid points; edir = 1, 2 or 3. Used only if tefield is .TRUE.'}, 'emaxpos': {'type': 'number', 'default': 0.5, 'description': 'Position of the maximum of the saw-like potential along crystal axis edir, within the  unit cell (see below), 0 < emaxpos < 1 Used only if tefield is .TRUE.'}, 'eopreg': {'type': 'number', 'default': 0.1, 'description': 'Zone in the unit cell where the saw-like potential decreases. ( see below, 0 < eopreg < 1 ). Used only if tefield is .TRUE.'}, 'eamp': {'type': 'number', 'default': 0.001}, 'angle1': {'type': 'array', 'items': {'type': 'number'}, 'maxItems': 1, 'minItems': 1, 'description': 'The angle expressed in degrees between the initial magnetization and the z-axis. For noncollinear calculations only; index i runs over the atom types.'}, 'angle2': {'type': 'array', 'items': {'type': 'number'}, 'maxItems': 1, 'minItems': 1, 'description': 'The angle expressed in degrees between the projection of the initial magnetization on x-y plane and the x-axis. For noncollinear calculations only.'}, 'lforcet': {'type': 'boolean', 'description': 'When starting a non collinear calculation using an existing density file from a collinear lsda calculation assumes previous density points in z direction and rotates it in the direction described by angle1 and angle2 variables for atomic type 1'}, 'constrained_magnetization': {'type': 'string', 'enum': ['none', 'total', 'atomic', 'total direction', 'atomic direction'], 'default': 'none', 'description': 'Used to perform constrained calculations in magnetic systems.'}, 'fixed_magnetization': {'type': 'array', 'items': {'type': 'number', 'default': 0}, 'maxItems': 3, 'minItems': 3}, 'lambda': {'type': 'number', 'default': 1, 'description': 'parameter used for constrained_magnetization calculations N.B.: if the scf calculation does not converge, try to reduce lambda to obtain convergence, then restart the run with a larger lambda'}, 'report': {'type': 'integer', 'default': -1, 'description': 'determines when atomic magnetic moments are printed on output'}, 'lspinorb': {'type': 'boolean', 'description': 'if .TRUE. the noncollinear code can use a pseudopotential with spin-orbit.'}, 'assume_isolated': {'type': 'string', 'enum': ['none', 'makov-payne', 'm-p', 'mp', 'martyna-tuckerman', 'm-t', 'mt', 'esm', '2D'], 'default': 'none', 'definition': 'Used to perform calculation assuming the system to be isolated (a molecule or a cluster in a 3D supercell)'}, 'esm_bc': {'type': 'string', 'enum': ['pbc', 'bc1', 'bc2', 'bc3'], 'default': 'pbc', 'definition': "If assume_isolated = 'esm', determines the boundary conditions used for either side of the slab."}, 'esm_w': {'type': 'number', 'default': 0, 'description': "If assume_isolated = 'esm', determines the position offset [in a.u.] of the start of the effective screening region, measured relative to the cell edge. (ESM region begins at z = +/- [L_z/2 + esm_w] )."}, 'esm_efield': {'type': 'number', 'default': 0, 'description': "If assume_isolated = 'esm' and esm_bc = 'bc2', gives the magnitude of the electric field [Ry/a.u.] to be applied between semi-infinite ESM electrodes."}, 'esm_nfit': {'type': 'integer', 'default': 4, 'description': "If assume_isolated = 'esm', gives the number of z-grid points for the polynomial fit along the cell edge."}, 'lgcscf': {'type': 'boolean', 'default': False, 'description': 'If .TRUE. perform a constant bias potential (constant-mu) calculation with Grand-Canonical SCF.'}, 'gcscf_mu': {'type': 'number', 'description': 'The target Fermi energy (eV) of GC-SCF. One can start with appropriate total charge of the system by giving tot_charge'}, 'gcscf_conv_thr': {'type': 'number', 'default': 0.01, 'description': 'Convergence threshold of Fermi energy (eV) for GC-SCF.'}, 'gcscf_beta': {'type': 'number', 'default': 0.05, 'description': 'Mixing factor for GC-SCF. Larger values are recommended, if systems with small DOS on Fermi surface as graphite.'}, 'vdw_corr': {'type': 'string', 'enum': ['grimme-d2', 'Grimme-D2', 'DFT-D', 'dft-d', 'grimme-d3', 'Grimme-D3', 'DFT-D3', 'dft-d3', 'TS', 'ts', 'ts-vdw', 'ts-vdW', 'tkatchenko-scheffler', 'MBD', 'mbd', 'many-body-dispersion', 'mbd_vdw', 'XDM', 'xdm'], 'default': 'none', 'description': 'Type of Van der Waals correction'}, 'london': {'type': 'boolean', 'default': False, 'description': "OBSOLESCENT, same as vdw_corr='DFT-D'"}, 'london_s6': {'type': 'number', 'default': 0.75, 'description': 'global scaling parameter for DFT-D. Default is good for PBE.'}, 'london_c6': {'type': 'number', 'description': 'atomic C6 coefficient of each atom type'}, 'london_rvdw': {'type': 'number', 'description': 'atomic vdw radii of each atom type'}, 'london_rcut': {'type': 'number', 'default': 200, 'description': 'cutoff radius (a.u.) for dispersion interactions'}, 'dftd3_version': {'type': 'integer', 'default': 3, 'minimum': 2, 'maximum': 6, 'description': 'Version of Grimme implementation of Grimme-D3'}, 'dftd3_threebody': {'type': 'boolean', 'default': True, 'description': 'Turn three-body terms in Grimme-D3 on. If .false. two-body contributions only are computed, using two-body parameters of Grimme-D3. If dftd3_version=2, three-body contribution is always disabled.'}, 'ts_vdw_econv_thr': {'type': 'number', 'default': 1e-06, 'description': 'Optional: controls the convergence of the vdW energy (and forces). The default value is a safe choice, likely too safe, but you do not gain much in increasing it'}, 'ts_vdw_isolated': {'type': 'boolean', 'default': False, 'description': 'Optional: set it to .TRUE. when computing the Tkatchenko-Scheffler vdW energy or the Many-Body dispersion (MBD) energy for an isolated (non-periodic) system.'}, 'xdm': {'type': 'boolean', 'default': False, 'description': "OBSOLESCENT, same as vdw_corr='xdm'"}, 'xdm_a1': {'type': 'number', 'default': 0.6836, 'description': 'Damping function parameter a1 (adimensional)'}, 'xdm_a2': {'type': 'number', 'default': 1.5045, 'description': 'Damping function parameter a2 (angstrom)'}, 'space_group': {'type': 'integer', 'default': 0, 'description': 'The number of the space group of the crystal, as given in the International Tables of Crystallography A (ITA)'}, 'uniqueb': {'type': 'boolean', 'default': False, 'description': 'Used only for monoclinic lattices'}, 'origin_choice': {'type': 'integer', 'default': 1, 'description': 'Used only for space groups that in the ITA allow the use of two different origins'}, 'rhombohedral': {'type': 'boolean', 'default': True, 'description': 'Used only for rhombohedral space groups.'}, 'zgate': {'type': 'number', 'default': 0.5, 'description': 'used only if gate = .TRUE.'}, 'relaxz': {'type': 'boolean', 'default': False, 'description': 'used only if gate = .TRUE.'}, 'block': {'type': 'boolean', 'default': False, 'description': 'used only if gate = .TRUE.'}, 'block_1': {'type': 'number', 'default': 0.45, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'block_2': {'type': 'number', 'default': 0.55, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'block_height': {'type': 'number', 'default': 0.1, 'description': 'used only if gate = .TRUE. and block = .TRUE.'}, 'nextffield': {'type': 'integer', 'default': 0, 'description': 'Number of activated external ionic force fields.'}}, 'required': ['ibrav', 'nat', 'ntyp', 'ecutwfc']}, {'$id': 'methods-directory/physical/smearing', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method smearing', 'description': 'Approximating Heaviside step function with smooth function', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/smearing', 'title': 'Smearing methods category schema', 'description': 'Approximating Heaviside step function with smooth function', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['smearing']}, 'subtype': {'enum': ['gaussian', 'marzari-vanderbilt', 'methfessel-paxton', 'fermi-dirac']}}}}, 'required': ['categories']}, {'$id': 'methods-directory/physical/psp', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method pseudopotential', 'description': 'Core-valence separation by means of pseudopotentials (effective potential)', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/psp', 'description': 'Pseudopotential category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['psp']}, 'subtype': {'enum': ['us', 'nc', 'paw', 'coulomb']}}}, 'data': {'type': 'array', 'items': {'description': 'Specific data based on pseudopotential file.', '$id': 'methods-directory/physical/psp/file', 'title': 'Pseudopotential file', 'properties': {'slug': {'enum': ['pseudopotential']}, 'data': {'description': 'Specific data based on pseudopotential file.', '$id': 'methods-directory/physical/psp/file-data-item', 'title': 'File data item', 'properties': {'element': {'type': 'string', 'description': 'chemical element'}, 'hash': {'type': 'string', 'description': 'MD5 hash of the pseudopotential file'}, 'type': {'type': 'string', 'description': 'type of pseudization', 'enum': ['us', 'nc', 'paw', 'coulomb']}, 'source': {'type': 'string', 'description': 'explains where this came from'}, 'version': {'type': 'string', 'description': 'explains the version of where this came from'}, 'exchangeCorrelation': {'type': 'object', 'properties': {'approximation': {'description': 'DFT approximation', 'type': 'string'}, 'functional': {'description': 'Exchange correlation functional', 'type': 'string'}, 'path': {'description': 'TODO: Use regex once schema draft version has been updated', '$id': 'core/reusable/category-path', 'title': 'category path schema', 'type': 'string'}}}, 'valenceConfiguration': {'type': 'array', 'description': 'contains pseudo orbital information, including orbital names and occupations', 'items': {'$id': 'core/reusable/atomic-orbital', 'title': 'atomic orbital schema', 'type': 'object', 'properties': {'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalIndex': {'type': 'integer', 'minimum': 1}, 'principalNumber': {'type': 'integer', 'minimum': 1, 'maximum': 7}, 'angularMomentum': {'type': 'integer', 'minimum': 0, 'maximum': 3}, 'occupation': {'type': 'number', 'description': 'Shell occupation', 'minimum': 0, 'maximum': 14}}}}, 'path': {'type': 'string', 'description': 'location of the pseudopotential file on filesystem'}, 'apps': {'type': 'array', 'description': 'The names of the simulation engines that can use this pseudopotential, e.g. espresso', 'items': {'type': 'string'}}, 'filename': {'type': 'string', 'description': 'filename of pseudopotential file on filesystem'}, 'name': {'type': 'string', 'description': 'name of the data category', 'enum': ['pseudopotential']}}, 'required': ['element', 'type', 'exchangeCorrelation', 'source', 'path', 'apps']}, 'source': {'type': 'object', 'description': 'TODO: remove in the future', 'properties': {'info': {'type': 'object'}, 'type': {'type': 'string'}}}}}}}, 'required': ['categories']}, {'$id': 'methods-directory/physical/pw', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method plane wave', 'description': 'Approximating the electronic wave function with a plane wave basis', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/pw', 'description': 'Plane wave catgeory schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['pw']}}}}, 'required': ['categories']}, {'$id': 'methods-directory/physical/tetrahedron', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method tetrahedron', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/tetrahedron', 'description': 'Tetrahedron method for Brillouin zone integration category schema ', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['tetrahedron']}, 'subtype': {'enum': ['linear', 'optimized', 'bloechl']}}}}, 'required': ['categories']}, {'$id': 'methods-directory/physical/psp/file', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Pseudopotential file', 'properties': {'slug': {'enum': ['pseudopotential']}, 'data': {'description': 'Specific data based on pseudopotential file.', '$id': 'methods-directory/physical/psp/file-data-item', 'title': 'File data item', 'properties': {'element': {'type': 'string', 'description': 'chemical element'}, 'hash': {'type': 'string', 'description': 'MD5 hash of the pseudopotential file'}, 'type': {'type': 'string', 'description': 'type of pseudization', 'enum': ['us', 'nc', 'paw', 'coulomb']}, 'source': {'type': 'string', 'description': 'explains where this came from'}, 'version': {'type': 'string', 'description': 'explains the version of where this came from'}, 'exchangeCorrelation': {'type': 'object', 'properties': {'approximation': {'description': 'DFT approximation', 'type': 'string'}, 'functional': {'description': 'Exchange correlation functional', 'type': 'string'}, 'path': {'description': 'TODO: Use regex once schema draft version has been updated', '$id': 'core/reusable/category-path', 'title': 'category path schema', 'type': 'string'}}}, 'valenceConfiguration': {'type': 'array', 'description': 'contains pseudo orbital information, including orbital names and occupations', 'items': {'$id': 'core/reusable/atomic-orbital', 'title': 'atomic orbital schema', 'type': 'object', 'properties': {'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalIndex': {'type': 'integer', 'minimum': 1}, 'principalNumber': {'type': 'integer', 'minimum': 1, 'maximum': 7}, 'angularMomentum': {'type': 'integer', 'minimum': 0, 'maximum': 3}, 'occupation': {'type': 'number', 'description': 'Shell occupation', 'minimum': 0, 'maximum': 14}}}}, 'path': {'type': 'string', 'description': 'location of the pseudopotential file on filesystem'}, 'apps': {'type': 'array', 'description': 'The names of the simulation engines that can use this pseudopotential, e.g. espresso', 'items': {'type': 'string'}}, 'filename': {'type': 'string', 'description': 'filename of pseudopotential file on filesystem'}, 'name': {'type': 'string', 'description': 'name of the data category', 'enum': ['pseudopotential']}}, 'required': ['element', 'type', 'exchangeCorrelation', 'source', 'path', 'apps']}, 'source': {'type': 'object', 'description': 'TODO: remove in the future', 'properties': {'info': {'type': 'object'}, 'type': {'type': 'string'}}}}}, {'$id': 'methods-directory/physical/psp/file-data-item', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'File data item', 'properties': {'element': {'type': 'string', 'description': 'chemical element'}, 'hash': {'type': 'string', 'description': 'MD5 hash of the pseudopotential file'}, 'type': {'type': 'string', 'description': 'type of pseudization', 'enum': ['us', 'nc', 'paw', 'coulomb']}, 'source': {'type': 'string', 'description': 'explains where this came from'}, 'version': {'type': 'string', 'description': 'explains the version of where this came from'}, 'exchangeCorrelation': {'type': 'object', 'properties': {'approximation': {'description': 'DFT approximation', 'type': 'string'}, 'functional': {'description': 'Exchange correlation functional', 'type': 'string'}, 'path': {'description': 'TODO: Use regex once schema draft version has been updated', '$id': 'core/reusable/category-path', 'title': 'category path schema', 'type': 'string'}}}, 'valenceConfiguration': {'type': 'array', 'description': 'contains pseudo orbital information, including orbital names and occupations', 'items': {'$id': 'core/reusable/atomic-orbital', 'title': 'atomic orbital schema', 'type': 'object', 'properties': {'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalIndex': {'type': 'integer', 'minimum': 1}, 'principalNumber': {'type': 'integer', 'minimum': 1, 'maximum': 7}, 'angularMomentum': {'type': 'integer', 'minimum': 0, 'maximum': 3}, 'occupation': {'type': 'number', 'description': 'Shell occupation', 'minimum': 0, 'maximum': 14}}}}, 'path': {'type': 'string', 'description': 'location of the pseudopotential file on filesystem'}, 'apps': {'type': 'array', 'description': 'The names of the simulation engines that can use this pseudopotential, e.g. espresso', 'items': {'type': 'string'}}, 'filename': {'type': 'string', 'description': 'filename of pseudopotential file on filesystem'}, 'name': {'type': 'string', 'description': 'name of the data category', 'enum': ['pseudopotential']}}, 'required': ['element', 'type', 'exchangeCorrelation', 'source', 'path', 'apps']}, {'$id': 'methods-directory/physical/ao/dunning', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method ao dunning', 'description': 'Dunning correlation-consistent basis set unit method', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/ao/dunning', 'description': 'Dunning correlation-consistent basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['dunning']}}}, 'parameters': {'allOf': [{'properties': {'basisSlug': {'enum': ['cc-pvdz', 'cc-pvtz', 'cc-pvqz']}}}]}}, 'required': ['categories'], 'definitions': {'ao-basis-dunning': {'properties': {'basisSlug': {'enum': ['cc-pvdz', 'cc-pvtz', 'cc-pvqz']}}}}}, {'$id': 'methods-directory/physical/ao/other', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method ao other', 'description': 'Other (neither Pople nor Dunning) basis set unit method', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/ao/other', 'description': 'Other (neither Pople nor Dunning) basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['other']}}}, 'parameters': {'allOf': [{'properties': {'basisSlug': {'enum': ['sto-3g', 'sto-4g', 'sto-6g', 'def2-svp', 'def2-tzvp', 'def2-qzvp', 'cbs-qb3']}}}]}}, 'required': ['categories'], 'definitions': {'ao-basis-other': {'properties': {'basisSlug': {'enum': ['sto-3g', 'sto-4g', 'sto-6g', 'def2-svp', 'def2-tzvp', 'def2-qzvp', 'cbs-qb3']}}}}}, {'$id': 'methods-directory/physical/ao/pople', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method ao pople', 'description': 'Pople basis set unit method', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/physical/qm/wf/ao/pople', 'description': 'Pople basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['pople']}}}, 'parameters': {'allOf': [{'properties': {'basisSlug': {'enum': ['3-21G', '6-31G', '6-311G']}}}]}}, 'required': ['categories'], 'definitions': {'ao-basis-pople': {'properties': {'basisSlug': {'enum': ['3-21G', '6-31G', '6-311G']}}}}}, {'$id': 'methods-directory/physical/ao/enum-options', 'popleAoBasis': {'enum': ['3-21G', '6-31G', '6-311G']}, 'dunningAoBasis': {'enum': ['cc-pvdz', 'cc-pvtz', 'cc-pvqz']}, 'otherAoBasis': {'enum': ['sto-3g', 'sto-4g', 'sto-6g', 'def2-svp', 'def2-tzvp', 'def2-qzvp', 'cbs-qb3']}}, {'$id': 'methods-directory/legacy/regression', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy method regression', 'allOf': [{'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}], 'properties': {'type': {'enum': ['linear', 'kernel_ridge']}, 'subtype': {'enum': ['least_squares', 'ridge']}, 'precision': {'$id': 'methods-directory/mathematical/regression/precision', 'title': 'regression precision', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'$id': 'methods-directory/mathematical/regression/precision-per-property', 'title': 'regression precision per property schema', 'type': 'object', 'properties': {'name': {'description': "property name in 'flattened' format", 'type': 'string'}, 'trainingError': {'description': 'training error of the estimator', 'type': 'number'}, 'score': {'description': 'prediction score of the estimator. Eg: r2_score', 'type': 'number'}}, 'required': ['trainingError']}}}}, 'data': {'$id': 'methods-directory/mathematical/regression/data', 'title': 'regression data', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'oneOf': [{'$id': 'methods-directory/mathematical/regression/linear/data-per-property', 'title': 'linear regression parameters schema', 'properties': {'intercept': {'description': 'intercept (shift) from the linear or non-linear fit of data points', 'type': 'number'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['intercept', 'perFeature']}, {'$id': 'methods-directory/mathematical/regression/kernel-ridge/data-per-property', 'title': 'kernel-ridge regression parameters schema', 'properties': {'xFit': {'description': 'training data', 'type': 'array'}, 'dualCoefficients': {'description': 'dual coefficients', 'type': 'array'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['xFit', 'dualCoefficients', 'perFeature']}]}}, 'dataSet': {'$id': 'methods-directory/mathematical/regression/dataset', 'description': 'dataset for ml', 'type': 'object', 'properties': {'exabyteIds': {'description': 'array of exabyteIds for materials in dataset', 'type': 'array', 'items': {'type': 'string'}}, 'extra': {'description': 'holder for any extra information, eg. coming from user-uploaded CSV file'}}, 'required': ['exabyteIds']}}}}, 'required': ['precision', 'data']}, {'$id': 'methods-directory/legacy/pseudopotential', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy method pseudopotential', 'allOf': [{'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}], 'properties': {'type': {'enum': ['pseudopotential']}, 'subtype': {'enum': ['paw', 'nc', 'us', 'any']}}}, {'$id': 'methods-directory/legacy/localorbital', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy method localorbital', 'allOf': [{'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}], 'properties': {'type': {'enum': ['localorbital']}, 'subtype': {'enum': ['pople']}}}, {'$id': 'methods-directory/legacy/unknown', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy method unknown', 'allOf': [{'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}], 'properties': {'type': {'enum': ['unknown']}, 'subtype': {'enum': ['unknown']}}}, {'$id': 'methods-directory/mathematical/regression', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method regression', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/mathematical/regression', 'title': 'linear methods category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'type': {'enum': ['linear', 'kernel_ridge']}, 'subtype': {'enum': ['least_squares', 'ridge']}}}, 'precision': {'$id': 'methods-directory/mathematical/regression/precision', 'title': 'regression precision', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'$id': 'methods-directory/mathematical/regression/precision-per-property', 'title': 'regression precision per property schema', 'type': 'object', 'properties': {'name': {'description': "property name in 'flattened' format", 'type': 'string'}, 'trainingError': {'description': 'training error of the estimator', 'type': 'number'}, 'score': {'description': 'prediction score of the estimator. Eg: r2_score', 'type': 'number'}}, 'required': ['trainingError']}}}}, 'data': {'$id': 'methods-directory/mathematical/regression/data', 'title': 'regression data', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'oneOf': [{'$id': 'methods-directory/mathematical/regression/linear/data-per-property', 'title': 'linear regression parameters schema', 'properties': {'intercept': {'description': 'intercept (shift) from the linear or non-linear fit of data points', 'type': 'number'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['intercept', 'perFeature']}, {'$id': 'methods-directory/mathematical/regression/kernel-ridge/data-per-property', 'title': 'kernel-ridge regression parameters schema', 'properties': {'xFit': {'description': 'training data', 'type': 'array'}, 'dualCoefficients': {'description': 'dual coefficients', 'type': 'array'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['xFit', 'dualCoefficients', 'perFeature']}]}}, 'dataSet': {'$id': 'methods-directory/mathematical/regression/dataset', 'description': 'dataset for ml', 'type': 'object', 'properties': {'exabyteIds': {'description': 'array of exabyteIds for materials in dataset', 'type': 'array', 'items': {'type': 'string'}}, 'extra': {'description': 'holder for any extra information, eg. coming from user-uploaded CSV file'}}, 'required': ['exabyteIds']}}}}, 'required': ['categories', 'precision', 'data']}, {'$id': 'methods-directory/mathematical/davidson', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method davidson', 'description': 'Davidson diagonalization method', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/mathematical/linalg/diag/davidson', 'description': 'Davidson diagonalization method', 'allOf': [{'$id': 'methods-category/mathematical/linalg/diag', 'description': 'Matrix diagonalization methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['diag']}}}], 'properties': {'type': {'enum': ['davidson']}}}}, 'required': ['categories']}, {'$id': 'methods-directory/mathematical/cg', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'unit method conjugate gradient', 'description': 'conjugate gradient method schema', 'allOf': [{'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}], 'properties': {'categories': {'$id': 'methods-category/mathematical/opt/diff/ordern/cg', 'description': 'conjugate gradient method schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff/ordern', 'description': 'Mixed order and higher order algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['ordern']}}}], 'properties': {'type': {'enum': ['cg']}}}}, 'required': ['categories']}, {'$id': 'methods-directory/mathematical/regression/data', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'regression data', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'oneOf': [{'$id': 'methods-directory/mathematical/regression/linear/data-per-property', 'title': 'linear regression parameters schema', 'properties': {'intercept': {'description': 'intercept (shift) from the linear or non-linear fit of data points', 'type': 'number'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['intercept', 'perFeature']}, {'$id': 'methods-directory/mathematical/regression/kernel-ridge/data-per-property', 'title': 'kernel-ridge regression parameters schema', 'properties': {'xFit': {'description': 'training data', 'type': 'array'}, 'dualCoefficients': {'description': 'dual coefficients', 'type': 'array'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['xFit', 'dualCoefficients', 'perFeature']}]}}, 'dataSet': {'$id': 'methods-directory/mathematical/regression/dataset', 'description': 'dataset for ml', 'type': 'object', 'properties': {'exabyteIds': {'description': 'array of exabyteIds for materials in dataset', 'type': 'array', 'items': {'type': 'string'}}, 'extra': {'description': 'holder for any extra information, eg. coming from user-uploaded CSV file'}}, 'required': ['exabyteIds']}}}, {'$id': 'methods-directory/mathematical/regression/per-feature-item', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}, {'$id': 'methods-directory/mathematical/regression/precision-per-property', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'regression precision per property schema', 'type': 'object', 'properties': {'name': {'description': "property name in 'flattened' format", 'type': 'string'}, 'trainingError': {'description': 'training error of the estimator', 'type': 'number'}, 'score': {'description': 'prediction score of the estimator. Eg: r2_score', 'type': 'number'}}, 'required': ['trainingError']}, {'$id': 'methods-directory/mathematical/regression/dataset', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'dataset for ml', 'type': 'object', 'properties': {'exabyteIds': {'description': 'array of exabyteIds for materials in dataset', 'type': 'array', 'items': {'type': 'string'}}, 'extra': {'description': 'holder for any extra information, eg. coming from user-uploaded CSV file'}}, 'required': ['exabyteIds']}, {'$id': 'methods-directory/mathematical/regression/precision', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'regression precision', 'type': 'object', 'properties': {'perProperty': {'type': 'array', 'items': {'$id': 'methods-directory/mathematical/regression/precision-per-property', 'title': 'regression precision per property schema', 'type': 'object', 'properties': {'name': {'description': "property name in 'flattened' format", 'type': 'string'}, 'trainingError': {'description': 'training error of the estimator', 'type': 'number'}, 'score': {'description': 'prediction score of the estimator. Eg: r2_score', 'type': 'number'}}, 'required': ['trainingError']}}}}, {'$id': 'methods-directory/mathematical/regression/kernel-ridge/data-per-property', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'kernel-ridge regression parameters schema', 'properties': {'xFit': {'description': 'training data', 'type': 'array'}, 'dualCoefficients': {'description': 'dual coefficients', 'type': 'array'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['xFit', 'dualCoefficients', 'perFeature']}, {'$id': 'methods-directory/mathematical/regression/linear/data-per-property', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'linear regression parameters schema', 'properties': {'intercept': {'description': 'intercept (shift) from the linear or non-linear fit of data points', 'type': 'number'}, 'perFeature': {'type': 'array', 'description': 'per-feature (property used for training the ML method/model) parameters', 'items': {'$id': 'methods-directory/mathematical/regression/per-feature-item', 'title': 'per-feature (property used for training the ML method/model) parameters schema', 'properties': {'coefficient': {'description': 'coefficient in linear regression', 'type': 'number'}, 'name': {'description': 'feature name', 'type': 'string'}, 'importance': {'description': 'pvalue: https://en.wikipedia.org/wiki/P-value', 'type': 'number'}}, 'required': ['name']}}}, 'required': ['intercept', 'perFeature']}, {'$id': 'properties-directory/derived-properties', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'derived properties schema', 'type': 'array', 'items': {'anyOf': [{'$id': 'properties-directory/structural/volume', 'title': 'volume schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['volume']}, 'units': {'enum': ['angstrom^3']}}}, {'$id': 'properties-directory/structural/density', 'title': 'density schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['density']}, 'units': {'enum': ['g/cm^3']}}}, {'$id': 'properties-directory/structural/symmetry', 'title': 'symmetry schema', 'properties': {'pointGroupSymbol': {'description': 'point group symbol in Schoenflies notation', 'type': 'string'}, 'spaceGroupSymbol': {'description': 'space group symbol in Hermann–Mauguin notation', 'type': 'string'}, 'tolerance': {'description': 'tolerance used for symmetry calculation', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'units': {'enum': ['angstrom']}}}, 'name': {'enum': ['symmetry']}}}, {'$id': 'properties-directory/structural/elemental-ratio', 'title': 'elemental-ratio', 'description': 'ration of this element in the compound', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['elemental_ratio']}, 'value': {'type': 'number', 'min': 0, 'max': 1}, 'element': {'type': 'string', 'description': 'the element this ratio is for'}}}, {'$id': 'properties-directory/structural/p-norm', 'title': 'p_norm', 'description': 'https://en.wikipedia.org/wiki/Norm_(mathematics)#p-norm', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['p-norm']}, 'degree': {'type': 'integer', 'description': 'degree of the dimensionality of the norm'}}}, {'$id': 'properties-directory/structural/inchi', 'title': 'InChI representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi']}}}, {'$id': 'properties-directory/structural/inchi-key', 'title': 'InChI key representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi_key']}}}]}}, {'$id': 'properties-directory/electronic-configuration', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'electronic configuration schema', 'type': 'object', 'properties': {'charge': {'description': 'total charge of the molecular system', 'type': 'integer'}, 'multiplicity': {'description': 'calculated as 2S+1, with S is the total spin angular momentum', 'type': 'integer'}}}, {'$id': 'properties-directory/elemental/ionization-potential', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ionization potential', 'description': 'ionization potential for the element', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['ionization_potential']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, {'$id': 'properties-directory/elemental/electronegativity', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'electronegativity', 'description': 'electronegativity for the element (Pauling scale)', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['electronegativity']}}}, {'$id': 'properties-directory/elemental/atomic-radius', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic radius', 'description': 'atomic radius', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['atomic_radius']}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, {'$id': 'properties-directory/workflow/convergence/kpoint', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'convergence schema for converging a property wrt kpoints', 'properties': {'tolerance': {'description': 'tolerance for the property under investigation'}, 'units': {'description': 'units for the property under investigation', 'type': 'string'}, 'property': {'description': 'name of the property under investigation', 'type': 'string'}, 'data': {'description': 'kpoint grid and property information', 'items': {'type': 'object', 'properties': {'value': {'description': 'value of the property at this step'}, 'grid': {'description': 'information about the kpoint grid', 'type': 'object'}, 'spacing': {'description': 'optional kpoint spacing information', 'type': 'number'}}, 'required': ['value', 'grid']}}}, 'required': ['tolerance', 'units', 'data']}, {'$id': 'properties-directory/workflow/convergence/electronic', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'electronic self consistency convergence schema', 'properties': {'name': {'enum': ['convergence_electronic']}, 'units': {'enum': ['eV', 'rydberg', 'hartree']}, 'data': {'type': 'array', 'items': {'type': 'array', 'items': {'type': 'number'}}}}, 'required': ['data']}, {'$id': 'properties-directory/workflow/convergence/ionic', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ionic convergence schema', 'properties': {'name': {'enum': ['convergence_ionic']}, 'tolerance': {'description': 'for ionic convergence tolerance shows force tolerance'}, 'units': {'description': 'units for force tolerance', 'enum': ['eV']}, 'data': {'description': 'energetic and structural information', 'items': {'type': 'object', 'properties': {'energy': {'description': 'converged electronic energy for this structure (last in `electronic`)', 'type': 'number'}, 'structure': {'description': 'TODO: structural information at each step to be here', 'type': 'object'}, 'electronic': {'description': 'data about electronic at this ionic step', 'type': 'object', 'properties': {'units': {'description': 'units for force tolerance', 'enum': ['eV', 'rydberg', 'hartree']}, 'data': {'type': 'array', 'items': {'type': 'number'}}}}}}}}, 'required': ['data']}, {'$id': 'properties-directory/structural/volume', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'volume schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['volume']}, 'units': {'enum': ['angstrom^3']}}}, {'$id': 'properties-directory/structural/symmetry', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'symmetry schema', 'properties': {'pointGroupSymbol': {'description': 'point group symbol in Schoenflies notation', 'type': 'string'}, 'spaceGroupSymbol': {'description': 'space group symbol in Hermann–Mauguin notation', 'type': 'string'}, 'tolerance': {'description': 'tolerance used for symmetry calculation', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'units': {'enum': ['angstrom']}}}, 'name': {'enum': ['symmetry']}}}, {'$id': 'properties-directory/structural/inchi', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'InChI representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi']}}}, {'$id': 'properties-directory/structural/magnetic-moments', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'magnetic moments', 'description': 'magnetization on each ion', 'properties': {'name': {'enum': ['magnetic_moments']}, 'values': {'$id': 'core/reusable/atomic-vectors', 'title': 'atomic vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, 'units': {'enum': ['uB']}}}, {'$id': 'properties-directory/structural/p-norm', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'p_norm', 'description': 'https://en.wikipedia.org/wiki/Norm_(mathematics)#p-norm', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['p-norm']}, 'degree': {'type': 'integer', 'description': 'degree of the dimensionality of the norm'}}}, {'$id': 'properties-directory/structural/lattice', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'lattice schema', 'type': 'object', 'allOf': [{'properties': {'vectors': {'$id': 'properties-directory/structural/lattice/lattice-vectors', 'title': 'lattice explicit unit', 'allOf': [{'$id': 'core/abstract/3d-vector-basis', 'title': '3 dimensional vector basis', 'type': 'object', 'properties': {'a': {'description': 'first vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'b': {'description': 'second vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'c': {'description': 'third vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}, 'required': ['a', 'b', 'c']}], 'properties': {'alat': {'description': 'lattice parameter for fractional coordinates', 'type': 'number', 'default': 1}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}}}, {'$id': 'properties-directory/structural/lattice/lattice-bravais', 'title': 'lattice implicit schema', 'allOf': [{'$id': 'core/primitive/3d-lattice', 'title': '3 dimensional lattice schema', 'type': 'object', 'properties': {'a': {'description': 'length of the first lattice vector', 'type': 'number'}, 'b': {'description': 'length of the second lattice vector', 'type': 'number'}, 'c': {'description': 'length of the third lattice vector', 'type': 'number'}, 'alpha': {'description': 'angle between first and second lattice vector', 'type': 'number'}, 'beta': {'description': 'angle between second and third lattice vector', 'type': 'number'}, 'gamma': {'description': 'angle between first and third lattice vector', 'type': 'number'}}, 'required': ['a', 'b', 'c', 'alpha', 'beta', 'gamma']}], 'properties': {'type': {'description': 'Bravais lattice type in short notation', '$id': 'properties-directory/structural/lattice/type-enum', 'title': 'lattice type schema', 'type': 'string', 'enum': ['CUB', 'BCC', 'FCC', 'TET', 'MCL', 'ORC', 'ORCC', 'ORCF', 'ORCI', 'HEX', 'BCT', 'TRI', 'MCLC', 'RHL']}, 'units': {'type': 'object', 'properties': {'length': {'type': 'string', 'enum': ['angstrom', 'bohr']}, 'angle': {'type': 'string', 'enum': ['degree', 'radian']}}}}, 'required': ['type']}], 'properties': {'name': {'enum': ['lattice']}}}, {'$id': 'properties-directory/structural/basis', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'basis schema', 'type': 'object', 'properties': {'elements': {'type': 'array', 'items': {'$id': 'properties-directory/structural/basis/atomic-element', 'title': 'atomic elements', 'description': 'elements of atoms by ids, string, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'type': 'string'}, 'occurrence': {'description': 'Occurrence is for fractional occupations', 'type': 'number'}, 'oxidationState': {'type': 'number'}}, 'required': ['id', 'value']}}, 'coordinates': {'type': 'array', 'items': {'$id': 'properties-directory/structural/basis/atomic-coordinate', 'title': 'atomic coordinate', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, 'name': {'type': 'string'}, 'units': {'type': 'string'}, 'bonds': {'$id': 'properties-directory/structural/basis/bonds', 'title': 'bonds schema', 'type': 'array', 'items': {'type': 'object', 'properties': {'atomPair': {'description': 'indices of the two connected atoms', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'minItems': 2, 'maxItems': 2}, 'bondType': {'type': 'string', 'enum': ['single', 'double', 'triple', 'quadruple', 'aromatic', 'tautomeric', 'dative', 'other']}}}, 'uniqueItems': True}}, 'required': ['elements', 'coordinates']}, {'$id': 'properties-directory/structural/elemental-ratio', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'elemental-ratio', 'description': 'ration of this element in the compound', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['elemental_ratio']}, 'value': {'type': 'number', 'min': 0, 'max': 1}, 'element': {'type': 'string', 'description': 'the element this ratio is for'}}}, {'$id': 'properties-directory/structural/inchi-key', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'InChI key representation schema', 'allOf': [{'$id': 'core/primitive/string', 'title': 'PrimitiveString', 'type': 'object', 'properties': {'value': {'type': 'string'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['inchi_key']}}}, {'$id': 'properties-directory/structural/atomic-forces', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic forces', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'name': {'enum': ['atomic_forces']}, 'values': {'$id': 'core/reusable/atomic-vectors', 'title': 'atomic vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, 'units': {'enum': ['eV/bohr', 'eV/angstrom', 'rydberg/a.u.', 'newton', 'kg*m/s^2', 'eV/a.u.']}}}, {'$id': 'properties-directory/structural/molecular-pattern', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'molecular pattern schema', 'type': 'array', 'items': {'anyOf': [{'$id': 'properties-directory/structural/patterns/functional-group', 'title': 'functional group pattern schema', 'type': 'object', 'properties': {'name': {'enum': ['functional_group']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}, 'SMARTS': {'description': 'SMARTS string for classification of FG; https://en.wikipedia.org/wiki/SMILES_arbitrary_target_specification', 'type': 'string'}}}, {'$id': 'properties-directory/structural/patterns/ring', 'title': 'ring pattern schema', 'type': 'object', 'properties': {'name': {'enum': ['ring']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}, 'isAromatic': {'type': 'boolean'}}}, {'$id': 'properties-directory/structural/patterns/special-bond', 'title': 'special bond pattern schema', 'type': 'object', 'description': 'Any bonding interaction that cannot be described by simple 2-atom picture, e.g. 3-center-2-electron bond in diborane', 'properties': {'name': {'enum': ['special_bond']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}}}]}}, {'$id': 'properties-directory/structural/density', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'density schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['density']}, 'units': {'enum': ['g/cm^3']}}}, {'$id': 'properties-directory/structural/patterns/functional-group', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'functional group pattern schema', 'type': 'object', 'properties': {'name': {'enum': ['functional_group']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}, 'SMARTS': {'description': 'SMARTS string for classification of FG; https://en.wikipedia.org/wiki/SMILES_arbitrary_target_specification', 'type': 'string'}}}, {'$id': 'properties-directory/structural/patterns/ring', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ring pattern schema', 'type': 'object', 'properties': {'name': {'enum': ['ring']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}, 'isAromatic': {'type': 'boolean'}}}, {'$id': 'properties-directory/structural/patterns/special-bond', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'special bond pattern schema', 'type': 'object', 'description': 'Any bonding interaction that cannot be described by simple 2-atom picture, e.g. 3-center-2-electron bond in diborane', 'properties': {'name': {'enum': ['special_bond']}, 'atoms': {'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'isConnector': {'description': 'whether atom connects to atoms outside of functional group.', 'type': 'boolean'}}}}}}, {'$id': 'properties-directory/structural/lattice/lattice-vectors', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'lattice explicit unit', 'allOf': [{'$id': 'core/abstract/3d-vector-basis', 'title': '3 dimensional vector basis', 'type': 'object', 'properties': {'a': {'description': 'first vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'b': {'description': 'second vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'c': {'description': 'third vector', '$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}, 'required': ['a', 'b', 'c']}], 'properties': {'alat': {'description': 'lattice parameter for fractional coordinates', 'type': 'number', 'default': 1}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, {'$id': 'properties-directory/structural/lattice/lattice-bravais', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'lattice implicit schema', 'allOf': [{'$id': 'core/primitive/3d-lattice', 'title': '3 dimensional lattice schema', 'type': 'object', 'properties': {'a': {'description': 'length of the first lattice vector', 'type': 'number'}, 'b': {'description': 'length of the second lattice vector', 'type': 'number'}, 'c': {'description': 'length of the third lattice vector', 'type': 'number'}, 'alpha': {'description': 'angle between first and second lattice vector', 'type': 'number'}, 'beta': {'description': 'angle between second and third lattice vector', 'type': 'number'}, 'gamma': {'description': 'angle between first and third lattice vector', 'type': 'number'}}, 'required': ['a', 'b', 'c', 'alpha', 'beta', 'gamma']}], 'properties': {'type': {'description': 'Bravais lattice type in short notation', '$id': 'properties-directory/structural/lattice/type-enum', 'title': 'lattice type schema', 'type': 'string', 'enum': ['CUB', 'BCC', 'FCC', 'TET', 'MCL', 'ORC', 'ORCC', 'ORCF', 'ORCI', 'HEX', 'BCT', 'TRI', 'MCLC', 'RHL']}, 'units': {'type': 'object', 'properties': {'length': {'type': 'string', 'enum': ['angstrom', 'bohr']}, 'angle': {'type': 'string', 'enum': ['degree', 'radian']}}}}, 'required': ['type']}, {'$id': 'properties-directory/structural/lattice/type-enum', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'lattice type schema', 'type': 'string', 'enum': ['CUB', 'BCC', 'FCC', 'TET', 'MCL', 'ORC', 'ORCC', 'ORCF', 'ORCI', 'HEX', 'BCT', 'TRI', 'MCLC', 'RHL']}, {'$id': 'properties-directory/structural/lattice/type-extended-enum', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'lattice type extended schema', 'type': 'string', 'enum': ['BCC', 'BCT-1', 'BCT-2', 'CUB', 'FCC', 'HEX', 'MCL', 'MCLC-1', 'MCLC-2', 'MCLC-3', 'MCLC-4', 'MCLC-5', 'ORC', 'ORCC', 'ORCF-1', 'ORCF-2', 'ORCF-3', 'ORCI', 'RHL-1', 'RHL-2', 'TET', 'TRI_1a', 'TRI_2a', 'TRI_1b']}, {'$id': 'properties-directory/structural/basis/atomic-element', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic elements', 'description': 'elements of atoms by ids, string, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'type': 'string'}, 'occurrence': {'description': 'Occurrence is for fractional occupations', 'type': 'number'}, 'oxidationState': {'type': 'number'}}, 'required': ['id', 'value']}, {'$id': 'properties-directory/structural/basis/atomic-coordinates', 'title': 'atomic coordinates', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'name': {'enum': ['atomic_coordinates']}, 'values': {'type': 'array', 'items': {'$id': 'properties-directory/structural/basis/atomic-coordinate', 'title': 'atomic coordinate', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, {'$id': 'properties-directory/structural/basis/atomic-constraints', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic constraints', 'description': 'atomic constraints schema', 'properties': {'name': {'enum': ['atomic_constraints']}, 'values': {'$id': 'core/reusable/atomic-vectors', 'title': 'atomic vectors schema', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'items': {'properties': {'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}}}}, {'$id': 'properties-directory/structural/basis/bonds', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'bonds schema', 'type': 'array', 'items': {'type': 'object', 'properties': {'atomPair': {'description': 'indices of the two connected atoms', 'allOf': [{'$id': 'core/primitive/array-of-ids', 'title': 'atomic ids', 'description': 'array of objects containing integer id each', 'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'description': 'integer id of this entry', 'type': 'integer'}}, 'uniqueItems': True}}], 'minItems': 2, 'maxItems': 2}, 'bondType': {'type': 'string', 'enum': ['single', 'double', 'triple', 'quadruple', 'aromatic', 'tautomeric', 'dative', 'other']}}}, 'uniqueItems': True}, {'$id': 'properties-directory/structural/basis/atomic-coordinate', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'atomic coordinate', 'description': 'coordinates of atoms by ids, vector, unitless', 'properties': {'id': {'type': 'number'}, 'value': {'$id': 'core/abstract/vector', 'title': 'vector schema', 'oneOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, {'$id': 'core/primitive/array-of-3-booleans', 'title': 'array of 3 boolean elements schema', 'type': 'array', 'items': {'type': 'boolean'}, 'minItems': 3, 'maxItems': 3}]}}}, {'$id': 'properties-directory/non-scalar/vibrational-spectrum', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Vibrational spectrum schema', 'type': 'object', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['frequency', 'wavenumber']}, 'units': {'enum': ['cm-1', 'THz', 'meV']}}}, 'yAxis': {'properties': {'label': {'enum': ['Intensity', 'Absorbance', 'Absorption coefficient']}, 'units': {'enum': ['(debye/angstrom)^2', 'km/mol', 'm/mol', 'a.u.']}}}, 'name': {'enum': ['vibrational_spectrum']}}}, {'$id': 'properties-directory/non-scalar/file-content', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'file_content', 'allOf': [{'$id': 'core/reusable/file-metadata', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}], 'properties': {'name': {'enum': ['file_content']}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string', 'enum': ['image', 'text', 'csv'], '$comment': 'isGenerative:true'}, 'objectData': {'$id': 'core/reusable/object-storage-container-data', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}}, 'required': ['name', 'objectData']}, {'$id': 'properties-directory/non-scalar/band-gaps', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'band gaps schema', 'description': 'contains band gap values', 'properties': {'name': {'enum': ['band_gaps']}, 'values': {'type': 'array', 'items': {'$id': 'core/reusable/band-gap', 'title': 'band gap schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'kpointConduction': {'description': 'kpoint inside conduction band in crystal coordinates', '$id': 'core/abstract/point', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, 'kpointValence': {'description': 'kpoint inside valence band in crystal coordinates', '$id': 'core/abstract/point', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, 'eigenvalueConduction': {'description': 'eigenvalue at k-point in conduction band', 'type': 'number'}, 'eigenvalueValence': {'description': 'eigenvalue at k-point in valence band', 'type': 'number'}, 'spin': {'type': 'number'}, 'type': {'type': 'string', 'enum': ['direct', 'indirect']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}, 'required': ['type']}}, 'eigenvalues': {'type': 'array', 'items': {'properties': {'kpoint': {'$id': 'core/abstract/point', 'title': 'point schema', 'allOf': [{'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}]}, 'weight': {'type': 'number'}, 'eigenvalues': {'type': 'array', 'items': {'properties': {'spin': {'type': 'number'}, 'energies': {'type': 'array'}, 'occupations': {'type': 'array'}}}}}}}}, 'required': ['name']}, {'$id': 'properties-directory/non-scalar/potential-profile', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'potential profile schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['z coordinate']}}}, 'yAxis': {'properties': {'label': {'enum': ['energy']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, 'name': {'enum': ['potential_profile']}}}, {'$id': 'properties-directory/non-scalar/density-of-states', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'density of states schema', 'type': 'object', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['energy']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, 'yAxis': {'properties': {'label': {'enum': ['density of states']}, 'units': {'enum': ['states/unitcell']}}}, 'name': {'enum': ['density_of_states']}, 'legend': {'items': {'type': 'object', 'properties': {'element': {'description': 'chemical element', 'type': 'string'}, 'index': {'description': 'index inside sub-array of atoms of the same element type', 'type': 'integer'}, 'electronicState': {'description': 'electronic character and shell of PDOS, such as `1s` or `s`, or `total`', 'type': 'string', 'pattern': '^([1-5]{1})?(s|p|d|f|g).*$'}, 'spin': {'description': 'spin of the electronic state', 'type': 'number', 'enum': [0.5, -0.5]}}}}}}, {'$id': 'properties-directory/non-scalar/charge-density-profile', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'charge density profile schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['z coordinate']}}}, 'yAxis': {'properties': {'label': {'enum': ['charge density']}, 'units': {'enum': ['e/A']}}}, 'name': {'enum': ['charge_density_profile']}}}, {'$id': 'properties-directory/non-scalar/hubbard-u', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Hubbard U parameters', 'description': 'Hubbard U values in eV corresponding to atomic species, orbital and site number.', 'properties': {'name': {'enum': ['hubbard_u']}, 'units': {'enum': ['eV']}, 'values': {'type': 'array', 'items': {'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data-per-orbital-numeric', 'title': 'Atomic data per orbital numeric', 'description': 'Atomic properties per orbital pair with numeric value e.g., Hubbard V parameters.', 'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data/per-orbital', 'title': 'Atomic data per orbital', 'description': 'Atomic properties per orbital e.g., Hubbard U parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}}}, {'$id': 'core/reusable/atomic-data/value-number', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}]}], 'required': ['id', 'atomicSpecies', 'orbitalName', 'value']}}}, 'required': ['name']}, {'$id': 'properties-directory/non-scalar/reaction-energy-profile', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'reaction energy profile schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['reaction coordinate']}}}, 'yAxis': {'properties': {'label': {'enum': ['energy']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, 'name': {'enum': ['reaction_energy_profile']}}}, {'$id': 'properties-directory/non-scalar/phonon-dispersions', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'phonon band structure schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['qpoints']}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, 'yAxis': {'properties': {'label': {'enum': ['frequency']}, 'units': {'enum': ['cm-1', 'THz', 'meV']}}}, 'name': {'enum': ['phonon_dispersions']}}}, {'$id': 'properties-directory/non-scalar/average-potential-profile', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'average potential profile schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['z coordinate']}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, 'yAxis': {'properties': {'label': {'enum': ['energy']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, 'name': {'enum': ['average_potential_profile']}}}, {'$id': 'properties-directory/non-scalar/dielectric-tensor', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'dielectric tensor property', 'description': 'The real and imaginary parts of the diagonal elements of the dieletric tensor', 'properties': {'name': {'enum': ['dielectric_tensor']}, 'values': {'type': 'array', 'items': {'$id': 'core/reusable/dielectric-tensor-component', 'title': 'Dielectric Tensor', 'allOf': [{'$id': 'core/reusable/frequency-function-matrix', 'title': '', 'description': 'Schema for a function of frequency yielding a nx3 matrix', 'type': 'object', 'properties': {'frequencies': {'description': 'Frequencies', 'type': 'array', 'items': {'type': 'number'}}, 'components': {'description': 'Matrix with 3 columns, e.g. x, y, z', 'type': 'array', 'items': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}}}}], 'properties': {'part': {'description': 'Real or imaginary part of the dielectric tensor component', 'type': 'string', 'enum': ['real', 'imaginary']}, 'spin': {'type': 'number'}}, 'required': ['part', 'frequencies', 'components']}}}, 'required': ['name']}, {'$id': 'properties-directory/non-scalar/hubbard-v', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Hubbard V parameters', 'description': 'Hubbard V values corresponding to atomic pairs', 'properties': {'name': {'enum': ['hubbard_v', 'hubbard_v_nn']}, 'units': {'enum': ['eV']}, 'values': {'type': 'array', 'items': {'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data-per-orbital-pair-numeric', 'title': 'Atomic data per orbital pair numeric', 'description': 'Atomic properties per orbital pair with numeric value e.g., Hubbard V parameters.', 'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data/per-orbital-pair', 'title': 'Atomic data per orbital pair', 'description': 'Atomic properties per orbital pair e.g., Hubbard V parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'id2': {'type': 'integer', 'description': 'Site number or index in the lattice of second site'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'atomicSpecies2': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co2, O'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalName2': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'distance': {'type': 'number', 'description': 'Distance between two sites in Bohr.'}}}, {'$id': 'core/reusable/atomic-data/value-number', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}]}], 'required': ['id', 'id2', 'atomicSpecies', 'atomicSpecies2', 'value']}}}, 'required': ['name']}, {'$id': 'properties-directory/non-scalar/band-structure', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'band structure schema', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['kpoints']}, 'units': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}}}, 'yAxis': {'properties': {'label': {'enum': ['energy']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, 'name': {'enum': ['band_structure']}, 'spin': {'description': 'spin of each band', 'type': 'array', 'items': {'type': 'number', 'enum': [0.5, -0.5]}}}}, {'$id': 'properties-directory/non-scalar/phonon-dos', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Phonon density of states schema', 'type': 'object', 'allOf': [{'$id': 'core/abstract/2d-plot', 'title': '2 dimension plot schema', 'allOf': [{'$id': 'core/abstract/2d-data', 'title': '2 dimension data schema', 'type': 'object', 'properties': {'xDataArray': {'description': 'array containing values of x Axis', 'type': 'array'}, 'yDataSeries': {'$id': 'core/primitive/1d-data-series', 'title': '1 dimension data series schema', 'type': 'array', 'items': {'type': 'array', 'minItems': 1, 'items': {'type': ['number', 'string']}}}}, 'required': ['xDataArray', 'yDataSeries']}], 'properties': {'xAxis': {'description': 'x Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'yAxis': {'description': 'y Axis object', '$id': 'core/primitive/axis', 'title': 'axis schema', 'properties': {'label': {'description': 'label of an axis object', 'type': 'string'}, 'units': {'description': 'units for an axis', 'type': 'string'}}, 'required': ['label']}, 'legend': {'description': 'Legend of y Axis data series', 'minItems': 1, 'type': 'array'}}, 'required': ['xAxis', 'yAxis']}], 'properties': {'xAxis': {'properties': {'label': {'enum': ['frequency']}, 'units': {'enum': ['cm-1', 'THz', 'meV']}}}, 'yAxis': {'properties': {'label': {'enum': ['Phonon DOS']}, 'units': {'enum': ['states/cm-1', 'states/THz', 'states/meV']}}}, 'name': {'enum': ['phonon_dos']}}}, {'$id': 'properties-directory/non-scalar/total-energy-contributions', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'total energy contributions schema', 'type': 'object', 'properties': {'temperatureEntropy': {'description': 'product of temperature and configurational entropy', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['temperature_entropy']}}}, 'harrisFoulkes': {'description': 'non self-consitent energy based on an input charge density', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['harris_foulkes']}}}, 'oneElectron': {'description': 'kinetic + pseudopotential energy', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['one_electron']}}}, 'hartree': {'description': 'energy due to coulomb potential', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['hartree']}}}, 'exchange': {'description': 'exchange energy', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['exchange']}}}, 'exchangeCorrelation': {'description': 'exchange and correlation energy per particle', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['exchange_correlation']}}}, 'ewald': {'description': 'summation of interaction energies at long length scales due to coloumbic interactions', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['ewald']}}}, 'alphaZ': {'description': 'divergent electrostatic ion interaction in compensating electron gas', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['alphaZ']}}}, 'atomicEnergy': {'description': 'kinetic energy of wavefunctions in the atomic limit', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['atomic_energy']}}}, 'eigenvalues': {'description': 'sum of one electron energies of kinetic, electrostatic, and exchange correlation', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['eigenvalues']}}}, 'PAWDoubleCounting2': {'description': 'double counting correction 2', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['PAW_double-counting_correction_2']}}}, 'PAWDoubleCounting3': {'description': 'double counting correction 3', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['PAW_double-counting_correction_3']}}}, 'hartreeFock': {'description': 'hartree-fock contribution', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['hartree_fock']}}}, 'name': {'enum': ['total_energy_contributions']}, 'units': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}}}, {'$id': 'properties-directory/non-scalar/hubbard-v-nn', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Hubbard V parameters for nearest neighbors', 'description': 'Hubbard V value in eV for nearest neighbors used in hp.x output parsing', 'allOf': [{'$id': 'properties-directory/non-scalar/hubbard-v', 'title': 'Hubbard V parameters', 'description': 'Hubbard V values corresponding to atomic pairs', 'properties': {'name': {'enum': ['hubbard_v', 'hubbard_v_nn']}, 'units': {'enum': ['eV']}, 'values': {'type': 'array', 'items': {'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data-per-orbital-pair-numeric', 'title': 'Atomic data per orbital pair numeric', 'description': 'Atomic properties per orbital pair with numeric value e.g., Hubbard V parameters.', 'type': 'object', 'allOf': [{'$id': 'core/reusable/atomic-data/per-orbital-pair', 'title': 'Atomic data per orbital pair', 'description': 'Atomic properties per orbital pair e.g., Hubbard V parameters.', 'type': 'object', 'properties': {'id': {'type': 'integer', 'description': 'Site number or index in the lattice'}, 'id2': {'type': 'integer', 'description': 'Site number or index in the lattice of second site'}, 'atomicSpecies': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co1, Mn'}, 'atomicSpecies2': {'type': 'string', 'pattern': '^[a-zA-Z]{1,2}[\\d+]?$', 'description': 'Example: Co2, O'}, 'orbitalName': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'orbitalName2': {'type': 'string', 'pattern': '^[1-7][sSpPdDfF]$'}, 'distance': {'type': 'number', 'description': 'Distance between two sites in Bohr.'}}}, {'$id': 'core/reusable/atomic-data/value-number', 'title': 'Atomic data numeric properties', 'description': 'Numeric value specific to atomic data', 'type': 'object', 'properties': {'value': {'type': 'number', 'description': 'Value related to a specific property, e.g., Hubbard U, V etc.'}}}]}], 'required': ['id', 'id2', 'atomicSpecies', 'atomicSpecies2', 'value']}}}, 'required': ['name']}]}, {'$id': 'properties-directory/non-scalar/stress-tensor', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'stress tensor schema', 'properties': {'value': {'$id': 'core/abstract/3d-tensor', 'title': '3 dimensional tensor schema', 'type': 'array', 'items': {'$id': 'core/primitive/array-of-3-numbers', 'title': 'array of 3 number elements schema', 'type': 'array', 'items': {'type': 'number'}, 'minItems': 3, 'maxItems': 3}, 'minItems': 3, 'maxItems': 3}, 'name': {'enum': ['stress_tensor']}, 'units': {'enum': ['kbar', 'pa']}}}, {'$id': 'properties-directory/scalar/reaction-energy-barrier', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'reaction energy barrier schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['reaction_energy_barrier']}}}, {'$id': 'properties-directory/scalar/pressure', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'pressure', 'description': 'average pressure in unit cell', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['pressure']}, 'units': {'enum': ['kbar', 'pa']}}}, {'$id': 'properties-directory/scalar/ionization-potential', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ionization potential schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['ionization_potential']}}}, {'$id': 'properties-directory/scalar/fermi-energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'fermi energy schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['fermi_energy']}}}, {'$id': 'properties-directory/scalar/formation-energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'formation energy schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['formation_energy']}}}, {'$id': 'properties-directory/scalar/valence-band-offset', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'valence band offset schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['valence_band_offset']}}}, {'$id': 'properties-directory/scalar/surface-energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'surface energy schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['surface_energy']}}}, {'$id': 'properties-directory/scalar/electron-affinity', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'electron affinity schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['electron_affinity']}}}, {'$id': 'properties-directory/scalar/zero-point-energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'zero point energy schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['zero_point_energy']}}}, {'$id': 'properties-directory/scalar/total-energy', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'total energy schema', 'allOf': [{'$id': 'core/reusable/energy', 'title': 'energy schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'type': 'string'}, 'units': {'anyOf': [{'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, {'enum': ['eV/A^2']}]}}, 'required': ['name', 'units']}], 'properties': {'name': {'enum': ['total_energy']}}}, {'$id': 'properties-directory/scalar/total-force', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'total forces schema', 'allOf': [{'$id': 'core/primitive/scalar', 'title': 'scalar schema', 'type': 'object', 'properties': {'value': {'type': 'number'}}, 'required': ['value']}], 'properties': {'name': {'enum': ['total_force']}, 'units': {'enum': ['eV/bohr', 'eV/angstrom', 'rydberg/a.u.', 'newton', 'kg*m/s^2', 'eV/a.u.']}}}, {'$id': 'models-category/st', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'statistical model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['st']}}}, {'$id': 'models-category/pb', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}, {'$id': 'models-category/enum-options', 'physicsBased': {'enum': ['pb']}, 'statistical': {'enum': ['st']}}, {'$id': 'models-category/pb/qm', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}, {'$id': 'models-category/pb/enum-options', 'quantumMechanical': {'enum': ['qm']}}, {'$id': 'models-category/pb/qm/semp', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Semi-empirical category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['semp']}}}, {'$id': 'models-category/pb/qm/abin', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Ab initio category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['abin']}}}, {'$id': 'models-category/pb/qm/enum-options', 'abInitio': {'enum': ['abin']}, 'densityFunctional': {'enum': ['dft']}, 'semiEmpirical': {'enum': ['semp']}}, {'$id': 'models-category/pb/qm/dft', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}, {'$id': 'models-category/pb/qm/abin/gw', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'GW category schema', 'allOf': [{'$id': 'models-category/pb/qm/abin', 'title': 'Ab initio category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['abin']}}}], 'properties': {'type': {'enum': ['gw']}, 'subtype': {'enum': ['g0w0', 'evgw0', 'evgw']}}}, {'$id': 'models-category/pb/qm/abin/enum-options', 'gwApproximation': {'enum': ['gw']}, 'gwSubtypes': {'enum': ['g0w0', 'evgw0', 'evgw']}}, {'$id': 'models-category/pb/qm/dft/ksdft', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}, {'$id': 'models-category/pb/qm/dft/enum-options', 'kohnSham': {'enum': ['ksdft']}}, {'$id': 'models-category/pb/qm/dft/ksdft/gga', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DFT GGA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['gga']}}}, {'$id': 'models-category/pb/qm/dft/ksdft/mgga', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DFT meta-GGA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['mgga']}}}, {'$id': 'models-category/pb/qm/dft/ksdft/lda', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DFT LDA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['lda']}}}, {'$id': 'models-category/pb/qm/dft/ksdft/enum-options', 'localDensityApproximation': {'enum': ['lda']}, 'generalizedGradientApproximation': {'enum': ['gga']}, 'metaGGA': {'enum': ['mgga']}, 'hybrid': {'enum': ['hybrid']}, 'doubleHybrid': {'enum': ['double-hybrid']}}, {'$id': 'models-category/pb/qm/dft/ksdft/double-hybrid', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DFT double hybrid functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['double-hybrid']}}}, {'$id': 'models-category/pb/qm/dft/ksdft/hybrid', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DFT hybrid functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['hybrid']}}}, {'$id': 'models-category/st/enum-options', 'deterministic': {'enum': ['det']}}, {'$id': 'models-category/st/det', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'deterministic model category schema', 'allOf': [{'$id': 'models-category/st', 'title': 'statistical model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['st']}}}], 'properties': {'tier2': {'enum': ['det']}}}, {'$id': 'models-category/st/det/ml', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'machine learning model category schema', 'allOf': [{'$id': 'models-category/st/det', 'title': 'deterministic model category schema', 'allOf': [{'$id': 'models-category/st', 'title': 'statistical model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['st']}}}], 'properties': {'tier2': {'enum': ['det']}}}], 'properties': {'tier3': {'enum': ['ml']}}}, {'$id': 'models-category/st/det/enum-options', 'machineLearning': {'enum': ['ml']}}, {'$id': 'models-category/st/det/ml/enum-options', 'regression': {'enum': ['re']}}, {'$id': 'models-category/st/det/ml/re', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'regression model category schema', 'allOf': [{'$id': 'models-category/st/det/ml', 'title': 'machine learning model category schema', 'allOf': [{'$id': 'models-category/st/det', 'title': 'deterministic model category schema', 'allOf': [{'$id': 'models-category/st', 'title': 'statistical model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['st']}}}], 'properties': {'tier2': {'enum': ['det']}}}], 'properties': {'tier3': {'enum': ['ml']}}}], 'properties': {'type': {'enum': ['re']}}}, {'$id': 'software/template', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'template schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/input/-inputItem', 'title': 'execution unit input item schema for physics-based simulation engines', 'type': 'object', 'properties': {'name': {'description': 'Input file name. e.g. pw_scf.in', 'type': 'string'}, 'content': {'description': "Content of the input file. e.g. &CONTROL    calculation='scf' ...", 'type': 'string'}}, 'required': ['name', 'content']}], 'properties': {'applicationName': {'type': 'string'}, 'applicationVersion': {'type': 'string'}, 'executableName': {'type': 'string'}, 'contextProviders': {'type': 'array', 'items': {'description': 'render context provider names', '$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}}}, 'required': ['name', 'content']}, {'$id': 'software/executable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, {'$id': 'software/application', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, {'$id': 'software/flavor', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, {'$id': 'methods-category/physical/qm', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}, {'$id': 'methods-category/physical/enum-options', 'quantumMechanical': {'enum': ['qm']}}, {'$id': 'methods-category/physical/qm/wf', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}, {'$id': 'methods-category/physical/qm/enum-options', 'wavefunction': {'enum': ['wf']}}, {'$id': 'methods-category/physical/qm/wf/smearing', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Smearing methods category schema', 'description': 'Approximating Heaviside step function with smooth function', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['smearing']}, 'subtype': {'enum': ['gaussian', 'marzari-vanderbilt', 'methfessel-paxton', 'fermi-dirac']}}}, {'$id': 'methods-category/physical/qm/wf/ao', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}, {'$id': 'methods-category/physical/qm/wf/psp', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Pseudopotential category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['psp']}, 'subtype': {'enum': ['us', 'nc', 'paw', 'coulomb']}}}, {'$id': 'methods-category/physical/qm/wf/pw', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Plane wave catgeory schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['pw']}}}, {'$id': 'methods-category/physical/qm/wf/enum-options', 'planewave': {'enum': ['pw']}, 'atomicOrbital': {'enum': ['ao']}, 'wavelet': {'enum': ['wvl']}, 'smearing': {'enum': ['smearing']}, 'tetrahedron': {'enum': ['tetrahedron']}, 'pseudization': {'enum': ['psp']}, 'pseudoSubtypes': {'enum': ['us', 'nc', 'paw', 'coulomb']}, 'smearingSubtypes': {'enum': ['gaussian', 'marzari-vanderbilt', 'methfessel-paxton', 'fermi-dirac']}, 'tetrahedronSubtypes': {'enum': ['linear', 'optimized', 'bloechl']}, 'aoTypes': {'enum': ['pople', 'dunning', 'other']}}, {'$id': 'methods-category/physical/qm/wf/tetrahedron', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Tetrahedron method for Brillouin zone integration category schema ', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['tetrahedron']}, 'subtype': {'enum': ['linear', 'optimized', 'bloechl']}}}, {'$id': 'methods-category/physical/qm/wf/ao/dunning', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Dunning correlation-consistent basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['dunning']}}}, {'$id': 'methods-category/physical/qm/wf/ao/other', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Other (neither Pople nor Dunning) basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['other']}}}, {'$id': 'methods-category/physical/qm/wf/ao/pople', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Pople basis set category schema', 'allOf': [{'$id': 'methods-category/physical/qm/wf/ao', 'description': 'Approximating the electronic wave function with a atomic orbital basis', 'allOf': [{'$id': 'methods-category/physical/qm/wf', 'description': 'Methods related to wave functions', 'allOf': [{'$id': 'methods-category/physical/qm', 'description': 'Quantum-Mechanical method category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['qm']}}}], 'properties': {'tier2': {'enum': ['wf']}}}], 'properties': {'type': {'enum': ['ao']}, 'subtype': {'enum': ['pople', 'dunning', 'other']}}}], 'properties': {'subtype': {'enum': ['pople']}}}, {'$id': 'methods-category/mathematical/regression', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'linear methods category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'type': {'enum': ['linear', 'kernel_ridge']}, 'subtype': {'enum': ['least_squares', 'ridge']}}}, {'$id': 'methods-category/mathematical/diff', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Numerical differentiation category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['diff']}}}, {'$id': 'methods-category/mathematical/linalg', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}, {'$id': 'methods-category/mathematical/fapprx', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}, {'$id': 'methods-category/mathematical/opt', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}, {'$id': 'methods-category/mathematical/enum-options', 'differentiation': {'enum': ['diff']}, 'discretization': {'enum': ['discr']}, 'functionApproximation': {'enum': ['fapprx']}, 'integration': {'enum': ['intgr']}, 'linearAlgebra': {'enum': ['linalg']}, 'optimization': {'enum': ['opt']}, 'regressionTypes': {'enum': ['linear', 'kernel_ridge']}, 'regressionSubtypes': {'enum': ['least_squares', 'ridge']}}, {'$id': 'methods-category/mathematical/intgr', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}, {'$id': 'methods-category/mathematical/discr', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}, {'$id': 'methods-category/mathematical/linalg/diag', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Matrix diagonalization methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['diag']}}}, {'$id': 'methods-category/mathematical/linalg/lintra', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Linear transformation methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['lintra']}}}, {'$id': 'methods-category/mathematical/linalg/enum-options', 'decomposition': {'enum': ['dcomp']}, 'diagonalization': {'enum': ['diag']}, 'linearTransformation': {'enum': ['lintra']}, 'matrixFunction': {'enum': ['matf']}}, {'$id': 'methods-category/mathematical/linalg/matf', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Matrix function methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['matf']}}}, {'$id': 'methods-category/mathematical/linalg/dcomp', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Matrix decomposition methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['dcomp']}}}, {'$id': 'methods-category/mathematical/linalg/diag/davidson', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Davidson diagonalization method', 'allOf': [{'$id': 'methods-category/mathematical/linalg/diag', 'description': 'Matrix diagonalization methods', 'allOf': [{'$id': 'methods-category/mathematical/linalg', 'description': 'Linear Algebra category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['linalg']}}}], 'properties': {'tier2': {'enum': ['diag']}}}], 'properties': {'type': {'enum': ['davidson']}}}, {'$id': 'methods-category/mathematical/linalg/diag/enum-options', 'davidson': {'enum': ['davidson']}}, {'$id': 'methods-category/mathematical/fapprx/basisexp', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Basis expansion category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}], 'properties': {'tier2': {'enum': ['basisExp']}}}, {'$id': 'methods-category/mathematical/fapprx/ipol', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}], 'properties': {'tier2': {'enum': ['ipol']}}}, {'$id': 'methods-category/mathematical/fapprx/enum-options', 'basisExpansion': {'enum': ['basisExp']}, 'interpolation': {'enum': ['ipol']}}, {'$id': 'methods-category/mathematical/fapprx/ipol/enum-options', 'linear': {'enum': ['lin']}, 'polynomial': {'enum': ['poly']}, 'spline': {'enum': ['spline']}}, {'$id': 'methods-category/mathematical/fapprx/ipol/lin', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Linear interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx/ipol', 'description': 'Interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}], 'properties': {'tier2': {'enum': ['ipol']}}}], 'properties': {'tier3': {'enum': ['lin']}}}, {'$id': 'methods-category/mathematical/fapprx/ipol/spline', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Spline interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx/ipol', 'description': 'Interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}], 'properties': {'tier2': {'enum': ['ipol']}}}], 'properties': {'tier3': {'enum': ['spline']}}}, {'$id': 'methods-category/mathematical/fapprx/ipol/poly', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Polynomial interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx/ipol', 'description': 'Interpolation category schema', 'allOf': [{'$id': 'methods-category/mathematical/fapprx', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['fapprx']}}}], 'properties': {'tier2': {'enum': ['ipol']}}}], 'properties': {'tier3': {'enum': ['poly']}}}, {'$id': 'methods-category/mathematical/discr/mesh', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Meshing method category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}], 'properties': {'tier2': {'enum': ['mesh']}}}, {'$id': 'methods-category/mathematical/discr/enum-options', 'meshing': {'enum': ['mesh']}}, {'$id': 'methods-category/mathematical/discr/mesh/struct', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Structured meshing category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr/mesh', 'description': 'Meshing method category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}], 'properties': {'tier2': {'enum': ['mesh']}}}], 'properties': {'tier3': {'enum': ['struct']}}}, {'$id': 'methods-category/mathematical/discr/mesh/nstruct', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Unstructured meshing category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr/mesh', 'description': 'Meshing method category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}], 'properties': {'tier2': {'enum': ['mesh']}}}], 'properties': {'tier3': {'enum': ['nstruct']}}}, {'$id': 'methods-category/mathematical/discr/mesh/enum-options', 'hybridMesh': {'enum': ['hybrid']}, 'unstructuredMesh': {'enum': ['nstruct']}, 'structuredMesh': {'enum': ['struct']}}, {'$id': 'methods-category/mathematical/discr/mesh/hybrid', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Hybrid meshing category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr/mesh', 'description': 'Meshing method category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}], 'properties': {'tier2': {'enum': ['mesh']}}}], 'properties': {'tier3': {'enum': ['hybrid']}}}, {'$id': 'methods-category/mathematical/discr/mesh/struct/cartesian', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Cartesian grid schema', 'allOf': [{'$id': 'methods-category/mathematical/discr/mesh/struct', 'description': 'Structured meshing category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr/mesh', 'description': 'Meshing method category schema', 'allOf': [{'$id': 'methods-category/mathematical/discr', 'description': 'Discretization category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['discr']}}}], 'properties': {'tier2': {'enum': ['mesh']}}}], 'properties': {'tier3': {'enum': ['struct']}}}], 'properties': {'type': {'enum': ['cartesian']}}}, {'$id': 'methods-category/mathematical/discr/mesh/struct/enum-options', 'cartesian': {'enum': ['cartesian']}}, {'$id': 'methods-category/mathematical/diff/fd', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Finite difference method category schema', 'allOf': [{'$id': 'methods-category/mathematical/diff', 'description': 'Numerical differentiation category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['diff']}}}], 'properties': {'tier2': {'enum': ['fd']}}}, {'$id': 'methods-category/mathematical/diff/enum-options', 'finiteDifference': {'enum': ['fd']}}, {'$id': 'methods-category/mathematical/intgr/diffeq', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Methods for the numerical integration of differential equations', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['diffeq']}}}, {'$id': 'methods-category/mathematical/intgr/analytic', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Analytic integral category schema', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['analytic']}}}, {'$id': 'methods-category/mathematical/intgr/numquad', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Methods for the numerical quadrature', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['numquad']}}}, {'$id': 'methods-category/mathematical/intgr/enum-options', 'analytic': {'enum': ['analytic']}, 'differentialEquation': {'enum': ['diffeq']}, 'numericalQuadrature': {'enum': ['numquad']}, 'transformation': {'enum': ['transf']}}, {'$id': 'methods-category/mathematical/intgr/transf', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Integral transform methods', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['transf']}}}, {'$id': 'methods-category/mathematical/intgr/transf/fourier', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Fourier transform methods', 'allOf': [{'$id': 'methods-category/mathematical/intgr/transf', 'description': 'Integral transform methods', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['transf']}}}], 'properties': {'type': {'enum': ['fourier']}}}, {'$id': 'methods-category/mathematical/intgr/transf/enum-options', 'fourierTransformation': {'enum': ['fourier']}}, {'$id': 'methods-category/mathematical/intgr/numquad/newcot', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Newton-Cotes quadrature rules', 'allOf': [{'$id': 'methods-category/mathematical/intgr/numquad', 'description': 'Methods for the numerical quadrature', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['numquad']}}}], 'properties': {'tier3': {'enum': ['newcot']}}}, {'$id': 'methods-category/mathematical/intgr/numquad/gauss', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Gaussian quadrature rules', 'allOf': [{'$id': 'methods-category/mathematical/intgr/numquad', 'description': 'Methods for the numerical quadrature', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['numquad']}}}], 'properties': {'tier3': {'enum': ['gauss']}}}, {'$id': 'methods-category/mathematical/intgr/numquad/enum-options', 'gaussQuadrature': {'enum': ['gauss']}, 'newtonCotes': {'enum': ['newcot']}}, {'$id': 'methods-category/mathematical/intgr/analytic/volume', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'analytic volume integral category schema', 'allOf': [{'$id': 'methods-category/mathematical/intgr/analytic', 'description': 'Analytic integral category schema', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['analytic']}}}], 'properties': {'type': {'enum': ['volume']}, 'subtype': {'enum': ['sphere', 'cube', 'rect-prism', 'tri-prism', 'cylinder', 'cone', 'tetrahedron', 'sq-pyr']}}}, {'$id': 'methods-category/mathematical/intgr/analytic/enum-options', 'volume': {'enum': ['volume']}, 'volumeSubtypes': {'enum': ['sphere', 'cube', 'rect-prism', 'tri-prism', 'cylinder', 'cone', 'tetrahedron', 'sq-pyr']}}, {'$id': 'methods-category/mathematical/intgr/diffeq/order1', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Categories for the numerical integration of differential equations', 'allOf': [{'$id': 'methods-category/mathematical/intgr/diffeq', 'description': 'Methods for the numerical integration of differential equations', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['diffeq']}}}], 'properties': {'tier3': {'enum': ['order1']}}}, {'$id': 'methods-category/mathematical/intgr/diffeq/order2', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Categories for the numerical integration of differential equations', 'allOf': [{'$id': 'methods-category/mathematical/intgr/diffeq', 'description': 'Methods for the numerical integration of differential equations', 'allOf': [{'$id': 'methods-category/mathematical/intgr', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['intgr']}}}], 'properties': {'tier2': {'enum': ['diffeq']}}}], 'properties': {'tier3': {'enum': ['order2']}}}, {'$id': 'methods-category/mathematical/intgr/diffeq/enum-options', 'firstOrder': {'enum': ['order1']}, 'secondOrder': {'enum': ['order2']}}, {'$id': 'methods-category/mathematical/opt/diff', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}, {'$id': 'methods-category/mathematical/opt/ndiff', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Optimization methods for non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['ndiff']}}}, {'$id': 'methods-category/mathematical/opt/enum-options', 'differentiable': {'enum': ['diff']}, 'nonDifferentiable': {'enum': ['ndiff']}, 'rootFinding': {'enum': ['root']}}, {'$id': 'methods-category/mathematical/opt/root', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Root finding category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['root']}}}, {'$id': 'methods-category/mathematical/opt/ndiff/direct', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Direct algorithms for the optimization of non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/ndiff', 'description': 'Optimization methods for non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['ndiff']}}}], 'properties': {'tier3': {'enum': ['direct']}}}, {'$id': 'methods-category/mathematical/opt/ndiff/enum-options', 'direct': {'enum': ['direct']}, 'population': {'enum': ['pop']}, 'stochastic': {'enum': ['stoch']}}, {'$id': 'methods-category/mathematical/opt/ndiff/stoch', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Stochastic algorithms for the optmization of non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/ndiff', 'description': 'Optimization methods for non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['ndiff']}}}], 'properties': {'tier3': {'enum': ['stoch']}}}, {'$id': 'methods-category/mathematical/opt/ndiff/pop', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Population algorithms for the optmization of non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/ndiff', 'description': 'Optimization methods for non-differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['ndiff']}}}], 'properties': {'tier3': {'enum': ['pop']}}}, {'$id': 'methods-category/mathematical/opt/diff/bracket', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Bracket algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['bracket']}}}, {'$id': 'methods-category/mathematical/opt/diff/order1', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'First order algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['order1']}}}, {'$id': 'methods-category/mathematical/opt/diff/order2', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Second order algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['order2']}}}, {'$id': 'methods-category/mathematical/opt/diff/enum-options', 'bracketing': {'enum': ['bracket']}, 'localDescent': {'enum': ['local']}, 'firstOrder': {'enum': ['order1']}, 'secondOrder': {'enum': ['order2']}, 'nOrder': {'enum': ['ordern']}}, {'$id': 'methods-category/mathematical/opt/diff/ordern', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Mixed order and higher order algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['ordern']}}}, {'$id': 'methods-category/mathematical/opt/diff/local', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Local descent methods for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['local']}}}, {'$id': 'methods-category/mathematical/opt/diff/ordern/enum-options', 'conjugateGradient': {'enum': ['cg']}}, {'$id': 'methods-category/mathematical/opt/diff/ordern/cg', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'conjugate gradient method schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff/ordern', 'description': 'Mixed order and higher order algorithms for the optimization of differentiable functions', 'allOf': [{'$id': 'methods-category/mathematical/opt/diff', 'description': 'Optimization methods for differentiable functions category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['diff']}}}], 'properties': {'tier3': {'enum': ['ordern']}}}], 'properties': {'type': {'enum': ['cg']}}}, {'$id': 'methods-category/mathematical/opt/root/bracket', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Bracketing method for finding roots category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/root', 'description': 'Root finding category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['root']}}}], 'properties': {'tier3': {'enum': ['bracket']}}}, {'$id': 'methods-category/mathematical/opt/root/iter', '$schema': 'http://json-schema.org/draft-04/schema#', 'description': 'Iterative method for root finding category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt/root', 'description': 'Root finding category schema', 'allOf': [{'$id': 'methods-category/mathematical/opt', 'description': 'Integration category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['opt']}}}], 'properties': {'tier2': {'enum': ['root']}}}], 'properties': {'tier3': {'enum': ['iterative']}}}, {'$id': 'methods-category/mathematical/opt/root/enum-options', 'iterative': {'enum': ['iterative']}, 'bracketing': {'enum': ['bracket']}}, {'$id': 'in-memory-entity/named-defaultable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named defaultable in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}]}, {'$id': 'in-memory-entity/named-defaultable-has-metadata', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}, {'$id': 'in-memory-entity/named-defaultable-runtime-items', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'in-memory-entity/named', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Named in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}]}, {'$id': 'in-memory-entity/defaultable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Defaultable in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}]}, {'$id': 'in-memory-entity/base', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/path', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}, {'$id': 'system/entity-reference', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, {'$id': 'system/creator', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Creator entity reference schema', 'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}], 'properties': {'cls': {'description': 'Creator class', 'type': 'string', 'enum': ['User']}}}, {'$id': 'system/use-values', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'use values schema', 'properties': {'useValues': {'type': 'boolean'}}}, {'$id': 'system/set', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Entity set schema', 'properties': {'isEntitySet': {'type': 'boolean'}, 'entitySetType': {'type': 'string'}, 'entityCls': {'type': 'string'}}}, {'$id': 'system/schema-version', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}, {'$id': 'system/in-set', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'System in-set schema', 'properties': {'inSet': {'type': 'array', 'items': {'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, {'type': 'object', 'properties': {'type': {'type': 'string'}, 'index': {'type': 'number'}}}]}}}}, {'$id': 'system/database-source', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'database source schema', 'description': 'information about a database source', 'type': 'object', 'properties': {'id': {'description': 'ID string for the materials uploaded from a third party source inside the third party source. For materialsproject.org an example ID is mp-32', 'type': 'string'}, 'source': {'description': 'Third party source name, e.g. materials project, 2dmatpedia, ICSD, etc.', 'type': 'string'}, 'origin': {'description': 'A flag that is true when material is initially imported from a third party * (as opposed to being independently designed from scratch).', 'type': 'boolean', 'comment': 'Deprecated. To be removed.'}, 'data': {'description': 'Original response from external source.', 'type': 'object'}, 'doi': {'description': 'Digital Object Identifier, e.g. 10.1088/0953-8984/25/10/105506', 'type': 'string'}, 'url': {'description': 'The URL of the original record, e.g. https://next-gen.materialsproject.org/materials/mp-48', 'type': 'string', 'comment': 'ToDo: update to use URI type per https://json-schema.org/understanding-json-schema/reference/string#resource-identifiers'}}, 'required': ['id', 'source', 'origin']}, {'$id': 'system/name', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/owner', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Entity owner reference schema', 'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}], 'properties': {'cls': {'description': 'Entity owner class', 'type': 'string', 'enum': ['Account']}}}, {'$id': 'system/scope', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'scope schema', 'properties': {'scope': {'type': 'string'}}}, {'$id': 'system/path-entity', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/metadata', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}, {'$id': 'system/history', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'history schema', 'properties': {'history': {'type': 'array', 'items': {'type': 'object', 'properties': {'id': {'type': 'string'}, 'revision': {'type': 'number'}}, 'required': ['id', 'revision']}}}}, {'$id': 'system/-project', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Project entity reference schema', 'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}], 'properties': {'cls': {'description': 'Project class', 'type': 'string', 'enum': ['Project']}}}, {'$id': 'system/description', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'extended base entity schema', 'properties': {'description': {'description': 'entity description', 'type': 'string'}, 'descriptionObject': {'type': 'object'}}}, {'$id': 'system/is-multi-material', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'is multi schema', 'properties': {'isMultiMaterial': {'type': 'boolean'}}}, {'$id': 'system/defaultable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/timestampable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'timestampable entity schema', 'properties': {'createdAt': {'description': 'entity creation time', 'type': 'string', 'format': 'date-time'}, 'updatedAt': {'description': 'entity last modification time', 'type': 'string', 'format': 'date-time'}, 'createdBy': {'oneOf': [{'type': 'string'}, {'type': 'number'}]}, 'updatedBy': {'oneOf': [{'type': 'string'}, {'type': 'number'}]}}}, {'$id': 'system/-parent-job', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Parent job entity reference schema', 'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}], 'properties': {'cls': {'description': 'Job class', 'type': 'string', 'enum': ['Job']}}}, {'$id': 'system/sharing', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'extended sharing schema', 'properties': {'sharedCount': {'type': 'number'}}}, {'$id': 'system/bankable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'bankable schema', 'properties': {'exabyteId': {'description': 'Identity of the corresponding bank entity', 'type': 'string'}, 'hash': {'description': 'Hash string which is calculated based on the meaningful fields of the entity. Used to identify equal entities.', 'type': 'string'}}}, {'$id': 'system/-material', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Material entity reference schema', 'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}], 'properties': {'cls': {'description': 'Material class', 'type': 'string', 'enum': ['Material']}}}, {'$id': 'system/job-extended', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'extended job schema', 'properties': {'mode': {'type': 'string'}, 'isExternal': {'type': 'boolean'}, '_materials': {'type': 'array', 'items': {'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}]}}, '_materialsSet': {'allOf': [{'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}]}, 'purged': {'type': 'boolean'}, 'purgedAt': {'type': 'number'}, 'dataset': {'type': 'object'}}}, {'$id': 'system/creator-account', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'creator account schema', 'properties': {'creatorAccount': {'$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}}}, {'$id': 'system/tags', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}, {'$id': 'system/is-outdated', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'is outdated schema', 'properties': {'isOutdated': {'type': 'boolean'}}}, {'$id': 'system/soft-removable', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'soft removable entity schema', 'properties': {'removedAt': {'description': 'Timestamp of the moment when entity was removed', 'type': 'string'}, 'removed': {'description': 'Identifies that entity was removed', 'type': 'boolean'}}}, {'$id': 'system/status', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'status schema', 'properties': {'status': {'type': 'string'}, 'statusTrack': {'type': 'array', 'items': {'type': 'object', 'properties': {'trackedAt': {'type': 'number'}, 'status': {'type': 'string'}, 'repetition': {'type': 'number'}}, 'required': ['trackedAt', 'status']}}}}, {'$id': 'system/consistency-check', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}, {'$id': 'system/file-source', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'file source schema', 'description': 'file source with the information inside', 'type': 'object', 'properties': {'extension': {'description': 'file extension', 'type': 'string'}, 'filename': {'description': 'file name without extension', 'type': 'string'}, 'text': {'description': 'file content as raw text', 'type': 'string'}, 'hash': {'description': 'MD5 hash based on file content', 'type': 'string'}}, 'required': ['filename', 'text', 'hash']}, {'$id': 'system/message', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'message schema', 'description': 'communication message between Rupy and web application.', 'properties': {'header': {'type': 'object', 'properties': {'entity': {'type': 'object', 'properties': {'_id': {'description': 'job identifier', 'type': 'string'}, 'name': {'description': 'entity name.', 'type': 'string', 'enum': ['job', 'unit']}, 'flowchartId': {'description': 'unit identifier within the workflow', 'type': 'string'}, 'probe': {'description': 'source of the message.', 'type': 'string', 'enum': ['monitor', 'postprocessor']}}, 'required': ['_id', 'name']}, 'version': {'description': 'Rupy-Webapp communication schema version.', 'type': 'string'}, 'timestamp': {'description': 'Timestamp of the message.', 'type': 'number'}}, 'required': ['entity', 'version', 'timestamp']}, 'payload': {'description': 'Actual payload of the message.', 'type': 'object'}}, 'required': ['header', 'payload']}, {'$id': 'models-directory/gga', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model generalized gradient approximation', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/dft/ksdft/gga', 'title': 'DFT GGA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['gga']}}}, 'parameters': {'allOf': [{'$id': 'model/mixins/dft/gga-functional', 'title': 'GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pbe', 'pbesol']}}}, {'anyOf': [{'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/hubbard', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/mgga', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model meta generalized gradient approximation', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/dft/ksdft/mgga', 'title': 'DFT meta-GGA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['mgga']}}}, 'parameters': {'allOf': [{'$id': 'model/mixins/dft/mgga-functional', 'title': 'Meta-GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['scan']}}}, {'anyOf': [{'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/hubbard', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/gw', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model gw approximation', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/abin/gw', 'title': 'GW category schema', 'allOf': [{'$id': 'models-category/pb/qm/abin', 'title': 'Ab initio category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['abin']}}}], 'properties': {'type': {'enum': ['gw']}, 'subtype': {'enum': ['g0w0', 'evgw0', 'evgw']}}}, 'parameters': {'allOf': [{'properties': {'require': {'description': 'TODO: Use regex once schema draft version has been updated', '$id': 'core/reusable/category-path', 'title': 'category path schema', 'type': 'string'}}}, {'oneOf': [{'$id': 'model/mixins/dft/lda-functional', 'title': 'LDA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pz']}}}, {'$id': 'model/mixins/dft/gga-functional', 'title': 'GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pbe', 'pbesol']}}}, {'$id': 'model/mixins/dft/mgga-functional', 'title': 'Meta-GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['scan']}}}]}, {'anyOf': [{'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/lda', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model local density approximation', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/dft/ksdft/lda', 'title': 'DFT LDA functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['lda']}}}, 'parameters': {'allOf': [{'$id': 'model/mixins/dft/lda-functional', 'title': 'LDA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pz']}}}, {'anyOf': [{'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/hubbard', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/double-hybrid', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model double hybrid functional', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/dft/ksdft/double-hybrid', 'title': 'DFT double hybrid functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['double-hybrid']}}}, 'parameters': {'allOf': [{'$id': 'model/mixins/dft/double-hybrid-functional', 'title': 'Double hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['b2plyp']}}}, {'anyOf': [{'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/re', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model regression', 'description': 'machine learning model type/subtype schema', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/st/det/ml/re', 'title': 'regression model category schema', 'allOf': [{'$id': 'models-category/st/det/ml', 'title': 'machine learning model category schema', 'allOf': [{'$id': 'models-category/st/det', 'title': 'deterministic model category schema', 'allOf': [{'$id': 'models-category/st', 'title': 'statistical model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['st']}}}], 'properties': {'tier2': {'enum': ['det']}}}], 'properties': {'tier3': {'enum': ['ml']}}}], 'properties': {'type': {'enum': ['re']}}}, 'parameters': {'type': 'object'}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/hybrid', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model hybrid functional', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'categories': {'$id': 'models-category/pb/qm/dft/ksdft/hybrid', 'title': 'DFT hybrid functional category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft/ksdft', 'title': 'Kohn-Sham DFT category schema', 'allOf': [{'$id': 'models-category/pb/qm/dft', 'title': 'Density functional theory category schema', 'allOf': [{'$id': 'models-category/pb/qm', 'title': 'Quantum mechanical category schema', 'allOf': [{'$id': 'models-category/pb', 'title': 'physics-based model category schema', 'allOf': [{'$id': 'core/reusable/categories', 'title': 'categories', 'description': 'Used to categorize entities such as models and methods', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}], 'properties': {'tier1': {'enum': ['pb']}}}], 'properties': {'tier2': {'enum': ['qm']}}}], 'properties': {'tier3': {'enum': ['dft']}}}], 'properties': {'type': {'enum': ['ksdft']}}}], 'properties': {'subtype': {'enum': ['hybrid']}}}, 'parameters': {'allOf': [{'$id': 'model/mixins/dft/hybrid-functional', 'title': 'Hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['hse06', 'b3lyp']}}}, {'anyOf': [{'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/hubbard', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}]}]}}, 'required': ['categories', 'parameters']}, {'$id': 'models-directory/legacy/ml', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy model regression', 'allOf': [{'$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}], 'properties': {'type': {'enum': ['ml']}, 'subtype': {'enum': ['re']}}}, {'$id': 'models-directory/legacy/unknown', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy model unknown', 'allOf': [{'$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}], 'properties': {'type': {'enum': ['unknown']}, 'subtype': {'enum': ['unknown']}}}, {'$id': 'models-directory/legacy/dft', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'legacy model density functional theory', 'allOf': [{'$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}, {'oneOf': [{'properties': {'subtype': {'enum': ['lda']}, 'functional': {'enum': ['pz', 'pw', 'vwn', 'other']}}}, {'properties': {'subtype': {'enum': ['gga']}, 'functional': {'enum': ['pbe', 'pbesol', 'pw91', 'other']}}}, {'properties': {'subtype': {'enum': ['hybrid']}, 'functional': {'enum': ['b3lyp', 'hse06']}}}]}], 'properties': {'type': {'enum': ['dft']}}, 'definitions': {'lda': {'properties': {'subtype': {'enum': ['lda']}, 'functional': {'enum': ['pz', 'pw', 'vwn', 'other']}}}, 'gga': {'properties': {'subtype': {'enum': ['gga']}, 'functional': {'enum': ['pbe', 'pbesol', 'pw91', 'other']}}}, 'hybrid': {'properties': {'subtype': {'enum': ['hybrid']}, 'functional': {'enum': ['b3lyp', 'hse06']}}}}}, {'$id': 'workflow/subworkflow', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Subworkflow', 'type': 'object', 'allOf': [{'$id': 'workflow/base-flow', 'title': 'BaseFlow', 'type': 'object', 'properties': {'_id': {'description': 'subworkflow identity', 'type': 'string'}, 'name': {'description': 'Human-readable name of the subworkflow. e.g. Total-energy', 'type': 'string'}, 'properties': {'description': 'Array of characteristic properties calculated by this subworkflow', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}}, 'required': ['name', 'units']}], 'properties': {'units': {'description': 'Contains the Units of the subworkflow', 'type': 'array', 'items': {'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}}, 'model': {'description': 'Model used inside the subworkflow', '$id': 'model', 'title': 'base model', 'properties': {'type': {'description': 'general type of the model, eg. `dft`', 'type': 'string'}, 'subtype': {'description': 'general subtype of the model, eg. `lda`', 'type': 'string'}, 'method': {'$id': 'method', 'title': 'base method', 'properties': {'type': {'description': 'general type of this method, eg. `pseudopotential`', 'type': 'string'}, 'subtype': {'description': 'general subtype of this method, eg. `ultra-soft`', 'type': 'string'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}, 'data': {'description': 'additional data specific to method, eg. array of pseudopotentials', 'type': 'object'}}, 'required': ['type', 'subtype']}}, 'additionalProperties': True, 'required': ['type', 'subtype', 'method']}, 'application': {'description': 'information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'isDraft': {'description': 'Defines whether to store the results/properties extracted in this unit to properties collection', 'type': 'boolean', 'default': False}}, 'required': ['model', 'application']}, {'$id': 'workflow/scope', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'workflow scope schema', 'type': 'object', 'properties': {'global': {'type': 'object', 'additionalProperties': True}, 'local': {'type': 'object', 'additionalProperties': True}}, 'required': ['global', 'local']}, {'$id': 'workflow/base-flow', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'BaseFlow', 'type': 'object', 'properties': {'_id': {'description': 'subworkflow identity', 'type': 'string'}, 'name': {'description': 'Human-readable name of the subworkflow. e.g. Total-energy', 'type': 'string'}, 'properties': {'description': 'Array of characteristic properties calculated by this subworkflow', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}}, 'required': ['name', 'units']}, {'$id': 'workflow/base', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'base workflow schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'properties': {'description': 'Array of characteristic properties calculated by this workflow (TODO: add enums)', 'type': 'array', 'items': {'description': 'property names, eg. `band_gaps`, `band_structure`', 'oneOf': [{'type': 'string'}, {'type': 'object'}]}}, 'isUsingDataset': {'description': 'Whether to use the dataset tab in the job designer. Mutually exclusive with using the materials tab.', 'type': 'boolean'}, 'workflows': {'description': 'Array of workflows with the same schema as the current one.', 'type': 'array', 'items': {'type': 'object'}}}}, {'$id': 'workflow/unit', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/condition', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'condition unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['condition']}, 'input': {'description': 'Input information for condition.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'statement': {'description': "Condition statement. e.g. 'abs(x-total_energy) < 1e-5'", 'type': 'string'}, 'then': {'description': 'Flowchart ID reference for `then` part of the condition.', 'type': 'string'}, 'else': {'description': 'Flowchart ID reference for `else` part of the condition.', 'type': 'string'}, 'maxOccurrences': {'description': 'Maximum occurrence of the condition, usable for loops.', 'type': 'integer'}, 'throwException': {'description': 'Throw exception on reaching to maximum occurence.', 'type': 'boolean'}}, 'required': ['input', 'statement', 'then', 'else', 'maxOccurrences']}, {'$id': 'workflow/unit/assignment', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'assignment unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assignment']}, 'input': {'description': 'Input information for assignment. if omitted, means that it is an initialization unit, otherwise it is an assignment.', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemScope', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}}, 'operand': {'description': "Name of the global variable. e.g. 'x'", 'type': 'string'}, 'value': {'description': "Value of the variable. The value content could be a simple integer, string or a python expression. e.g. '0' (initialization), 'sin(x)+1' (expression)", 'type': 'string'}}, 'required': ['name', 'operand', 'value']}, {'$id': 'workflow/unit/processing', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}, {'$id': 'workflow/unit/reduce', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'reduce unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['reduce']}, 'mapFlowchartId': {'description': 'corresponding map unit flowchart ID', 'type': 'string'}, 'input': {'description': 'input information for reduce unit', 'type': 'array', 'items': {'type': 'object', 'properties': {'operation': {'description': 'reduce operation, e.g. aggregate', 'type': 'string'}, 'arguments': {'description': 'arguments which are passed to reduce operation function', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['operation', 'arguments']}}}, 'required': ['mapFlowchartId', 'input']}, {'$id': 'workflow/unit/assertion', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'assertion unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['assertion']}, 'statement': {'type': 'string', 'description': 'The statement to be evaluated'}, 'errorMessage': {'type': 'string', 'description': 'The error message to be displayed if the assertion fails'}}, 'required': ['name', 'statement']}, {'$id': 'workflow/unit/io', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'data IO unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['io']}, 'subtype': {'enum': ['input', 'output', 'dataFrame']}, 'source': {'enum': ['api', 'db', 'object_storage']}, 'input': {'type': 'array', 'items': {'anyOf': [{'$id': 'workflow/unit/io/api', 'title': 'data IO rest API input schema', 'type': 'object', 'properties': {'endpoint': {'description': 'rest API endpoint', 'type': 'string'}, 'endpoint_options': {'description': 'rest API endpoint options', 'type': 'object'}, 'name': {'description': 'the name of the variable in local scope to save the data under', 'type': 'string'}}, 'required': ['endpoint', 'endpoint_options']}, {'$id': 'workflow/unit/io/db', 'title': 'data IO database input/output schema', 'type': 'object', 'oneOf': [{'properties': {'ids': {'description': 'IDs of item to retrieve from db', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['ids']}, {'properties': {'collection': {'description': 'db collection name', 'type': 'string'}, 'draft': {'description': 'whether the result should be saved as draft', 'type': 'boolean', 'default': True}}, 'required': ['collection', 'draft']}]}, {'$id': 'workflow/unit/io/object-storage', 'title': 'object_storage io schema', 'type': 'object', 'allof': [{'$id': 'core/reusable/file-metadata', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}], 'properties': {'objectData': {'$id': 'core/reusable/object-storage-container-data', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}, 'overwrite': {'description': 'if a file with the same filename already exists, whether to overwrite the old file', 'type': 'boolean', 'default': False}}, 'required': ['objectData', 'basename']}]}}}, 'required': ['subtype', 'source', 'input']}, {'$id': 'workflow/unit/map', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'map unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['map']}, 'workflowId': {'description': 'Id of workflow to run inside map', 'type': 'string'}, 'input': {'description': 'Input information for map.', 'type': 'object', 'properties': {'target': {'description': 'Name of the target variable to substitute using the values below. e.g. K_POINTS', 'type': 'string'}, 'scope': {'description': 'Scope to retrieve `values` from, global or flowchartId. Optional if `values` is given.', 'type': 'string'}, 'name': {'description': 'Name of the variable inside the scope to retrieve `values` from. Optional if `values` is given.', 'type': 'string'}, 'values': {'description': 'Sequence of values for the target Jinja variable. Optional if `scope` and `name` are given. This can be used for map-reduce type parallel execution', 'type': 'array', 'items': {'oneOf': [{'type': 'string'}, {'type': 'number'}, {'type': 'object'}]}}, 'useValues': {'type': 'boolean'}}, 'required': ['target']}}, 'required': ['input', 'workflowId']}, {'$id': 'workflow/unit/execution', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}, {'$id': 'workflow/unit/input/-inputItem', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit input item schema for physics-based simulation engines', 'type': 'object', 'properties': {'name': {'description': 'Input file name. e.g. pw_scf.in', 'type': 'string'}, 'content': {'description': "Content of the input file. e.g. &CONTROL    calculation='scf' ...", 'type': 'string'}}, 'required': ['name', 'content']}, {'$id': 'workflow/unit/input/-input', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit input schema for physics-based simulation engines', 'type': 'object', 'properties': {'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'oneOf': [{'$id': 'workflow/unit/input/-inputItem', 'title': 'execution unit input item schema for physics-based simulation engines', 'type': 'object', 'properties': {'name': {'description': 'Input file name. e.g. pw_scf.in', 'type': 'string'}, 'content': {'description': "Content of the input file. e.g. &CONTROL    calculation='scf' ...", 'type': 'string'}}, 'required': ['name', 'content']}, {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}]}}}}, {'$id': 'workflow/unit/input/-inputItemId', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}, {'$id': 'workflow/unit/input/-inputItemScope', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'workflow unit input schema', 'type': 'object', 'properties': {'scope': {'description': "Scope of the variable. e.g. 'global' or 'flowchart_id_2'", 'type': 'string'}, 'name': {'description': 'Name of the input data. e.g. total_energy', 'type': 'string'}}, 'required': ['scope', 'name']}, {'$id': 'workflow/unit/io/object-storage', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'object_storage io schema', 'type': 'object', 'allof': [{'$id': 'core/reusable/file-metadata', 'title': 'file_metadata', 'properties': {'pathname': {'description': 'Relative path to the directory that contains the file.', 'type': 'string'}, 'basename': {'description': 'Basename of the file', 'type': 'string', '$comment': 'isGenerative:true'}, 'filetype': {'description': 'What kind of file this is, e.g. image / text', 'type': 'string'}}}], 'properties': {'objectData': {'$id': 'core/reusable/object-storage-container-data', 'title': 'Object Storage Container Data', 'properties': {'CONTAINER': {'description': 'Object storage container for the file', 'type': 'string'}, 'NAME': {'description': 'Name of the file inside the object storage bucket', 'type': 'string'}, 'PROVIDER': {'description': 'Object storage provider', 'type': 'string'}, 'REGION': {'description': 'Region for the object container specified in Container', 'type': 'string'}, 'SIZE': {'description': 'Size of the file in bytes', 'type': 'integer'}, 'TIMESTAMP': {'description': 'Unix timestamp showing when the file was last modified', 'type': 'string'}}, 'required': ['CONTAINER', 'NAME', 'PROVIDER', 'REGION']}, 'overwrite': {'description': 'if a file with the same filename already exists, whether to overwrite the old file', 'type': 'boolean', 'default': False}}, 'required': ['objectData', 'basename']}, {'$id': 'workflow/unit/io/api', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'data IO rest API input schema', 'type': 'object', 'properties': {'endpoint': {'description': 'rest API endpoint', 'type': 'string'}, 'endpoint_options': {'description': 'rest API endpoint options', 'type': 'object'}, 'name': {'description': 'the name of the variable in local scope to save the data under', 'type': 'string'}}, 'required': ['endpoint', 'endpoint_options']}, {'$id': 'workflow/unit/io/db', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'data IO database input/output schema', 'type': 'object', 'oneOf': [{'properties': {'ids': {'description': 'IDs of item to retrieve from db', 'type': 'array', 'items': {'type': 'string'}}}, 'required': ['ids']}, {'properties': {'collection': {'description': 'db collection name', 'type': 'string'}, 'draft': {'description': 'whether the result should be saved as draft', 'type': 'boolean', 'default': True}}, 'required': ['collection', 'draft']}]}, {'$id': 'workflow/unit/runtime/-runtime-item-full-object', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'full result schema', 'type': 'object', 'properties': {'name': {'description': "The name of this item. e.g. 'my_custom_property. <OTHER FIELDS TO BE ADDED>'", 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/runtime-item', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}, {'$id': 'workflow/unit/runtime/runtime-items', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}, {'$id': 'workflow/unit/runtime/-runtime-item-string', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}, {'$id': 'workflow/unit/runtime/-runtime-item-name-object', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'model/model-without-method', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}, {'$id': 'model/categorized-model', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'categorized model', 'allOf': [{'$id': 'model/model-without-method', 'title': 'model without method schema (base)', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'type': 'object', 'description': 'Model parameters defined in-place or via model mixins'}, 'reference': {'$id': 'core/reference/literature', 'title': 'literature reference schema', 'properties': {'type': {'enum': ['literature']}, 'doi': {'type': 'string', 'description': 'Digital Object Identifier of the reference.'}, 'isbn': {'type': 'string', 'description': 'International Standard Book Number of the reference.'}, 'issn': {'type': 'string', 'description': 'International Standard Serial Number of the reference.'}, 'url': {'type': 'string', 'description': 'Internet address of the reference.'}, 'title': {'type': 'string', 'description': 'Title of the work.'}, 'publisher': {'type': 'string', 'description': 'Publisher of the work.'}, 'journal': {'type': 'string', 'description': 'Journal in which the work appeared.'}, 'volume': {'type': 'string', 'description': 'Volume of the series in which the work appeared.'}, 'year': {'type': 'string', 'description': 'Year in which the reference was published.'}, 'issue': {'type': 'string', 'description': 'Issue of the collection in which the work appeared.'}, 'pages': {'type': 'object', 'description': 'Start and end pages of the work.', 'allOf': [{'$id': 'core/reference/literature/pages', 'title': 'pages schema', 'type': 'object', 'properties': {'start': {'type': 'string'}, 'end': {'type': 'string'}}, 'required': ['start']}]}, 'authors': {'type': 'array', 'description': 'List of authors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'editors': {'type': 'array', 'description': 'List of editors of the work.', 'items': {'$id': 'core/reference/literature/name', 'title': 'experiment author schema', 'type': 'object', 'properties': {'first': {'type': 'string'}, 'middle': {'type': 'string'}, 'last': {'type': 'string'}, 'affiliation': {'type': 'string'}}, 'required': ['first', 'last']}}, 'reference': {'type': 'array', 'items': {'type': 'object'}, 'description': 'References cited by the work. Reference objects can nest as deeply as needed. This is useful, for example, when tracking the history of a value referenced in a scholarly article; the top level reference would contain information about where the data was accessed while the nested reference would contain information about where it was originally published.'}}}}, 'required': ['categories', 'parameters']}], 'properties': {'method': {'$id': 'method/categorized-method', 'title': 'categorized method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'units': {'type': 'array', 'items': {'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}}}, 'required': ['units']}}, 'required': ['method']}, {'$id': 'model/model-parameters', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'ModelParameters', 'allOf': [{'$id': 'model/mixins/hubbard', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}, {'$id': 'model/mixins/spin-polarization', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/spin-orbit-coupling', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'oneOf': [{'$id': 'model/mixins/dft/lda-functional', 'title': 'LDA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pz']}}}, {'$id': 'model/mixins/dft/gga-functional', 'title': 'GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pbe', 'pbesol']}}}, {'$id': 'model/mixins/dft/mgga-functional', 'title': 'Meta-GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['scan']}}}, {'$id': 'model/mixins/dft/hybrid-functional', 'title': 'Hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['hse06', 'b3lyp']}}}, {'$id': 'model/mixins/dft/double-hybrid-functional', 'title': 'Double hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['b2plyp']}}}]}]}, {'$id': 'model/mixins/spin-polarization', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Spin-polarization mixin', 'type': 'object', 'properties': {'spinPolarization': {'type': 'string', 'enum': ['collinear', 'non-collinear']}}}, {'$id': 'model/mixins/spin-orbit-coupling', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Spin-orbit coupling mixin', 'type': 'object', 'properties': {'spinOrbitCoupling': {'type': 'boolean'}}}, {'$id': 'model/mixins/dispersion-correction', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Dispersion correction mixin', 'type': 'object', 'properties': {'dispersionCorrection': {'type': 'string', 'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}}}, {'$id': 'model/mixins/enum-options', 'spinPolarization': {'enum': ['collinear', 'non-collinear']}, 'dispersionCorrection': {'enum': ['dft-d2', 'dft-d3', 'xdm', 'ts']}, 'hubbardType': {'enum': ['u']}}, {'$id': 'model/mixins/hubbard', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Hubbard model mixin', 'type': 'object', 'properties': {'hubbardType': {'type': 'string', 'enum': ['u']}}}, {'$id': 'model/mixins/dft/mgga-functional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Meta-GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['scan']}}}, {'$id': 'model/mixins/dft/double-hybrid-functional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Double hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['b2plyp']}}}, {'$id': 'model/mixins/dft/hybrid-functional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Hybrid functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['hse06', 'b3lyp']}}}, {'$id': 'model/mixins/dft/enum-options', 'lda': {'enum': ['pz']}, 'gga': {'enum': ['pbe', 'pbesol']}, 'mgga': {'enum': ['scan']}, 'hybrid': {'enum': ['hse06', 'b3lyp']}, 'doubleHybrid': {'enum': ['b2plyp']}}, {'$id': 'model/mixins/dft/gga-functional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'GGA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pbe', 'pbesol']}}}, {'$id': 'model/mixins/dft/lda-functional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'LDA functional mixin', 'type': 'object', 'properties': {'functional': {'type': 'string', 'enum': ['pz']}}}, {'$id': 'job/base', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'base job schema', 'allOf': [{'description': 'in-memory entity', '$id': 'in-memory-entity/named-defaultable-has-metadata', 'title': 'Named defaultable has metadata in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'system/metadata', 'title': 'metadata schema', 'properties': {'metadata': {'type': 'object'}}}]}], 'properties': {'rmsId': {'description': 'Identity used to track jobs originated from command-line', 'type': 'string'}, 'status': {'description': 'job status', 'enum': ['pre-submission', 'queued', 'submitted', 'active', 'finished', 'terminate-queued', 'terminated', 'error', 'deleted', 'timeout']}, 'startTime': {'description': 'Approximate start time of the job. e.g. within 10 min', 'type': 'string'}, 'workDir': {'description': 'The path to the working directory of this job, when the job originates from command-line', 'type': 'string'}, 'compute': {'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', '$id': 'job/compute', 'title': 'compute arguments schema', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}, '_project': {'description': 'Subset of the full information about the project that this job belongs to.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, '_material': {'description': 'Subset of the full information about the materials used inside this job.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, 'parent': {'description': 'Subset of the full information about the parent job for the job. Only a job from the same project can be assigned as a parent one.', '$id': 'system/entity-reference', 'title': 'entity reference schema', 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'cls': {'description': 'entity class', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}}}, 'runtimeContext': {'description': 'Context variables that the job will have access to at runtime', 'type': 'object'}, 'scopeTrack': {'description': 'history of the workflow scope on each update', 'type': 'array', 'items': {'type': 'object', 'properties': {'repetition': {'type': 'number'}, 'scope': {'$id': 'workflow/scope', 'title': 'workflow scope schema', 'type': 'object', 'properties': {'global': {'type': 'object', 'additionalProperties': True}, 'local': {'type': 'object', 'additionalProperties': True}}, 'required': ['global', 'local']}}}}}, 'required': ['status', 'compute', '_project']}, {'$id': 'job/compute', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'compute arguments schema', 'description': 'Custom keywords prefixed with validate correspond to custom validation methods implemented downstream', 'properties': {'queue': {'description': 'Name of the submission queues: https://docs.mat3ra.com/infrastructure/resource/queues/. Below enums are for Azure, then AWS circa 2022-08, hence the duplication.', 'type': 'string', 'enum': ['D', 'OR', 'OF', 'OFplus', 'SR', 'SF', 'SFplus', 'GPOF', 'GP2OF', 'GP4OF', 'GPSF', 'GP2SF', 'GP4SF', 'OR4', 'OR8', 'OR16', 'SR4', 'SR8', 'SR16', 'GOF', 'G4OF', 'G8OF', 'GSF', 'G4SF', 'G8SF']}, 'nodes': {'description': 'number of nodes used for the job inside the RMS.', 'type': 'integer', 'validateNodes': {}}, 'ppn': {'description': 'number of CPUs used for the job inside the RMS.', 'type': 'integer', 'validatePpn': {}}, 'timeLimit': {'description': "Wallclock time limit for computing a job. Clock format: 'hh:mm:ss'", 'type': 'string', 'validateTimeLimit': {}}, 'timeLimitType': {'description': 'Convention to use when reasoning about time limits', 'type': 'string', 'default': 'per single attempt', 'enum': ['per single attempt', 'compound']}, 'isRestartable': {'description': 'Job is allowed to restart on termination.', 'type': 'boolean', 'default': True}, 'notify': {'description': 'Email notification for the job: n - never, a - job aborted, b - job begins, e - job ends. Last three could be combined.', 'type': 'string'}, 'email': {'description': 'Email address to notify about job execution.', 'type': 'string'}, 'maxCPU': {'description': 'Maximum CPU count per node. This parameter is used to let backend job submission infrastructure know that this job is to be charged for the maximum CPU per node instead of the actual ppn. For premium/fast queues where resources are provisioned on-demand and exclusively per user.', 'type': 'integer'}, 'arguments': {'description': 'Optional arguments specific to using application - VASP, Quantum Espresso, etc. Specified elsewhere', 'type': 'object', 'default': {}, 'oneOf': [{'$id': 'software-directory/modeling/espresso/arguments', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}]}, 'cluster': {'description': 'Cluster where the job is executed. Optional on create. Required on job submission.', 'type': 'object', 'properties': {'fqdn': {'description': 'FQDN of the cluster. e.g. master-1-staging.exabyte.io', 'type': 'string'}, 'jid': {'description': "Job's identity in RMS. e.g. 1234.master-1-staging.exabyte.io", 'type': 'string'}}}, 'errors': {'description': 'Computation error. Optional. Appears only if something happens on jobs execution.', 'type': 'array', 'items': {'type': 'object', 'properties': {'domain': {'description': 'Domain of the error appearance (internal).', 'type': 'string', 'enum': ['rupy', 'alfred', 'celim', 'webapp']}, 'reason': {'description': 'Should be a short, unique, machine-readable error code string. e.g. FileNotFound', 'type': 'string'}, 'message': {'description': "Human-readable error message. e.g. 'File Not Found: /home/demo/data/project1/job-123/job-config.json'", 'type': 'string'}, 'traceback': {'description': 'Full machine-readable error traceback. e.g. FileNotFound', 'type': 'string'}}}}, 'excludeFilesPattern': {'description': 'A Python compatible regex to exclude files from upload. e.g. ^.*.txt& excludes all files with .txt suffix', 'type': 'string'}}, 'required': ['queue', 'nodes', 'ppn', 'timeLimit']}, {'$id': 'software-directory/scripting/jupyter-lab', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Jupyter Lab Application Schema', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['jupyterLab']}, 'flavor': {'enum': ['notebook']}, 'summary': {'enum': ['Jupyter Lab']}, 'version': {'enum': ['0.33.12']}, 'exec': {'enum': ['jupyter']}}}, {'$id': 'software-directory/scripting/shell', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Shell Scripting Language Schema', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['shell']}, 'flavor': {'enum': ['sh', 'bash', 'zsh', 'csh']}, 'summary': {'enum': ['Shell Script']}, 'version': {'enum': ['4.2.46']}, 'exec': {'enum': ['sh', 'bash', 'zsh', 'csh']}, 'arguments': {'description': 'Optional arguments passed to the Shell script', 'type': 'string'}, 'environment': {'description': 'Optional environment variables exported before running the Shell script', 'type': 'object'}}}, {'$id': 'software-directory/scripting/python', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'Python Programing Language Schema', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['python']}, 'flavor': {'enum': ['python2', 'python3']}, 'summary': {'enum': ['Python Script']}, 'version': {'enum': ['2.7.5', '3.6.1']}, 'exec': {'enum': ['python']}, 'arguments': {'description': 'Optional arguments passed to the Python script', 'type': 'string'}, 'environment': {'description': 'Optional environment variables exported before running the Python script', 'type': 'object'}, 'dependencies': {'description': 'Optional Python dependencies, e.g. amqp==1.4.6', 'type': 'array'}}}, {'$id': 'software-directory/scripting/unit/execution', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit schema for scripting-based applications', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}, {'$id': 'workflow/unit/input/-input', 'title': 'execution unit input schema for physics-based simulation engines', 'type': 'object', 'properties': {'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'oneOf': [{'$id': 'workflow/unit/input/-inputItem', 'title': 'execution unit input item schema for physics-based simulation engines', 'type': 'object', 'properties': {'name': {'description': 'Input file name. e.g. pw_scf.in', 'type': 'string'}, 'content': {'description': "Content of the input file. e.g. &CONTROL    calculation='scf' ...", 'type': 'string'}}, 'required': ['name', 'content']}, {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}]}}}}]}, {'$id': 'software-directory/modeling/nwchem', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'NWChem', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['NWChem']}, 'summary': {'enum': ['NWChem: a comprehensive and scalable open-source solution for large scale molecular simulations']}, 'version': {'enum': ['6.6', '7.0.2']}, 'exec': {'enum': ['nwchem']}}}, {'$id': 'software-directory/modeling/deepmd', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'DeePMD app schema', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['deepmd']}, 'summary': {'enum': ['DeePMD is a deep learning package that is based on neural network fitted first-principles data for many-body potential energy representation and molecular dynamics']}, 'version': {'enum': ['2.0.2']}, 'exec': {'enum': ['dp', 'lmp', 'python']}}}, {'$id': 'software-directory/modeling/espresso', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'espresso app schema', 'type': 'object', 'properties': {'name': {'enum': ['espresso']}, 'summary': {'enum': ['Quantum Espresso']}, 'version': {'enum': ['5.2.1', '5.4.0', '6.0.0', '6.3', '6.4.1', '6.5.0', '6.6.0', '6.7.0', '6.8.0', '7.0', '7.2', '7.3']}}}, {'$id': 'software-directory/modeling/vasp', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'vienna ab-inito simulation package', 'type': 'object', 'allOf': [{'$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}], 'properties': {'name': {'enum': ['vasp']}, 'summary': {'enum': ['vienna ab-initio simulation package']}, 'flavor': {'enum': ['vasp', 'vasp_nscf', 'vasp_bands']}, 'version': {'enum': ['5.3.5']}, 'exec': {'enum': ['vasp']}}}, {'$id': 'software-directory/modeling/unit/execution', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'execution unit schema for physics-based simulation engines (defined using espresso as example)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}, {'$id': 'workflow/unit/input/-input', 'title': 'execution unit input schema for physics-based simulation engines', 'type': 'object', 'properties': {'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'oneOf': [{'$id': 'workflow/unit/input/-inputItem', 'title': 'execution unit input item schema for physics-based simulation engines', 'type': 'object', 'properties': {'name': {'description': 'Input file name. e.g. pw_scf.in', 'type': 'string'}, 'content': {'description': "Content of the input file. e.g. &CONTROL    calculation='scf' ...", 'type': 'string'}}, 'required': ['name', 'content']}, {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}]}}}}]}, {'$id': 'software-directory/modeling/espresso/arguments', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'quantum espresso arguments schema', 'type': 'object', 'properties': {'nimage': {'description': 'Processors can be divided into different `images`, each corresponding to a different self-consistent or linear-response calculation, loosely coupled to others.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'npools': {'description': 'Each image can be subpartitioned into `pools`, each taking care of a group of k-points.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'nband': {'description': 'Each pool is subpartitioned into `band groups`, each taking care of a group of Kohn-Sham orbitals (also called bands, or wavefunctions).', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ntg': {'description': 'In order to allow good parallelization of the 3D FFT when the number of processors exceeds the number of FFT planes, FFTs on Kohn-Sham states are redistributed to `task` groups so that each group can process several wavefunctions at the same time.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}, 'ndiag': {'description': 'A further level of parallelization, independent on PW or k-point parallelization, is the parallelization of subspace diagonalization / iterative orthonormalization. Both operations required the diagonalization of arrays whose dimension is the number of Kohn-Sham states (or a small multiple of it). All such arrays are distributed block-like across the `linear-algebra group`, a subgroup of the pool of processors, organized in a square 2D grid. As a consequence the number of processors in the linear-algebra group is given by n2, where n is an integer; n2 must be smaller than the number of processors in the PW group. The diagonalization is then performed in parallel using standard linear algebra operations.', 'type': 'integer', 'default': 1, 'minimum': 1, 'maximum': 100}}, 'additionalProperties': False}, {'$id': 'software-directory/ml/exabyteml', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'exabyte machine learning engine schema', 'type': 'object', 'properties': {'name': {'enum': ['exabyteml']}, 'summary': {'enum': ['exabyte machine learning engine']}, 'version': {'enum': ['0.2.0']}}}, {'$id': 'software-directory/ml/unit/processing', 'oneOf': [{'$id': 'software-directory/ml/unit/processing/data-transformation', 'oneOf': [{'$id': 'software-directory/ml/unit/processing/data-transformation/scale-and-reduce', 'title': 'scale and reduce unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['data_transformation']}, 'operationType': {'type': 'string', 'enum': ['scale_and_reduce']}, 'inputData': {'type': 'object', 'properties': {'scaler': {'description': 'type of scaler to be applied', 'type': 'string', 'default': 'standard_scaler', 'enum': ['standard_scaler']}, 'perFeature': {'decription': 'per-feature scaling data', 'type': 'array', 'items': {'type': 'object', 'properties': {'variance': {'description': 'variance in original training data', 'type': 'number'}, 'mean': {'description': 'mean value of the original training data', 'type': 'number'}, 'scale': {'description': 'scale multiplier for this feature/property', 'type': 'number'}, 'name': {'description': "feature/property name in 'flattened' format", 'type': 'string'}}, 'required': ['name', 'scale']}}}, 'required': ['scaler']}}}]}, {'$id': 'software-directory/ml/unit/processing/feature-selection', 'oneOf': [{'$id': 'software-directory/ml/unit/processing/feature-selection/filter-based', 'title': 'filter-based feature selection unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['feature_selection']}, 'operationType': {'type': 'string', 'enum': ['filter_based']}, 'inputData': {'type': 'object', 'properties': {'nFeatures': {'description': 'number of features to select for model training. If equal to 0, will use all available features', 'type': 'number'}, 'algorithm': {'description': 'feature selection algorithm following sklearn.feature_selection', 'type': 'string', 'enum': ['f_regression']}}, 'required': ['nFeatures', 'algorithm']}}}]}]}, {'$id': 'software-directory/ml/unit/execution', 'oneOf': [{'$id': 'software-directory/ml/unit/execution/evaluate/cross-validate', 'title': 'cross-validation unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'TODO: consider keeping executable `evaluate` and flavor `cross-validate` as before', 'type': 'object', 'properties': {'nSplits': {'description': 'number of groups to split the training dataset for cross-validation', 'type': 'number'}}, 'required': ['nSplits']}}}, {'$id': 'software-directory/ml/unit/execution/train', 'title': 'train unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'model train unit (NOTE: info about method, eg. regression/linear is taken from (sub)workflow)', 'type': 'object', 'properties': {'features': {'description': 'material features used for model fitting', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}, 'targets': {'description': 'target properties to train for', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}}, 'required': ['features', 'targets']}}}, {'$id': 'software-directory/ml/unit/execution/score', 'title': 'train score schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {}}}, {'$id': 'software-directory/ml/unit/execution/initialize', 'title': 'initialize unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'model init unit (NOTE: info about method, eg. regression/linear is taken from (sub)workflow)', 'type': 'object', 'properties': {'targets': {'description': 'target properties to predict (NOTE: must be a subset of targets for which training was done)', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}}, 'required': ['targets']}}}]}, {'$id': 'software-directory/ml/unit/processing/feature-selection', 'oneOf': [{'$id': 'software-directory/ml/unit/processing/feature-selection/filter-based', 'title': 'filter-based feature selection unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['feature_selection']}, 'operationType': {'type': 'string', 'enum': ['filter_based']}, 'inputData': {'type': 'object', 'properties': {'nFeatures': {'description': 'number of features to select for model training. If equal to 0, will use all available features', 'type': 'number'}, 'algorithm': {'description': 'feature selection algorithm following sklearn.feature_selection', 'type': 'string', 'enum': ['f_regression']}}, 'required': ['nFeatures', 'algorithm']}}}]}, {'$id': 'software-directory/ml/unit/processing/data-transformation', 'oneOf': [{'$id': 'software-directory/ml/unit/processing/data-transformation/scale-and-reduce', 'title': 'scale and reduce unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['data_transformation']}, 'operationType': {'type': 'string', 'enum': ['scale_and_reduce']}, 'inputData': {'type': 'object', 'properties': {'scaler': {'description': 'type of scaler to be applied', 'type': 'string', 'default': 'standard_scaler', 'enum': ['standard_scaler']}, 'perFeature': {'decription': 'per-feature scaling data', 'type': 'array', 'items': {'type': 'object', 'properties': {'variance': {'description': 'variance in original training data', 'type': 'number'}, 'mean': {'description': 'mean value of the original training data', 'type': 'number'}, 'scale': {'description': 'scale multiplier for this feature/property', 'type': 'number'}, 'name': {'description': "feature/property name in 'flattened' format", 'type': 'string'}}, 'required': ['name', 'scale']}}}, 'required': ['scaler']}}}]}, {'$id': 'software-directory/ml/unit/processing/data-transformation/manipulation', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'manipulation unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['data_transformation']}, 'operationType': {'type': 'string', 'enum': ['manipulation']}, 'inputData': {'type': 'object', 'properties': {'cleanMissingData': {'description': 'whether to clean missing data, eg. NaN', 'type': 'boolean', 'default': True}, 'removeDuplicateRows': {'description': 'whether to remove duplicate rows', 'type': 'boolean', 'default': True}, 'replaceNoneValuesWith': {'description': 'replace None values with a given value', 'default': 0, 'type': 'number'}}, 'required': ['cleanMissingData', 'removeDuplicateRows', 'replaceNoneValuesWith']}}}, {'$id': 'software-directory/ml/unit/processing/data-transformation/scale-and-reduce', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'scale and reduce unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['data_transformation']}, 'operationType': {'type': 'string', 'enum': ['scale_and_reduce']}, 'inputData': {'type': 'object', 'properties': {'scaler': {'description': 'type of scaler to be applied', 'type': 'string', 'default': 'standard_scaler', 'enum': ['standard_scaler']}, 'perFeature': {'decription': 'per-feature scaling data', 'type': 'array', 'items': {'type': 'object', 'properties': {'variance': {'description': 'variance in original training data', 'type': 'number'}, 'mean': {'description': 'mean value of the original training data', 'type': 'number'}, 'scale': {'description': 'scale multiplier for this feature/property', 'type': 'number'}, 'name': {'description': "feature/property name in 'flattened' format", 'type': 'string'}}, 'required': ['name', 'scale']}}}, 'required': ['scaler']}}}, {'$id': 'software-directory/ml/unit/processing/feature-selection/filter-based', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'filter-based feature selection unit schema', 'allOf': [{'$id': 'workflow/unit/processing', 'title': 'processing unit schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}], 'properties': {'type': {'enum': ['processing']}, 'operation': {'description': 'Contains information about the operation used.', 'type': 'string'}, 'operationType': {'description': 'Contains information about the specific type of the operation used.', 'type': 'string'}, 'inputData': {'description': 'unit input (type to be specified by the child units)'}}, 'required': ['operation', 'operationType', 'inputData']}], 'type': 'object', 'properties': {'operation': {'type': 'string', 'enum': ['feature_selection']}, 'operationType': {'type': 'string', 'enum': ['filter_based']}, 'inputData': {'type': 'object', 'properties': {'nFeatures': {'description': 'number of features to select for model training. If equal to 0, will use all available features', 'type': 'number'}, 'algorithm': {'description': 'feature selection algorithm following sklearn.feature_selection', 'type': 'string', 'enum': ['f_regression']}}, 'required': ['nFeatures', 'algorithm']}}}, {'$id': 'software-directory/ml/unit/execution/score', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'train score schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {}}}, {'$id': 'software-directory/ml/unit/execution/train', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'train unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'model train unit (NOTE: info about method, eg. regression/linear is taken from (sub)workflow)', 'type': 'object', 'properties': {'features': {'description': 'material features used for model fitting', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}, 'targets': {'description': 'target properties to train for', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}}, 'required': ['features', 'targets']}}}, {'$id': 'software-directory/ml/unit/execution/initialize', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'initialize unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'model init unit (NOTE: info about method, eg. regression/linear is taken from (sub)workflow)', 'type': 'object', 'properties': {'targets': {'description': 'target properties to predict (NOTE: must be a subset of targets for which training was done)', 'type': 'array', 'items': {'description': "material features (properties) in a 'flattened' format", 'type': 'string'}}}, 'required': ['targets']}}}, {'$id': 'software-directory/ml/unit/execution/evaluate/cross-validate', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'cross-validation unit schema', 'allOf': [{'$id': 'workflow/unit/execution', 'title': 'execution unit schema (base)', 'type': 'object', 'allOf': [{'$id': 'workflow/unit', 'title': 'workflow unit schema', 'type': 'object', 'allOf': [{'$id': 'in-memory-entity/named-defaultable-runtime-items', 'title': 'Named defaultable runtime items in-memory entity schema', 'allOf': [{'$id': 'in-memory-entity/base', 'title': 'Base in-memory entity schema', 'allOf': [{'description': 'entity schema version', '$id': 'system/schema-version', 'title': 'schema version', 'properties': {'schemaVersion': {'description': "entity's schema version. Used to distinct between different schemas.", 'type': 'string', 'default': '2022.8.16'}}}], 'properties': {'_id': {'description': 'entity identity', 'type': 'string'}, 'slug': {'description': 'entity slug', 'type': 'string'}, 'systemName': {'type': 'string'}, 'consistencyChecks': {'type': 'array', 'items': {'$id': 'system/consistency-check', 'title': 'consistency check', 'type': 'object', 'description': 'The output of consistency checks performed on data adhering to JSON schema, but inconsistent with scientific or logical rules, to show problems in UI.', 'properties': {'key': {'type': 'string', 'description': "Key of the property of the entity on which the consistency check is performed in Mongo dot notation, e.g. 'basis.coordinates.1'"}, 'name': {'enum': ['default', 'atomsTooClose', 'atomsOverlap'], 'description': 'Name of the consistency check that is performed, which is listed in an enum.'}, 'severity': {'enum': ['info', 'warning', 'error'], 'description': 'Severity level of the problem, which is used in UI to differentiate.'}, 'message': {'type': 'string', 'description': 'Message generated by the consistency check describing the problem.'}}, 'required': ['key', 'name', 'severity', 'message']}}}}, {'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/defaultable', 'title': 'defaultable entity schema', 'properties': {'isDefault': {'description': 'Identifies that entity is defaultable', 'type': 'boolean', 'default': False}}}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'_id': {'type': 'string'}, 'isDraft': {'type': 'boolean'}, 'type': {'description': 'type of the unit', 'type': 'string'}, 'name': {'description': 'name of the unit. e.g. pw_scf', 'type': 'string'}, 'status': {'description': 'Status of the unit.', 'type': 'string', 'enum': ['idle', 'active', 'warning', 'error', 'finished']}, 'head': {'description': 'Whether this unit is the first one to be executed.', 'type': 'boolean'}, 'flowchartId': {'description': 'Identity of the unit in the workflow. Used to trace the execution flow of the workflow.', 'type': 'string'}, 'next': {'description': "Next unit's flowchartId. If empty, the current unit is the last.", 'type': 'string'}, 'enableRender': {'description': 'Whether Rupy should attempt to use Jinja templating to add context variables into the unit', 'type': 'boolean'}, 'context': {'type': 'object'}}, 'additionalProperties': True, 'required': ['type', 'flowchartId']}, {'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'type': {'enum': ['execution']}, 'application': {'description': 'Contains information about the simulation engine/application.', '$id': 'software/application', 'title': 'application schema (base)', 'type': 'object', 'properties': {'name': {'description': 'The name of the application. e.g. espresso', 'type': 'string'}, 'shortName': {'description': 'The short name of the application. e.g. qe', 'type': 'string'}, 'summary': {'description': "Application's short description.", 'type': 'string'}, 'version': {'description': 'Application version. e.g. 5.3.5', 'type': 'string'}, 'build': {'description': 'Application build. e.g. VTST', 'type': 'string'}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}, 'isLicensed': {'description': 'Whether licensing is present', 'type': 'boolean'}}, 'additionalProperties': True, 'required': ['name']}, 'executable': {'description': 'Contains information about the simulation engine/application executable.', '$id': 'software/executable', 'title': 'executable schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable. e.g. pw.x', 'type': 'string'}, 'applicationId': {'description': '_ids of the application this executable belongs to', 'type': 'array', 'items': {'type': 'string'}}, 'hasAdvancedComputeOptions': {'description': 'Whether advanced compute options are present', 'type': 'boolean'}}, 'required': ['name', 'applicationId']}, 'flavor': {'description': 'Contains information about the simulation engine/application flavor.', '$id': 'software/flavor', 'title': 'flavor schema', 'type': 'object', 'allOf': [{'$id': 'workflow/unit/runtime/runtime-items', 'title': 'runtime items schema (pre-/post-processors, monitors, results', 'type': 'object', 'properties': {'preProcessors': {'description': 'names of the pre-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'postProcessors': {'description': 'names of the post-processors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'monitors': {'description': 'names of the monitors for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}, 'results': {'description': 'names of the results for this calculation', 'type': 'array', 'items': {'$id': 'workflow/unit/runtime/runtime-item', 'title': 'runtime item schema', 'oneOf': [{'$id': 'workflow/unit/runtime/-runtime-item-name-object', 'title': 'name result schema', 'type': 'object', 'properties': {'name': {'description': 'The name of this item. e.g. scf_accuracy', 'type': 'string'}}, 'required': ['name']}, {'$id': 'workflow/unit/runtime/-runtime-item-string', 'title': 'RuntimeItemString', 'description': 'name of runtime item in shortened notation', 'type': 'string'}]}}}}], 'properties': {'name': {'description': 'The name of the executable flavor. e.g. `pw_scf`', 'type': 'string'}, 'executableId': {'description': '_id of the executable this flavor belongs to', 'type': 'string'}, 'executableName': {'description': 'name of the executable this flavor belongs to', 'type': 'string'}, 'applicationName': {'description': 'name of the application this flavor belongs to', 'type': 'string'}, 'input': {'title': 'execution unit input schema', 'type': 'array', 'items': {'$id': 'workflow/unit/input/-inputItemId', 'title': 'execution unit input id item schema for physics-based simulation engines', 'type': 'object', 'properties': {'templateId': {'type': 'string'}, 'templateName': {'type': 'string'}, 'name': {'description': 'name of the resulting input file, if different than template name', 'type': 'string'}}, 'required': ['templateId']}}}, 'required': ['name', 'executableId']}, 'input': {'description': "unit input (type to be specified by the application's execution unit)"}}, 'required': ['input', 'application']}], 'properties': {'input': {'description': 'TODO: consider keeping executable `evaluate` and flavor `cross-validate` as before', 'type': 'object', 'properties': {'nSplits': {'description': 'number of groups to split the training dataset for cross-validation', 'type': 'number'}}, 'required': ['nSplits']}}}, {'$id': 'definitions/units', 'length': {'enum': ['km', 'm', 'pm', 'nm', 'angstrom', 'a.u.', 'bohr', 'fractional', 'crystal', 'cartesian', 'alat']}, 'energy': {'enum': ['kJ/mol', 'eV', 'J/mol', 'hartree', 'cm-1', 'rydberg', 'eV/atom']}, 'surface_energy': {'enum': ['eV/A^2']}, 'force': {'enum': ['eV/bohr', 'eV/angstrom', 'rydberg/a.u.', 'newton', 'kg*m/s^2', 'eV/a.u.']}, 'volume': {'enum': ['cm^3', 'angstrom^3']}, 'numberDensity': {'enum': ['1/angstrom^3']}, 'density': {'enum': ['g/cm^3']}, 'frequency': {'enum': ['cm-1', 'THz', 'meV']}, 'pressure': {'enum': ['kbar', 'pa']}, 'phononDOS': {'enum': ['states/cm-1', 'states/THz', 'states/meV']}, 'electronicDOS': {'enum': ['states/unitcell']}, 'magnetic': {'enum': ['uB']}, 'chargeDensity': {'enum': ['e/A']}, 'spectralIntensity': {'enum': ['(debye/angstrom)^2', 'km/mol', 'm/mol', 'a.u.']}}, {'$id': 'method/categorized-method', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'categorized method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'units': {'type': 'array', 'items': {'$id': 'method/unit-method', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}}}, 'required': ['units']}, {'$id': 'method/method-parameters', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'MethodParameters', 'allOf': [{'oneOf': [{'properties': {'basisSlug': {'enum': ['cc-pvdz', 'cc-pvtz', 'cc-pvqz']}}}, {'properties': {'basisSlug': {'enum': ['3-21G', '6-31G', '6-311G']}}}, {'properties': {'basisSlug': {'enum': ['sto-3g', 'sto-4g', 'sto-6g', 'def2-svp', 'def2-tzvp', 'def2-qzvp', 'cbs-qb3']}}}]}]}, {'$id': 'method/unit-method', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'categorized unit method', 'allOf': [{'$id': 'system/path-entity', 'title': 'path entity schema', 'allOf': [{'$id': 'system/name', 'title': 'name entity schema', 'properties': {'name': {'description': 'entity name', 'type': 'string'}}}, {'$id': 'system/path', 'title': 'path schema', 'type': 'object', 'properties': {'path': {'$id': 'core/reusable/category-path', 'title': 'category path schema', 'description': 'TODO: Use regex once schema draft version has been updated', 'type': 'string'}}}]}, {'$id': 'system/tags', 'title': 'entity tags schema', 'properties': {'tags': {'description': 'entity tags', 'type': 'array', 'items': {'type': 'string'}}}}], 'properties': {'categories': {'description': 'Used to categorize entities such as models and methods', '$id': 'core/reusable/categories', 'title': 'categories', 'type': 'object', 'properties': {'tier1': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier2': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'tier3': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'type': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}, 'subtype': {'description': 'contains either object with slugified entry or slug only as a string', '$id': 'core/primitive/slugified-entry-or-slug', 'title': 'slugified entry or slug', 'oneOf': [{'$id': 'core/primitive/slugified-entry', 'title': 'slugified entry', 'description': 'container for machine- and human-readable identifier', 'type': 'object', 'properties': {'name': {'description': 'descriptive human-readable name of entry', 'type': 'string'}, 'slug': {'description': 'machine-readable identifier', 'type': 'string'}}, 'required': ['name', 'slug']}, {'type': 'string'}]}}}, 'parameters': {'description': 'Instructive parameters defining the method', 'type': 'object'}, 'precision': {'description': 'Object showing the actual possible precision based on theory and implementation', 'type': 'object'}}}, {'$id': 'material/conventional', '$schema': 'http://json-schema.org/draft-04/schema#', 'title': 'material conventional schema', 'properties': {'conventional': {'type': 'object'}}}]))